
////////////////////////////////////////////////////////////////////////////
/////////////////////DESCRIPTION////////////////////////////////////////////
/*
SCAD FILE FOR PRODUCING CYLINDERICAL FORMS

REV-1 DESIGN STARTED 06-03-2019
REV-1 2nd set of axial holes 07-03-2020
REV=2 cams for limit switches added 27-07-2020
REV-3 acme/trapizoidal thread form added
      shaft clamps diameter or radius split added 01-12-2021
REV-4 graduations error re what section it is applied too corrected 
      (was on all sections now selected section) 12-12-2022
REV-5 fine tune parameters added to pulley generation  10-02-2023


AUTHOR JOHN MCPHEE

The program is to generate cylinerical forms such as bolts, gears, knobs, wheels, shafts
Basically lots of things you can think of that is cylinerical in form WETHER ROUND OR NOT.
The script covers a cylinder of five sections:- 
For example a bolt head could be section 1 the next section up could be a thread.
Another example could be finger screw with only section used.
its really down to your imagination.

The script builds up the five sections one at a time.
The bottom section is section one rising through to section five.

things to think about
//sort torx slight mismatch, how does it print??
//transfer parameters for all modules remove global only section reqd???
//45mm missing from screw tables
//add tollerances to nut slots??
//sect_1 text in variables shorten where possible
//external torx to define
//add square fixing holes ??
//keyways with rounded ends
//countersunk bores on axial fixing
//split definitions to sep file
//split total for 1/2 printing
//add initial rotation to axial and radial text
//trapazoidal thread diameters to simplify

*/

////////////////////////////////////////////////////////////////////////////
//START OF PARAMETER ENTRY
////////////////////////////////////////////////////////////////////////////

//% shows object and does nothing with it
//! draws that object only
//# shows that object in red


//$fa=1;
$fn=60;

use <my_gears.scad>;
use <my_pulley.scad>;
use <my_threads.scad>;

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE THE OVERALL STRUCTURE OF THE PART
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

// up to 5 body sections can be defined to exist
section_1_body_exists = 1;
section_2_body_exists = 0;
section_3_body_exists = 0;
section_4_body_exists = 0;
section_5_body_exists = 0;

//never need changed, used for logical decisions
section_1_identity =1;
section_2_identity =2;
section_3_identity =3;
section_4_identity =4;
section_5_identity =5;

//section profiles are 0= dont draw, 1=circular, 2=screw head
//3 or more equals polygon with side = number
//note other than cylinder and cuboid poligon diameterS are outside dimention
section_1_profile = 1;
section_2_profile = 1;
section_3_profile = 1;
section_4_profile = 1;
section_5_profile = 1;

//if section_x_profile is 1 then enter section_x_diameter
//a section with a thread or gear does not need a diameter
//section diameters
section_1_diameter = 0;
section_2_diameter = 0;
section_3_diameter = 0;
section_4_diameter = 0;
section_5_diameter = 0;

//if section_x_profile is 4 then enter section_x_length and section_x_width

//section heights
section_1_height = 16;
section_2_height = 0;
section_3_height = 0;
section_4_height = 0;
section_5_height = 0;

//section length if not round
section_1_length = 0;
section_2_length = 0;
section_3_length = 0;
section_4_length = 0;
section_5_length = 0;

//section width if not round
section_1_width = 0;
section_2_width = 0;
section_3_width = 0;
section_4_width = 0;
section_5_width = 0;



//base height of each section 
//used to assemble sections, no change required
section_1_base_height = 0;
section_2_base_height = section_1_base_height+section_1_height;
section_3_base_height = section_2_base_height+section_2_height;    
section_4_base_height = section_3_base_height+section_3_height;
section_5_base_height = section_4_base_height+section_4_height;
max_body_height=section_5_base_height + section_5_height + 0.1;
echo("MAX BODY HEIGHT",max_body_height);

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE THE BORE IF ONE EXISTS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//define if section 1 bore exist and its format
//bore types : 1=cylinder
//             2=shaft with flats, if no flats required make flats parameter bigger than bore parameter       
//3 or more equals bore polygon with side = number
//note other than cylinder and cuboid poligon  diameter is outside dimention

sect_1_bore_exists = 1;
sect_1_bore_type = 1;
sect_1_bore_dia = 20.2;//20.2 60 for test
sect_1_bore_height = 17;
sect_1_bore_width_of_flats =0;

//define if section 2 bore exist and its format
sect_2_bore_exists = 0;
sect_2_bore_type = 1;
sect_2_bore_dia = 8;
sect_2_bore_height = 10;
sect_2_bore_width_of_flats =0;

//define if section 3 bore exist and its format
sect_3_bore_exists = 0;
sect_3_bore_type = 1;
sect_3_bore_dia = 8;
sect_3_bore_height = 5;
sect_3_bore_width_of_flats =20;

//define if section 4 bore exist and its format
sect_4_bore_exists = 0;
sect_4_bore_type = 1;
sect_4_bore_dia = 20;
sect_4_bore_height = 40;
sect_4_bore_width_of_flats =10;

//define if section 5 bore exist and its format
sect_5_bore_exists = 0;
sect_5_bore_type = 1;
sect_5_bore_dia = 25;
sect_5_bore_height = 50;
sect_5_bore_width_of_flats =10;

// two bore radius exists, one at a section top and one at a section bottom
// define the sections and size here

// defines if and which section has a top bore radius
top_inner_radius_exists = 0;
section_with_top_inner_radius = 3;
top_inner_radius_size = 3;

//defines if and which section has a bottom bore radius
bot_inner_radius_exists = 0;
section_with_bot_inner_radius = 3;
bot_inner_radius_size = 2;

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE EXTERNAL CHAMFERS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//defines if the top of a section has chamfers
//the overall height of the section remains as declared in section_x_height
//the chamfer consists of two halves to allow for flexibility of design

sect_1_top_cham_exists = 1;
sect_1_top_cham_1_top_thickness = 1;
sect_1_top_cham_1_top_dia = 78;
sect_1_top_cham_1_bot_dia = 78;
sect_1_top_cham_2_bot_thickness =1.5;
sect_1_top_cham_2_top_dia = 78;
sect_1_top_cham_2_bot_dia = 75;

sect_2_top_cham_exists = 0;
sect_2_top_cham_1_top_thickness = 1;
sect_2_top_cham_1_top_dia = 12;
sect_2_top_cham_1_bot_dia = 8;
sect_2_top_cham_2_bot_thickness =1;
sect_2_top_cham_2_top_dia = 8;
sect_2_top_cham_2_bot_dia = 6;

sect_3_top_cham_exists = 0;
sect_3_top_cham_1_top_thickness = 3;
sect_3_top_cham_1_top_dia = 25;
sect_3_top_cham_1_bot_dia = 30;
sect_3_top_cham_2_bot_thickness =0;
sect_3_top_cham_2_top_dia = 55;
sect_3_top_cham_2_bot_dia = 60;

sect_4_top_cham_exists = 0;
sect_4_top_cham_1_top_thickness = 5;
sect_4_top_cham_1_top_dia = 50;
sect_4_top_cham_1_bot_dia = 45;
sect_4_top_cham_2_bot_thickness =5;
sect_4_top_cham_2_top_dia = 45;
sect_4_top_cham_2_bot_dia = 50;

sect_5_top_cham_exists = 0;
sect_5_top_cham_1_top_thickness = 5;
sect_5_top_cham_1_top_dia = 40;
sect_5_top_cham_1_bot_dia = 35;
sect_5_top_cham_2_bot_thickness =5;
sect_5_top_cham_2_top_dia = 35;
sect_5_top_cham_2_bot_dia = 40;

//defines if the bottom of a section has chamfers
//the overall height of the section remains as declared in section_x_height
//the chamfer consists of two halves to allow for flexibility of design

sect_1_bot_cham_exists = 1;
sect_1_bot_cham_1_top_thickness = 1.5;
sect_1_bot_cham_1_top_dia = 75;
sect_1_bot_cham_1_bot_dia = 78;
sect_1_bot_cham_2_bot_thickness = 1.0;
sect_1_bot_cham_2_top_dia = 78;
sect_1_bot_cham_2_bot_dia = 78;

sect_2_bot_cham_exists = 0;
sect_2_bot_cham_1_top_thickness = 1;
sect_2_bot_cham_1_top_dia = 6;
sect_2_bot_cham_1_bot_dia = 5;
sect_2_bot_cham_2_bot_thickness = 0;
sect_2_bot_cham_2_top_dia = 5;
sect_2_bot_cham_2_bot_dia = 6;

sect_3_bot_cham_exists = 0;
sect_3_bot_cham_1_top_thickness = 1;
sect_3_bot_cham_1_top_dia = 6;
sect_3_bot_cham_1_bot_dia = 5;
sect_3_bot_cham_2_bot_thickness = 1;
sect_3_bot_cham_2_top_dia = 5;
sect_3_bot_cham_2_bot_dia = 6;

sect_4_bot_cham_exists = 0;
sect_4_bot_cham_1_top_thickness = 5;
sect_4_bot_cham_1_top_dia = 50;
sect_4_bot_cham_1_bot_dia = 45;
sect_4_bot_cham_2_bot_thickness = 5;
sect_4_bot_cham_2_top_dia = 45;
sect_4_bot_cham_2_bot_dia = 50;

sect_5_bot_cham_exists = 0;
sect_5_bot_cham_1_top_thickness = 5;
sect_5_bot_cham_1_top_dia = 40;
sect_5_bot_cham_1_bot_dia = 35;
sect_5_bot_cham_2_bot_thickness = 5;
sect_5_bot_cham_2_top_dia = 35;
sect_5_bot_cham_2_bot_dia = 40;

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE EXTERNAL RADIUS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//defines if the top of a section has an external radius
//the overall height of the section remains as declared in section_x_height
//the radius consists of two halves to allow for more flexibility

sect_1_top_radius_exists = 0;
sect_1_top_radius_1_top_thickness = 1.5;
sect_1_top_radius_1_major_dia = 10;
sect_1_top_radius_2_bot_thickness = 1.5;
sect_1_top_radius_2_major_dia = 10;

sect_2_top_radius_exists = 0;
sect_2_top_radius_1_top_thickness = 5;
sect_2_top_radius_1_major_dia = 75;
sect_2_top_radius_2_bot_thickness = 5;
sect_2_top_radius_2_major_dia = 75;

sect_3_top_radius_exists = 0;
sect_3_top_radius_1_top_thickness = 1.5;
sect_3_top_radius_1_major_dia = 16;
sect_3_top_radius_2_bot_thickness = 0;
sect_3_top_radius_2_major_dia = 16;

sect_4_top_radius_exists = 0;
sect_4_top_radius_1_top_thickness = 5;
sect_4_top_radius_1_major_dia = 55;
sect_4_top_radius_2_bot_thickness = 5;
sect_4_top_radius_2_major_dia = 55;

sect_5_top_radius_exists = 0;
sect_5_top_radius_1_top_thickness = 5;
sect_5_top_radius_1_major_dia = 45;
sect_5_top_radius_2_bot_thickness = 5;
sect_5_top_radius_2_major_dia = 45;

//defines if the bottom of a section has an external radius
//the overall height of the section remains as declared in section_x_height
//the radius consists of two halves to allow for more flexibility

sect_1_bot_radius_exists = 0;
sect_1_bot_radius_1_top_thickness = 0;
sect_1_bot_radius_1_major_dia = 85;
sect_1_bot_radius_2_bot_thickness = 2;
sect_1_bot_radius_2_major_dia = 140;

sect_2_bot_radius_exists = 0;
sect_2_bot_radius_1_top_thickness = 5;
sect_2_bot_radius_1_major_dia = 75;
sect_2_bot_radius_2_bot_thickness = 5;
sect_2_bot_radius_2_major_dia = 75;

sect_3_bot_radius_exists = 0;
sect_3_bot_radius_1_top_thickness = 0;
sect_3_bot_radius_1_major_dia = 16;
sect_3_bot_radius_2_bot_thickness = 1;
sect_3_bot_radius_2_major_dia = 16;

sect_4_bot_radius_exists = 0;
sect_4_bot_radius_1_top_thickness = 5;
sect_4_bot_radius_1_major_dia = 55;
sect_4_bot_radius_2_bot_thickness = 5;
sect_4_bot_radius_2_major_dia = 55;

sect_5_bot_radius_exists = 0;
sect_5_bot_radius_1_top_thickness = 5;
sect_5_bot_radius_1_major_dia = 45;
sect_5_bot_radius_2_bot_thickness = 5;
sect_5_bot_radius_2_major_dia = 45;

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE SPINDLE CLAMPS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//defines if a section has spindle clamping features
//two spindle clamps can exists, one at the top of the design and one at the bottom

// defines if and which section has a spindle clamp
//bottom spindle clamp
bottom_spindle_clamp_exists = 0;
section_with_bottom_spindle_clamp =1;
bot_spin_clamp_split_type = 0;      // 0= diameter split 1= radius split
bot_spin_clamp_split_width = 1;
bot_spin_clamp_fixing_height = 4;
bot_spin_clamp_fixing_radius = 9;
bot_spin_clamp_head_offset = 5;
bot_spin_clamp_nut_offset = 5;
bot_spin_clamp_rotate = 0;
bot_spin_clamp_screw_bore_size = 3; // by std metric thread size
bot_spin_clamp_screw_head_size = 3; // by std metric counterbore size
bot_spin_clamp_hex_nut_size = 3;    // by std metric nut size

//top spindle clamp
top_spindle_clamp_exists = 0;
section_with_top_spindle_clamp =2;
top_spin_clamp_split_type = 0;      // 0= diameter split 1= radius split
top_spin_clamp_split_width = 4;
top_spin_clamp_fixing_height = 15;
top_spin_clamp_fixing_radius = 25;
top_spin_clamp_head_offset = 5;
top_spin_clamp_nut_offset = 5;
top_spin_clamp_rotate = 90;         
top_spin_clamp_screw_bore_size = 5; // by std metric thread size
top_spin_clamp_screw_head_size = 5; // by std metric counterbore size
top_spin_clamp_hex_nut_size = 5;    // by std metric nut size

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE TOP AND BOTTOM RECESSES AND INTERNAL BOSS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//define section lower recess
bottom_face_recess_exists = 0;
section_with_bottom_recess =1;
bottom_face_recess_dia = 40;
bottom_face_recess_depth = 5;

//define section upper recess
top_face_recess_exists = 0;
section_with_top_recess =1;
top_face_recess_dia = 35;
top_face_recess_depth = 3.5;

//define section bottom boss
lower_boss_exists = 0;
section_with_lower_boss =1;
lower_boss_top_dia = 30;
lower_boss_bot_dia = 25;
lower_boss_height = 5;

//define section upper boss
upper_boss_exists = 0;
section_with_upper_boss =1;
upper_boss_bot_dia = 5;
upper_boss_top_dia = 5;
upper_boss_height = 1;

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE KEYWAYS AND SPLINES
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//define internal keyway, cuts through specified body sections
//uses bore diameter of specified section

internal_keyway_exists = 1;
section_with_internal_key = 1;
number_of_int_keyway = 1;
internal_key_offset =0; //angular position
internal_key_width = 4.1;
internal_key_height = 3.0; //depth of keyway

//define external keyway, cuts through specific body section
//udes external diameter of section

external_keyway_exists = 0;
section_with_external_key = 1;
number_of_ext_keyway = 1;
external_key_offset =10;
external_key_width = 5;
external_key_height = 3;                

////////////////////////
//define internal spline
//cuts through specified body section for specified length

internal_spline_exists = 0;
section_with_internal_spline = 2;
// spline type
// 1=negative (cut away) 2=positive (addid on)
internal_spline_type = 2;

number_of_int_splines = 6;
internal_spline_offset = 0;
internal_spline_width = 4;
internal_spline_depth = 4;

/////////////////////////
//define external spline, cuts through specific body section
//cuts through specified body section for specified length

external_spline_exists = 0;
section_with_external_spline = 2;

// spline type
// 1=negative (cut away) 2=positive (addid on)
external_spline_type = 1;

number_of_ext_splines = 1;
external_spline_offset = 10;
external_spline_width = 5;
external_spline_depth = 3;
external_spline_length = 30;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE FIXING METHODS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////


//radial grub screw fixing, optional nut recess and counterbore
//added to selected section

radial_fixing_exists = 0;
section_with_radial_fixing = 2;

radial_fixing_qty = 1;
radial_fixing_centre_height = 5;
radial_fixing_diameter = 3;
//radial_fixing_hole_length = (section_1_diameter)/2;
radial_fixing_hole_offset = 0 ;

//radial counterbore for screw head
radial_cbore_exists =0;
radial_cbore_depth =10;

//radial nut housing slot for screw
radial_fixing_nut_exists = 0;
radial_fixing_nut_from_centre = 8;


//fixing is axial throgh body for metric screw size
//axially fixing generally on PCD circle
//square option generates 4 holes PCD dimention used for square value

//axial fixing 1 parameters
axial_fixing_1_exists = 0;
section_with_axial_fixing_1 = 1;
axial_fixing_1_pcd_is_square = 0;
axial_fixing_1_pcd = 80;
axial_fixing_1_qty = 4;
axial_metric_thread_1_size = 3;
axial_fixing_1_depth = 15;
axial_fixing_1_offset = 45;

//axial fixing 1 counterbore for screw head
axial_cbore_1_exists =0;
axial_cbore_1_depth =6;

//axial fixing 2 parameters
axial_fixing_2_exists = 0;
section_with_axial_fixing_2 = 3;
axial_fixing_2_pcd_is_square = 0;
axial_fixing_2_pcd = 40;
axial_fixing_2_qty = 3;
axial_metric_thread_2_size = 5;
axial_fixing_2_depth = 15;
axial_fixing_2_offset = 0;

//axial fixing 2 counterbore for screw head
axial_cbore_2_exists =0;
axial_cbore_2_depth =6;
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE RADIAL OR AXIAL GRADUATION MARKS
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

/////////////////////radial graduation details///////////////////////
/////////////////////////////////////////////////////////////////////


radial_graduations_exist = 0;
section_with_radial_graduations = 2;

//scale type recessed or protruding
// 1=recessed
// 2=protruding

scale_type=1;

// number of major ticks
number_major_ticks = 10;
//divisions between major ticks  for medium ticks
medium_ticks_between = 2;
//divisions between major ticks  for minor ticks
minor_ticks_between = 10;


//scale distance from outside edge
major_dist_from_edge = 2;
medium_dist_from_edge = 4;
minor_dist_from_edge = 6;

//length of scale
major_scale_length = 8.0;
medium_scale_length = 5.0;
minor_scale_length = 3.0;


//height or depth of scale
major_scale_depth = 6.0;
medium_scale_depth = 4.0;
minor_scale_depth = 2.0;

depth_of_scale =1; // [0:0.1:3]

//width of scale
major_scale_width = 1.5;
medium_scale_width = 1.0;
minor_scale_width = 0.5;

/////////////////////axial graduation details////////////////////////
/////////////////////////////////////////////////////////////////////

axial_graduations_exist = 0;
section_with_axial_graduations = 2;
// number of major ticks
number_major_ticks_ax = 2;
//divisions between major ticks  for medium ticks
medium_ticks_between_ax = 2;
//divisions between major ticks  for minor ticks
minor_ticks_between_ax = 10;


//scale type recessed or protruding
// 1=recessed
// 2=protruding

scale_type_ax=1;

//scale distance from outside edge
major_dist_from_top = 2;
medium_dist_from_top = 2;
minor_dist_from_top = 2;

//length of scale
major_scale_length_ax = 8.0;
medium_scale_length_ax = 6.0;
minor_scale_length_ax = 4.0;


//height or depth of scale
major_scale_depth_ax = 2.0;
medium_scale_depth_ax = 1.5;
minor_scale_depth_ax = 1.0;

depth_of_scale_ax =1; // [0:0.1:3]

//width of scale
major_scale_width_ax = 1.0;
medium_scale_width_ax = 0.75;
minor_scale_width_ax = 0.75;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE RADIAL OR AXIAL TEXT ON BODY
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

/////////////////////axial text on body side//////////////////////////
//////////////////////////////////////////////////////////////////////

axial_text_exist = 0;
section_with_axial_text = 1;

//text type recessed or protruding
// 1=protruding
// 2=recessed

axial_text_type=2;

// define how far the characters protrude / indent from the surface
axial_text_height = 2;
// this is a trim figure to stop characters / strings having a gap from the surface
//it pulls the text back towards the centre of object
axial_text_height_trim = 1;

//can be individual characters or strings of characters
axial_text_to_print=["A","B","C","D","E","F","G","H","I","J","K","L","M",
                    "N","O","P","Q","R","S","T","U","V","W","X","Y","Z",
                    "0","1","2","3","4","5","6","7","8","9"];

// number of letters around 
axial_number_of_strings = 26;

//angle within where the text will be printed
axial_text_within_angle = 360;
axial_text_start_rotation = 6;

//text direction makes the characters / strings go clockwise or anti-clockwise
//  1=counterclockwise
// -1 = clockwise 
axial_text_direction = 1;

// define the text center height distance from the base of the object
axial_distance_from_base = 5.0;

// define the height of the characters
axial_character_height = 10;

//define the orientation of the characters / strings 
axial_text_rotation_angle = 90;

//define font type
axial_text_font = "DejaVu Sans      Mono:style=Bold";


/////////////////////radial text on body top//////////////////////////
//////////////////////////////////////////////////////////////////////

radial_text_exist = 0;
section_with_radial_text = 1;

//text type recessed or protruding
// 1=protruding
// 2=recessed

radial_text_type=1;

// define how far the characters protrude / indent from the surface
radial_text_height = 3;

//can be individual characters or strings of characters
radial_text_to_print=["a","b","c","d","e","f","g","h","i","j",
                        "k","l","m","n","o","p","q","r","s","t"
                        ];

// number of letters around 
radial_number_of_strings = 10;

//angle within where the text will be printed
radial_text_within_angle = 360;
radial_text_start_rotation = 6;

//text direction makes the characters / strings go clockwise or anti-clockwise
//  1=counterclockwise
// -1 = clockwise 
radial_text_direction = 1;

// define the text distance from the centre of the object
radial_distance_from_centre = 60;

// define the height of the characters
radial_character_height = 13;

//define the orientation of the characters / strings 
radial_text_rotation_angle = 90;

//define font type
radial_text_font ="Wingdings:style=Regular"; 

//"DejaVu Sans      Mono:style=Bold";
//"Wingdings:style=Regular";
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE INTERNAL DIAMETER GROOVES
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

/////////////////////internal circular groove////////////////////////
/////////////////////////////////////////////////////////////////////

internal_circ_groove_exist = 0;
section_with_circular_groove = 1;
number_of_circ_grooves = 3;
distance_between_circ_groove = 10;

//define the dimentions of the groove

//define the diameter of the circular groove
circ_groove_diameter=5;

// height from base
circ_groove_from_base=5;

//position the groove from the centre
circ_groove_from_centre = 35.0;


/////////////////////internal square groove//////////////////////////
////////////////////////////////////////////////////////////////////

internal_rect_groove_exist = 0;
section_with_rectangular_groove = 2;
number_of_rect_grooves = 10;
distance_between_rect_groove = 20;

//define the dimentions of the groove

rect_groove_width=10;
rect_groove_depth=8;

// start height from base
rect_groove_from_base=10;

//position the groove from the centre
rect_groove_from_centre = 60.0;





/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//WEIGHT SAVING UNDERCUTS AND HOLES
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

weight_saving_exists = 0;
section_with_weight_saving = 2;

// outer diameter of saving area
circle_outer_limit =70;
// inner diameter of saving area
circle_inner_limit =40;
// ratio of weight saving on holes, generally between 0.1 and 4.0
weigh_saving_ratio = 0.6;
//cut out area thickness reduction in percentage
cut_area_reduction = 20;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
//DEFINE KNURLING ON OUTSIDE OF BODY
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

knurling_exists = 0;
section_with_knurling = 2;

// total number of knurl starts
number_knurl_starts = 40;
// total angle from top to bottom
total_knurl_angle = 72;
//depth of knurl into surface
knurling_depth =2;
// width of knurling
knurling_width = 35;
// centre height from base of knurling
knurling_base_height = 5;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
// DEFINE DRIVE TYPE OR NUT HOLE
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////


/////////////////////base driver type///////////////////////////////
//////////////////////////////////////////////////////////////////////

//if section_x_profile is 2 then select a screw head type
//these are based on standard metric bolts
//1=hex head bolt, 2=cap head bolt, 3=dome head screw, 4=countersunk screw

screw_head_type = 2;
screw_metric_size = 20;

// puts a head drive type in the base of a body section
// base driver type  0=allen key 1=posidrive 2=screwdriver slot 3=triangle
//4=square 5=torx 6=hole for metric nut

base_drive_exists = 0;
section_with_base_drive = 1;

base_driver_type = 0;

// allen key size across the flats and depth in MM
allen_key_bolt_size = 6;
allen_key_depth = 5;

// posidrive dimentions
pozi_slot_width=6;
pozi_slot_length=10;
pozi_slot_depth=15;

// screwdriver slot dimentions
number_of_screw_slots = 4;
screw_slot_width=1;
screw_slot_length=40;
screw_slot_depth=3;

// triangle key dimentions points diameter and depth

triangle_slot_points_dia = 25;
triangle_slot_depth = 10;

// square key dimentions points diameter and depth

square_slot_width = 25;
square_slot_depth = 10;

// torx drive dimentions
torx_size=100;
torx_depth=12;

// metric nut size and depth
nut_thread_size = 3;
nut_housing_depth = 5;

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
// DEFINE GEARS ON THE BODY
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

// two gears are catered for, can be added to sections 1-5
// gear types are :-
// 1=spur, 2=spur bevel, 3=spur ring,
// 4=herringbone spur, 5=herringbone spur bevel, 6=herringbone spur ring,
// 7 worm gear

// gear 1 definition
gear_1_exists = 0;
section_with_gear_1 = 1;
gear_1_type = 1;

// common parameters for gear 1
g1_modulus=1.5;
g1_number_of_teeth=38;
g1_missing_teeth =-1; //number -1 ,?????????
g1_gear_thickness=6;
g1_bore_size=6; 
g1_bore_width_of_flats=4; 
g1_pressure_angle=20; 
g1_helix_angle=30;
g1_optimized=0;
// additional parameters for bevel gear 1
g1_bevel_angle = 45;
// additional parameters for ring gear 1
g1_ring_edge_width = 5;
// additional parameters for worm
g1_number_of_starts=2;
g1_worm_length = 20;
g1_gradient_angle=10;
g1_gear_base_height = 4;
g1_unsplit_worm_print = 1;


// gear 2 definition
gear_2_exists = 0;
section_with_gear_2 = 2;
gear_2_type = 1;

// common parameters for gear 1
g2_modulus=2;
g2_number_of_teeth=15;
g2_missing_teeth =0; 
g2_gear_thickness=15;
g2_bore_size=5; 
g2_bore_width_of_flats=2; 
g2_pressure_angle=20; 
g2_helix_angle=20;
g2_optimized=0;
// additional parameters for bevel gear 1
g2_bevel_angle = 45;
// additional parameters for ring gear 1
g2_ring_edge_width = 5;
// additional parameters for worm
g2_number_of_starts=2;
g2_worm_length = 20;
g2_gradient_angle=10;
g2_gear_base_height = 4;
g2_unsplit_worm_print = 1;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
// DEFINE A PULLEY ON THE BODY
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////


// Pulley Profiles=  1:MXL, 2:40DP, 3:XL, 4:H, 5:T2.5, 6:T5, 7:T10, 8:AT5, 9:HTD_3mm, 10:HTD_5mm, 11:HTD_8mm, 12:GT2_2mm, 13:GT2_3mm, 14:GT2_5mm

//pulley 1
pulley_1_exists = 1;
section_with_pulley_1 = 1;
pulley_1_profile = 6;
pulley_1_number_of_teeth = 48;
pulley_1_tooth_width = 11;
pulley_1_base_height = 2.5;
pulley_1_additional_tooth_width = 0.1; //fine tune to fit belt
pulley_1_additional_tooth_depth = 0.2; //fine tune to fit belt
pulley_1_diameter_reduction = 0.9; //fine tune to fit belt

// on a 48T belt best results of tuning :- 0.1 , 0.2 , 0.9

//pulley 2
pulley_2_exists = 0;
section_with_pulley_2 = 2;
pulley_2_profile = 6;
pulley_2_number_of_teeth = 26;
pulley_2_tooth_width = 10;
pulley_2_base_height = 5;
pulley_2_additional_tooth_width = 0.2;
pulley_2_additional_tooth_depth = 0.2;
pulley_2_diameter_reduction = 0.9;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
// DEFINE A THREAD ON THE BODY
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

//type 0=thread 1=hole
//if pitch=0 std metric pitches used otherwise entered size is used
//base_height is thread bottom height from base of section
//for thread size above 64 pitch must be entered manually

//thread 1
//common variables
thread_1_exists = 0;
section_with_thread_1 = 1;
thread_1_type = 1;              //0=external 1=internal
thread_1_form = 1;              //0=metric  1=trapazoidal
thread_1_length = 14;           //length of thread
thread_1_pitch=2.0;             //0 defaults to std metric course pitch
thread_1_tooth_angle=30;        //0 defaults to 30 for std metric thread TR=30 acme=29
thread_1_base_height =0;        // height thread is raised from base

//metric thread variable
thread_1_metric_size = 8;      //outside diameter of thread

//trapazoidal only parameters
thread_1_pitch_radius = 3;    //pitch radius is radial distance from shaft center to mid-profile
thread_1_clearance = 0.1;	    // radial clearance, normalized to thread height
thread_1_backlash= 0.1;			// axial clearance, normalized to pitch
thread_1_countersunk =0;        //0=not countersunk 1=countersunk used on trapazoidal only
thread_1_steps_per_turn = 50;   //resolution of thread used on trapazoidal only
thread_1_RH = 1;                //0=LH thread 1=RH thread

//thread 2
//common variables
thread_2_exists = 0;
section_with_thread_2 = 3;
thread_2_type = 0;              //0=external 1=internal
thread_2_form = 0;              //0=metric  1=trapazoidal
thread_2_length = 12;           //length of thread
thread_2_pitch=1.5;             //0 defaults to std metric course pitch
thread_2_tooth_angle=30;        //0 defaults to 30 for std metric thread TR=30 acme=29
thread_2_base_height =0;        // height thread is raised from base

//metric thread variable
thread_2_metric_size = 15;      //outside diameter of thread

//trapazoidal only parameters
thread_2_pitch_radius = 3.2;    //pitch radius is radial distance from shaft center to mid-profile
thread_2_clearance = 0.1;	    // radial clearance, normalized to thread height
thread_2_backlash= 0.1;			// axial clearance, normalized to pitch
thread_2_countersunk =0;        //0=not countersunk 1=countersunk used on trapazoidal only
thread_2_steps_per_turn = 50;   //resolution of thread used on trapazoidal only
thread_2_RH = 1;                //0=LH thread 1=RH thread

/////////////////////////////////////////////////////////
//axial holes, these cut through all body sections
/////////////////////////////////////////////////////////
//define holes
/*

[0]  = [ size of hole in mm: number ]
[1]  = [ length of hole:number ]
[2]  = [ hole exists: 0=no 1= exists ]
[3]  = [ hole size relates to metric screw: 0=actual 1=thread clearance ]

[4]  = [ rectangular array of holes exist: 0-no 1=exists ]
[5]  = [ x axis qty of holes: number ]
[6]  = [ x axis pitch of holes: number ]
[7]  = [ y axis qty of holes: number ]
[8]  = [ y axis pitch of holes: number ]
[9]  = [ rectangular array outside holes only: 0= all holes 1=outside only ]

[10] = [ circular array of holes exist: 0=no 1=exists ]
[11] = [ qty of holes in circular array: number ]
[12] = [ P.C.D. of circular array of holes: number ]

[13] = [ recess exists on hole: 0=no 1=exists ]
[14] = [ recess type: 0=countersink 1=counterbore ]
[15] = [ counterbore depth: number ]
[16] = [ counterbore height offset: number ]

[17] = [ nut retainer hole exists on hole: 0=no 1=exists ]
[18] = [ nut retainer depth: number ]
[19] = [ nut retainer height offset: number ]

[20] = [ thread on hole exists: 0=no 1=exists ]
[21] = [ pitch of thread: number ]
[22] = [ depth of thread: number ]
[23] = [ angle of thread: number ]
[24] = [ thread height offset: number ]

[25] = [ translation in x direction: number ]
[26] = [ translation in y direction: number ]
[27] = [ translation in z direction: number ]
[28] = [ rotation in x plane: number ]
[29] = [ rotation in y plane: number ]
[30] = [ rotation in z plane: number ]

*/
//define full length holes,holes_from_array decodes holes
full_holes_exist = 0;
full_first_hole = 0;
full_last_hole = 0;

hole =
[ //start array

// hole       rect            circ     recess     nut      thread         position
[0,0,0,0,   0,0,0,0,0,0,    0,0,0,  0,0,0,0,   0,0,0,   0,0,0,0,0, 0,0,0,0,0,0], //hole 0
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 1
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 2
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 3
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 4
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 5
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 6
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 7
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 8
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 9
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 10
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 11
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 12
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 13
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 14
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 15
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 16
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 17
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 18
[0,0,0,0,     0,0,0,0,0,0,    0,0,0,   0,0,0,0,   0,0,0,   0,0,0,0,0,     0,0,0,0,0,0], //hole 19

]; //end array , add more if required

/////////////////////////////////////////////////////////////////////
//axial slotted holes, this shapes cuts through all body sections
////////////////////////////////////////////////////////////////////

slotted_holes_exist = 0;

// total number of slots
number_of_slots = 6;
// total angle for number of slots
total_slots_angle = 360;
//initial angle offset
initial_slot_offset =0;
// angle covered by each slot
individual_slot_angle = 25;
// diameter of each slot
slot_diameter =3;
// slot centre distance from body centre
slot_distance_from_centre = 5;

/////////////////////////////////////////////////////////////////////
//axial spoke holes, this shapes cuts through all body sections
////////////////////////////////////////////////////////////////////
spoke_holes_exist = 0;
// total number of spokes
number_of_spoke = 8;
// total angle for number of spokes
total_spoke_angle = 360;
//initial angle offset
initial_spoke_offset =0;
// angle covered by each spoke
individual_spoke_angle = 3;
// radius of inner spoke corners
spoke_inner_corner_radius = 0.5;
// radius of inner spoke corners
spoke_outer_corner_radius =3;
// spoke inner curve distance from body centre
spoke_inner_distance_from_centre = 10;
// spoke outer curve distance from body centre
spoke_outer_distance_from_centre = 22;


/////////////////////////////////////////////////////////////////////
//axial solid spokes, these are spokes like bicycle wheels
////////////////////////////////////////////////////////////////////

spoke_bars_exist = 0;
//section with spoke bars
section_with_spoke_bar = 1;
// total number of spokes
number_of_spoke_bars = 6;
// total angle for number of spokes
total_spoke_bar_angle = 360;
//initial angle offset
initial_spoke_bar_offset =0;
//shape of spoke, 0 = circular 1 =rectangular
spoke_bar_shape = 1;
//length of spoke bar
spoke_bar_length = 20;
//diameter of spoke bar if round
spoke_bar_dia = 2;
//width of spoke bar if rectangular
spoke_bar_width = 0.8;
//height of spoke bar if rectangular
spoke_bar_height = 1.5;
//spoke_bar_distance from centre
spoke_bar_dist_from_centre = 2;
//spoke bar tilt up
spoke_bar_tilt_up_angle = 30;
//z height of spoke bar
spoke_bar_z_base_pos = 1;

/////////////////////////////////////////////////////////////////////
//generate limit switch cams on a diameter
/////////////////////////////////////////////////////////////////////

//cam set 1
switch_cams_1_exist = 0;
section_with_sw_cams_1 = 2;
switch_cams_1_type = 0; // 0=negative 1=positive
switch_cams_1_qty = 1;
switch_cams_1_spacing = 0;
switch_cams_1_width = 6;
switch_cams_1_height = 5;
switch_cams_1_major_dia = 32;
switch_cams_1_neg_cam_dia = 25;
switch_cams_1_rot_offset = 0;
switch_cams_1_base_height = 3;

//cam set 2
switch_cams_2_exist = 0;
section_with_sw_cams_2 = 2;
switch_cams_2_type = 0; // 0=negative 1=positive
switch_cams_2_qty = 1;
switch_cams_2_spacing = 0;
switch_cams_2_width = 6;
switch_cams_2_height = 5;
switch_cams_2_major_dia = 32;
switch_cams_2_neg_cam_dia = 25;
switch_cams_2_rot_offset = 120;
switch_cams_2_base_height = 10;

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//END OF PARAMETER ENTRY SECTION DONT FIDDLE BELOW UNLESS YOUR SURE !!!!!!!!
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

//start the build
round_stage_3_build();

////////////////////////////////////////////////////////////////////////////
//BUILD UP THE BODY IN 5 SECTIONS
////////////////////////////////////////////////////////////////////////////

module  round_stage_1_build()
{
    difference()
    {
        union()
        {
            //add section 1 of the build
           
            if (section_1_body_exists == 1) 
            {
                build_section(  section_1_identity,
                                section_1_body_exists,
                                section_1_base_height,
                                section_1_profile,
                                section_1_diameter,
                                section_1_height,
                                section_1_length,
                                section_1_width, 
                                sect_1_bore_exists,
                                sect_1_bore_type,
                                sect_1_bore_dia,
                                sect_1_bore_height,
                                sect_1_bore_width_of_flats,
                                sect_1_top_radius_exists,
                                sect_1_top_radius_1_top_thickness,
                                sect_1_top_radius_1_major_dia,
                                sect_1_top_radius_2_bot_thickness,
                                sect_1_top_radius_2_major_dia,
                                sect_1_bot_radius_exists,
                                sect_1_bot_radius_1_top_thickness,
                                sect_1_bot_radius_1_major_dia,
                                sect_1_bot_radius_2_bot_thickness,
                                sect_1_bot_radius_2_major_dia,
                                bot_inner_radius_exists,
                                section_with_bot_inner_radius,
                                bot_inner_radius_size,
                                sect_1_top_cham_exists,
                                sect_1_top_cham_1_top_thickness,
                                sect_1_top_cham_1_top_dia,
                                sect_1_top_cham_1_bot_dia,
                                sect_1_top_cham_2_bot_thickness,
                                sect_1_top_cham_2_top_dia,
                                sect_1_top_cham_2_bot_dia,
                                sect_1_bot_cham_exists,
                                sect_1_bot_cham_1_top_thickness,
                                sect_1_bot_cham_1_top_dia,
                                sect_1_bot_cham_1_bot_dia,
                                sect_1_bot_cham_2_bot_thickness,
                                sect_1_bot_cham_2_top_dia,
                                sect_1_bot_cham_2_bot_dia
                                ); 
                            }
            else
            {
                echo("SECTION 1 DOES NOT EXIST"); 
            }
            
            //add section 2 of the build
           
            if (section_2_body_exists == 1) 
            {
                build_section(  section_2_identity,
                                section_2_body_exists,
                                section_2_base_height,
                                section_2_profile,
                                section_2_diameter,
                                section_2_height,
                                section_2_length,
                                section_2_width, 
                                sect_2_bore_exists,
                                sect_2_bore_type,
                                sect_2_bore_dia,
                                sect_2_bore_height,
                                sect_2_bore_width_of_flats,
                                sect_2_top_radius_exists,
                                sect_2_top_radius_1_top_thickness,
                                sect_2_top_radius_1_major_dia,
                                sect_2_top_radius_2_bot_thickness,
                                sect_2_top_radius_2_major_dia,
                                sect_2_bot_radius_exists,
                                sect_2_bot_radius_1_top_thickness,
                                sect_2_bot_radius_1_major_dia,
                                sect_2_bot_radius_2_bot_thickness,
                                sect_2_bot_radius_2_major_dia,
                                bot_inner_radius_exists,
                                section_with_bot_inner_radius,
                                bot_inner_radius_size,
                                sect_2_top_cham_exists,
                                sect_2_top_cham_1_top_thickness,
                                sect_2_top_cham_1_top_dia,
                                sect_2_top_cham_1_bot_dia,
                                sect_2_top_cham_2_bot_thickness,
                                sect_2_top_cham_2_top_dia,
                                sect_2_top_cham_2_bot_dia,
                                sect_2_bot_cham_exists,
                                sect_2_bot_cham_1_top_thickness,
                                sect_2_bot_cham_1_top_dia,
                                sect_2_bot_cham_1_bot_dia,
                                sect_2_bot_cham_2_bot_thickness,
                                sect_2_bot_cham_2_top_dia,
                                sect_2_bot_cham_2_bot_dia
                                );
            }    
            else
            {
                echo("SECTION 2 DOES NOT EXIST"); 
            }
            
            //add section 3 of the build
           
           if (section_3_body_exists == 1) 
           {
                build_section(  section_3_identity,
                                section_3_body_exists,
                                section_3_base_height,
                                section_3_profile,
                                section_3_diameter,
                                section_3_height,
                                section_3_length,
                                section_3_width, 
                                sect_3_bore_exists,
                                sect_3_bore_type,
                                sect_3_bore_dia,
                                sect_3_bore_height,
                                sect_3_bore_width_of_flats,
                                sect_3_top_radius_exists,
                                sect_3_top_radius_1_top_thickness,
                                sect_3_top_radius_1_major_dia,
                                sect_3_top_radius_2_bot_thickness,
                                sect_3_top_radius_2_major_dia,
                                sect_3_bot_radius_exists,
                                sect_3_bot_radius_1_top_thickness,
                                sect_3_bot_radius_1_major_dia,
                                sect_3_bot_radius_2_bot_thickness,
                                sect_3_bot_radius_2_major_dia,
                                bot_inner_radius_exists,
                                section_with_bot_inner_radius,
                                bot_inner_radius_size,
                                sect_3_top_cham_exists,
                                sect_3_top_cham_1_top_thickness,
                                sect_3_top_cham_1_top_dia,
                                sect_3_top_cham_1_bot_dia,
                                sect_3_top_cham_2_bot_thickness,
                                sect_3_top_cham_2_top_dia,
                                sect_3_top_cham_2_bot_dia,
                                sect_3_bot_cham_exists,
                                sect_3_bot_cham_1_top_thickness,
                                sect_3_bot_cham_1_top_dia,
                                sect_3_bot_cham_1_bot_dia,
                                sect_3_bot_cham_2_bot_thickness,
                                sect_3_bot_cham_2_top_dia,
                                sect_3_bot_cham_2_bot_dia
                                );     
            }
            else
            {
                echo("SECTION 3 DOES NOT EXIST"); 
            }
            
            //add section 4 of the build
           
           if (section_4_body_exists == 1) 
           {
                build_section(  section_4_identity,
                                section_4_body_exists,
                                section_4_base_height,
                                section_4_profile,
                                section_4_diameter,
                                section_4_height,
                                section_4_length,
                                section_4_width, 
                                sect_4_bore_exists,
                                sect_4_bore_type,
                                sect_4_bore_dia,
                                sect_4_bore_height,
                                sect_4_bore_width_of_flats,
                                sect_4_top_radius_exists,
                                sect_4_top_radius_1_top_thickness,
                                sect_4_top_radius_1_major_dia,
                                sect_4_top_radius_2_bot_thickness,
                                sect_4_top_radius_2_major_dia,
                                sect_4_bot_radius_exists,
                                sect_4_bot_radius_1_top_thickness,
                                sect_4_bot_radius_1_major_dia,
                                sect_4_bot_radius_2_bot_thickness,
                                sect_4_bot_radius_2_major_dia,
                                bot_inner_radius_exists,
                                section_with_bot_inner_radius,
                                bot_inner_radius_size,
                                sect_4_top_cham_exists,
                                sect_4_top_cham_1_top_thickness,
                                sect_4_top_cham_1_top_dia,
                                sect_4_top_cham_1_bot_dia,
                                sect_4_top_cham_2_bot_thickness,
                                sect_4_top_cham_2_top_dia,
                                sect_4_top_cham_2_bot_dia,
                                sect_4_bot_cham_exists,
                                sect_4_bot_cham_1_top_thickness,
                                sect_4_bot_cham_1_top_dia,
                                sect_4_bot_cham_1_bot_dia,
                                sect_4_bot_cham_2_bot_thickness,
                                sect_4_bot_cham_2_top_dia,
                                sect_4_bot_cham_2_bot_dia
                                );    
            }
            else
            {
                echo("SECTION 4 DOES NOT EXIST"); 
            }
  
            //add section 5 of the build 
           
                
            if (section_5_body_exists == 1) 
            {
                build_section(  section_5_identity,
                                section_5_body_exists,
                                section_5_base_height,
                                section_5_profile,
                                section_5_diameter,
                                section_5_height,
                                section_5_length,
                                section_5_width, 
                                sect_5_bore_exists,
                                sect_5_bore_type,
                                sect_5_bore_dia,
                                sect_5_bore_height,
                                sect_5_bore_width_of_flats,
                                sect_5_top_radius_exists,
                                sect_5_top_radius_1_top_thickness,
                                sect_5_top_radius_1_major_dia,
                                sect_5_top_radius_2_bot_thickness,
                                sect_5_top_radius_2_major_dia,
                                sect_5_bot_radius_exists,
                                sect_5_bot_radius_1_top_thickness,
                                sect_5_bot_radius_1_major_dia,
                                sect_5_bot_radius_2_bot_thickness,
                                sect_5_bot_radius_2_major_dia,
                                bot_inner_radius_exists,
                                section_with_bot_inner_radius,
                                bot_inner_radius_size,
                                sect_5_top_cham_exists,
                                sect_5_top_cham_1_top_thickness,
                                sect_5_top_cham_1_top_dia,
                                sect_5_top_cham_1_bot_dia,
                                sect_5_top_cham_2_bot_thickness,
                                sect_5_top_cham_2_top_dia,
                                sect_5_top_cham_2_bot_dia,
                                sect_5_bot_cham_exists,
                                sect_5_bot_cham_1_top_thickness,
                                sect_5_bot_cham_1_top_dia,
                                sect_5_bot_cham_1_bot_dia,
                                sect_5_bot_cham_2_bot_thickness,
                                sect_5_bot_cham_2_top_dia,
                                sect_5_bot_cham_2_bot_dia
                                );    
            }
            else
            {
                echo("SECTION 5 DOES NOT EXIST"); 
            }

        }//end of union
            
            //remove bits from the round
            
            
            
        }//end difference
        
}//end module

////////////////////////////////////////////////////////
//round stage 2 build
////////////////////////////////////////////////////////

module round_stage_2_build()
{

    difference()
    {
        union()
        {
            //draw the box base 
            round_stage_1_build();
                
            //add additional bits
                
            }//end of union stage 2 build

            // start of difference stage 2 build
            
/////////////////////////////////////////////////////////
//remove slotted holes
/////////////////////////////////////////////////////////
        
            if (slotted_holes_exist ==1)
            {
                circular_slots  (   number_of_slots,
                                    total_slots_angle,
                                    initial_slot_offset,
                                    individual_slot_angle,
                                    slot_diameter,
                                    slot_distance_from_centre);
            }
            else
                echo("NO SLOTTED HOLES EXIST");
            
/////////////////////////////////////////////////////////
//remove spoke holes
/////////////////////////////////////////////////////////

            if (spoke_holes_exist ==1)
            {
            circular_spokes(  number_of_spoke,
                                total_spoke_angle,
                                initial_spoke_offset,
                                individual_spoke_angle,
                                spoke_inner_corner_radius,
                            spoke_outer_corner_radius,
                                spoke_inner_distance_from_centre,
                                spoke_outer_distance_from_centre);  
            }
            else
                echo("NO SPOKE HOLES EXIST");
       

        }//end difference
}//end module stage 2 build

////////////////////////////////////////////////////////
//shaft stage 3 build- not yet used
//if required add 
//
//then if required remove 
//
////////////////////////////////////////////////////////

module round_stage_3_build()
{
        translate([0,0,0])
        rotate ([0,0,0])   
        difference()
        {
            union()
                {
                //import stage 2 build of the part
                round_stage_2_build();
                    
                //add stage 3 bits to go onto the assembly
                    
                //full length ext spline
                //full length int spline
                    
                    
                }//end union
 
                
                //subtract stage 3 bits from the assembly
                
                //remove holes
                if (full_holes_exist == 1)
                {    
                    holes_from_array ( full_first_hole,
                                       full_last_hole);
                }
                
                
                //full length ext spline
                //full length int spline


        }//end difference
}


////////////////////////////////////////////////////////////////////////////
// CONSTUCT A SECTION OF THE BODY
////////////////////////////////////////////////////////////////////////////       
    
module build_section (  section_identity,
                        body_exist,
                        section_base_height,
                        section_profile,
                        section_diameter,
                        section_height,
                        section_length,
                        section_width, 
                        sect_bore_exists,
                        sect_bore_type,
                        sect_bore_dia,
                        sect_bore_height,
                        sect_bore_width_of_flats,
                        top_radius_exists,
                        top_radius_1_top_thickness,
                        top_radius_1_major_dia,
                        top_radius_2_bot_thickness,
                        top_radius_2_major_dia,
                        bot_radius_exists,
                        bot_radius_1_top_thickness,
                        bot_radius_1_major_dia,
                        bot_radius_2_bot_thickness,
                        bot_radius_2_major_dia,
                        bot_inner_radius_exists,
                        section_with_bot_inner_radius,
                        bot_inner_radius_size,
                        top_cham_exists,
                        top_cham_1_top_thickness,
                        top_cham_1_top_dia,
                        top_cham_1_bot_dia,
                        top_cham_2_bot_thickness,
                        top_cham_2_top_dia,
                        top_cham_2_bot_dia,
                        bot_cham_exists,
                        bot_cham_1_top_thickness,
                        bot_cham_1_top_dia,
                        bot_cham_1_bot_dia,
                        bot_cham_2_bot_thickness,
                        bot_cham_2_top_dia,
                        bot_cham_2_bot_dia
                        )
{
    difference()
    {
        union()
        {          
            
            //add the bits of section 1 together from here
            if (section_profile==1)
            {
                
            draw_body_middle(section_identity,
                            body_exist,
                            section_base_height,
                            section_height,
                            section_diameter,
                            top_cham_exists,
                            top_cham_1_top_thickness,
                            top_cham_1_top_dia,
                            top_cham_1_bot_dia,
                            top_cham_2_bot_thickness,
                            top_cham_2_top_dia,
                            top_cham_2_bot_dia,
                            bot_cham_exists,
                            bot_cham_1_top_thickness,
                            bot_cham_1_top_dia,
                            bot_cham_1_bot_dia,
                            bot_cham_2_bot_thickness,
                            bot_cham_2_top_dia,
                            bot_cham_2_bot_dia,
                            top_radius_exists,
                            top_radius_1_top_thickness,
                            top_radius_1_major_dia,
                            top_radius_2_bot_thickness,
                            top_radius_2_major_dia,
                            bot_radius_exists,
                            bot_radius_1_top_thickness,
                            bot_radius_1_major_dia,
                            bot_radius_2_bot_thickness,
                            bot_radius_2_major_dia,
                            top_inner_radius_exists,
                            section_with_top_inner_radius,
                            top_inner_radius_size,
                            bot_inner_radius_exists,
                            section_with_bot_inner_radius,
                            bot_inner_radius_size);       
                
            }
            else if (section_profile==2) //screw head
            {
                if (screw_head_type==1) //hex head bolt
                {
                    generate_polygon(   section_base_height,
                                        6,
                                        section_height,
                                        ThreadToHexHeadAcrossFlats(screw_metric_size)/cos(30)
                                        );
                }
                
                else if (screw_head_type==2) //allen head bolt
                {
                    translate([0,0,section_base_height])
                    cylinder(ThreadToCapHeadDepth(screw_metric_size),
                             d1=ThreadToCapHeadDiameter(screw_metric_size),
                             d2=ThreadToCapHeadDiameter(screw_metric_size));
                }
                
                else if (screw_head_type==3) // dome head screw
                {
                    
                }
                
                else if (screw_head_type==4) //couner sunk screw
                {
                    translate([0,0,section_base_height])
                    cylinder((screw_metric_size/2)*tan(45),
                             d1=screw_metric_size*2,
                             d2=screw_metric_size);   
                }
                
                else
                    
                echo("SECTION",section_identity,"INCORRECT SCREW HEAD TYPE");
             
                
            }             
 
            else if (section_profile==4)
            {
                translate([0,0,section_base_height+section_height/2])
                cube([section_length,section_width,section_height],center=true);
        
            }
            else if (section_profile==0)
            {
            echo("SECTION",section_identity,"NO BODY TO DRAW");
            }
            else 
                generate_polygon(section_base_height,section_profile,section_height,section_diameter);
 
        
//////////////////////////////////////////////////////////////       
// add the chamfers and radiuis here
//////////////////////////////////////////////////////////////       

////// make into modules
//////////////////////////////////////////////////////////////       
//add the top chamfer to the build
     
        
        if  (top_cham_exists == 1)
            {
            add_upper_chamfer(  section_base_height,
                                section_height,
                                top_cham_1_top_thickness,
                                top_cham_1_top_dia,
                                top_cham_1_bot_dia,
                                top_cham_2_bot_thickness,
                                top_cham_2_top_dia,
                                top_cham_2_bot_dia);    
            }
        else
        
            echo("SECTION",section_identity,"NO TOP CHAMFER SELECTED");
        
//////////////////////////////////////////////////////////////       
//add the bottam chamfer to the build 
        
        if  (bot_cham_exists == 1)
            {
            add_lower_chamfer(  section_base_height,
                                section_height,
                                bot_cham_1_top_thickness,
                                bot_cham_1_top_dia,
                                bot_cham_1_bot_dia,
                                bot_cham_2_bot_thickness,
                                bot_cham_2_top_dia,
                                bot_cham_2_bot_dia);               
            }
        else
            echo("SECTION",section_identity,"NO BOTTOM CHAMFER SELECTED"); 
        
//////////////////////////////////////////////////////////////       
//add the top radius to the build 
        
        if  (top_radius_exists == 1)
            {

                //call the module to do the cad
                add_upper_radius (  section_base_height,
                                    section_height,
                                    top_radius_1_top_thickness,
                                    top_radius_1_major_dia,
                                    top_radius_2_bot_thickness,
                                    top_radius_2_major_dia);
            }
        else
            echo("SECTION",section_identity,"NO TOP RADIUS SELECTED");
        
//////////////////////////////////////////////////////////////       
//add the bottom radius to the build 
        
        if  (bot_radius_exists == 1)
            {
                //call the module to do the cad
                add_lower_radius (  section_base_height,
                                    section_height,
                                    bot_radius_1_top_thickness,
                                    bot_radius_1_major_dia,
                                    bot_radius_2_bot_thickness,
                                    bot_radius_2_major_dia);                
            }
        else
            echo("SECTION",section_identity,"NO BOTTOM RADIUS SELECTED");    

 
        
//////////////////////////////////////////////////////////////       
//add sticky out external spline to the build 
        
        if ((external_spline_exists==1)&&(external_spline_type==2)&&(section_identity==section_with_external_spline)&&(body_exist!=0))
            {
                translate([0,0,section_base_height])
                external_spline_add(section_diameter,
                                    section_height,
                                    number_of_ext_splines,
                                    external_spline_offset,
                                    external_spline_width,
                                    external_spline_depth);
            }
        else
            echo("SECTION",section_identity,"HAS NO POSITIVE EXTERNAL SPLINES");         



//////////////////////////////////////////////////////////////       
//add sticky out radial graduations to the surface 
        
        if  ((radial_graduations_exist == 1)&&(scale_type==2 )&&(section_identity==section_with_radial_graduations))
            {
                radial_ticks(section_base_height,
                             section_diameter,
                             section_height);                
            }
        else
            echo("SECTION",section_identity,"NO SURFACE RADIAL GRADUATIONS SELECTED");
    
//////////////////////////////////////////////////////////////       
//add sticky out axial graduations to the surface
//////////////////////////////////////////////////////////////       
        
        
        if  ((axial_graduations_exist == 1)&&(scale_type_ax==2 )&&(section_identity==section_with_axial_graduations))
            {
                axial_ticks(section_base_height,
                            section_diameter,
                            section_height);                
            }
        else
            echo("SECTION",section_identity,"NO SURFACE AXIAL GRADUATIONS SELECTED"); 
//////////////////////////////////////////////////////////////       
//add protruding cams to the surface
//////////////////////////////////////////////////////////////       

        if  ((switch_cams_1_exist == 1)&&
             (switch_cams_1_type==1 )&&(section_identity==section_with_sw_cams_1))
            {
                generate_cams(section_base_height,
                              section_diameter,
                              switch_cams_1_type,
                              switch_cams_1_qty,
                              switch_cams_1_spacing,
                              switch_cams_1_width,
                              switch_cams_1_height,    
                              switch_cams_1_major_dia,
                              switch_cams_1_neg_cam_dia,
                              switch_cams_1_rot_offset,
                              switch_cams_1_base_height);
            }
        else
            echo("SECTION",section_identity,"NO POSITIVE CAM SET 1"); 
 
        if  ((switch_cams_2_exist == 1)&&
             (switch_cams_2_type==1 )&&(section_identity==section_with_sw_cams_2))
            {
                generate_cams(section_base_height,
                              section_diameter,
                              switch_cams_2_type,
                              switch_cams_2_qty,
                              switch_cams_2_spacing,
                              switch_cams_2_width,
                              switch_cams_2_height,    
                              switch_cams_2_major_dia,
                              switch_cams_2_neg_cam_dia,
                              switch_cams_2_rot_offset,
                              switch_cams_2_base_height);
            }
        else
            echo("SECTION",section_identity,"NO POSITIVE CAM SET 2"); 
        
//////////////////////////////////////////////////////////////       
//add sticky out axial text to the surface     
//////////////////////////////////////////////////////////////       
     
     //protruding axial text to surface
        if  ((axial_text_exist ==1)&&(axial_text_type==1)&&(section_identity==section_with_axial_text))
            {
 
                add_axial_text( section_base_height,
                                section_diameter,
                                axial_text_type,
                                axial_text_height,
                                axial_text_height_trim,
                                axial_text_to_print,
                                axial_number_of_strings,
                                axial_text_within_angle,
                                axial_text_direction,
                                axial_distance_from_base,
                                axial_character_height,
                                axial_text_rotation_angle,
                                axial_text_start_rotation,
                                axial_text_font);
            }
        else
            echo("SECTION",section_identity,"NO SURFACE AXIAL TEXT SELECTED"); 
     
        //protruding radial text to surface
        if((radial_text_exist ==1)&&(radial_text_type==1)&&(section_identity==section_with_radial_text))
        {
 
                add_radial_text(    section_base_height,
                                    section_height,
                                    radial_text_type,
                                    radial_text_height,
                                    radial_text_to_print,
                                    radial_number_of_strings,
                                    radial_text_within_angle,
                                    radial_text_direction,
                                    radial_distance_from_centre,
                                    radial_character_height,
                                    radial_text_rotation_angle,
                                    radial_text_start_rotation,
                                    radial_text_font);
        }
        else
            echo("SECTION",section_identity,"NO SURFACE RADIAL TEXT SELECTED"); 
 
//////////////////////////////////////////////////////////////       
//add gear 1 to the build
        
        
        if  ((gear_1_exists == 1)&&(section_identity==section_with_gear_1))
            {
                translate([0,0,section_base_height])
                if(gear_1_type==1)    //spur gear
                {
                     spur_gear (g1_modulus,
                                g1_number_of_teeth,
                                g1_missing_teeth,
                                g1_gear_thickness,
                                g1_bore_size,
                                g1_bore_width_of_flats, 
                                g1_pressure_angle, 
                                g1_helix_angle, 
                                g1_optimized);
                }
                else if(gear_1_type==2)     //spur bevel gear
                {
                     bevel_gear(g1_modulus,
                                g1_number_of_teeth,
                                g1_missing_teeth,
                                g1_bevel_angle,
                                g1_gear_thickness,
                                g1_bore_size,
                                g1_bore_width_of_flats, 
                                g1_pressure_angle, 
                                g1_helix_angle);
                }
                else if(gear_1_type==3)     //spur ring gear
                {
                     spur_ring_gear(g1_modulus,
                                    g1_number_of_teeth,
                                    g1_missing_teeth,
                                    g1_gear_thickness,
                                    g1_ring_edge_width,
                                    g1_pressure_angle, 
                                    g1_helix_angle);
                }
                else if(gear_1_type==4)     //herringbone spur gear
                {
                    herring_bone_spur_gear (g1_modulus,
                                            g1_number_of_teeth,
                                            g1_missing_teeth,
                                            g1_gear_thickness,
                                            g1_bore_size,
                                            g1_bore_width_of_flats, 
                                            g1_pressure_angle, 
                                            g1_helix_angle, 
                                            g1_optimized);
                }
                else if(gear_1_type==5)     //herringbone bevel gear
                {
                 herring_bone_bevel_gear(g1_modulus,
                                         g1_number_of_teeth,
                                         g1_missing_teeth,
                                         g1_bevel_angle,
                                         g1_gear_thickness,
                                         g1_bore_size,
                                         g1_bore_width_of_flats, 
                                         g1_pressure_angle, 
                                         g1_helix_angle);   
                }
                else if(gear_1_type==6)     //herringbone spur ring gear
                {
                 herring_bone_ring_gear(g1_modulus,
                                        g1_number_of_teeth,
                                        g1_missing_teeth,
                                        g1_gear_thickness,
                                        g1_ring_edge_width,
                                        g1_pressure_angle,
                                        g1_helix_angle);   
                }
                else if(gear_1_type==7)     //worm gear
                {
                     worm(g1_modulus,
                          g1_number_of_starts,
                          g1_worm_length,
                          g1_bore_size,
                          g1_bore_width_of_flats,
                          g1_pressure_angle, 
                          g1_gradient_angle,
                          g1_unsplit_worm_print  );
                }
            }
            else
                echo("SECTION",section_identity,"NO GEAR 1 SELECTED");         
        
 
 
 //////////////////////////////////////////////////////////////       
//add gear 2 to the build
        
        
        if  ((gear_2_exists == 1)&&(section_identity==section_with_gear_2))
            {
                translate([0,0,section_base_height])
                if(gear_2_type==1)    //spur gear
                {
                     spur_gear (g2_modulus,
                                g2_number_of_teeth,
                                g2_missing_teeth,
                                g2_gear_thickness,
                                g2_bore_size,
                                g2_bore_width_of_flats, 
                                g2_pressure_angle, 
                                g2_helix_angle, 
                                g2_optimized);
                }
                else if(gear_2_type==2)     //spur bevel gear
                {
                     bevel_gear(g2_modulus,
                                g2_number_of_teeth,
                                g2_missing_teeth,
                                g2_bevel_angle,
                                g2_gear_thickness,
                                g2_bore_size,
                                g2_bore_width_of_flats, 
                                g2_pressure_angle, 
                                g2_helix_angle);
                }
                else if(gear_2_type==3)     //spur ring gear
                {
                     spur_ring_gear(g2_modulus,
                                    g2_number_of_teeth,
                                    g2_missing_teeth,
                                    g2_gear_thickness,
                                    g2_ring_edge_width,
                                    g2_pressure_angle, 
                                    g2_helix_angle);
                }
                else if(gear_2_type==4)     //herringbone spur gear
                {
                    herring_bone_spur_gear (g2_modulus,
                                            g2_number_of_teeth,
                                            g2_missing_teeth,
                                            g2_gear_thickness,
                                            g2_bore_size,
                                            g2_bore_width_of_flats, 
                                            g2_pressure_angle, 
                                            g2_helix_angle, 
                                            g2_optimized);
                }
                else if(gear_2_type==5)     //herringbone bevel gear
                {
                 herring_bone_bevel_gear(g2_modulus,
                                         g2_number_of_teeth,
                                         g2_missing_teeth,
                                         g2_bevel_angle,
                                         g2_gear_thickness,
                                         g2_bore_size,
                                         g2_bore_width_of_flats, 
                                         g2_pressure_angle, 
                                         g2_helix_angle);   
                }
                else if(gear_2_type==6)     //herringbone spur ring gear
                {
                 herring_bone_ring_gear(g2_modulus,
                                        g2_number_of_teeth,
                                        g2_missing_teeth,
                                        g2_gear_thickness,
                                        g2_ring_edge_width,
                                        g2_pressure_angle,
                                        g2_helix_angle);   
                }
                else if(gear_2_type==7)     //worm gear
                {
                     worm(g2_modulus,
                          g2_number_of_starts,
                          g2_worm_length,
                          g2_bore_size,
                          g2_bore_width_of_flats,
                          g2_pressure_angle, 
                          g2_gradient_angle,
                          g2_unsplit_worm_print  );
                }
            }
            else
                echo("SECTION",section_identity,"NO GEAR 2 SELECTED");         
        
 
        
//////////////////////////////////////////////////////////////       
//add pulley 1 to the body 
      
        if  ((pulley_1_exists == 1)&&(section_identity==section_with_pulley_1))
            {
                //position the pulley in the section                
                translate([0,0,section_base_height+pulley_1_base_height])
                //generate the pulley
                generate_pulley( pulley_1_profile,
                                 pulley_1_number_of_teeth,
                                 pulley_1_tooth_width,
                                 pulley_1_additional_tooth_width,
                                 pulley_1_additional_tooth_depth,
                                 pulley_1_diameter_reduction);
            }
        else
            echo("SECTION",section_identity,"NO PULLEY 1 SELECTED");    

//////////////////////////////////////////////////////////////       
//add pulley 2 to the body 
      
        if  ((pulley_2_exists == 1)&&(section_identity==section_with_pulley_2))
            {
                //position the pulley in the section                
                translate([0,0,section_base_height+pulley_2_base_height])
                //generate the pulley
                generate_pulley( pulley_2_profile,
                                 pulley_2_number_of_teeth,
                                 pulley_2_tooth_width,
                                 pulley_2_additional_tooth_width,
                                 pulley_2_additional_tooth_depth,
                                 pulley_2_diameter_reduction);

            }
        else
            echo("SECTION",section_identity,"NO PULLEY 2 SELECTED");
        
//////////////////////////////////////////////////////////////       
//add thread 1 to the body 
      
        if  ((thread_1_exists == 1)&&(thread_1_type==0)&&(section_identity==section_with_thread_1))
            {
                //position the thread in the section                
                translate([0,0,section_base_height])
                //generate the thread
                
                if (thread_1_form == 0)
                {
                //ScrewThread(outer_diam, height, pitch=0, tooth_angle=30, tolerance=0.4, tip_height=0, tooth_height=0, tip_min_fract=0)
                ScrewThread( thread_1_metric_size,
                             thread_1_length,
                             thread_1_pitch,
                             thread_1_tooth_angle);
                }   
                else if (thread_1_form == 1)
                {
                trapezoidThread(
	thread_1_length,				    // axial length of the threaded rod
	thread_1_pitch,				        // axial distance from crest to crest
	thread_1_pitch_radius,			    // radial distance from center to mid-profile
	thread_1_tooth_angle,			    // angle between the two faces of the thread
	thread_1_RH,				        // true/false the thread winds clockwise / counterclockwise
	thread_1_steps_per_turn,			// number of slices to create per turn
	thread_1_clearance,			    // radial clearance, normalized to thread height
	thread_1_backlash,			    // axial clearance, normalized to pitch
	threadHeightToPitch=0.5,	// ratio between the height of the profile and the pitch
	profileRatio=0.5			// ratio between the lengths of the raised part of the profile and the pitch
                      );    
                }
                    
                
                
            }
        else
            echo("SECTION",section_identity,"NO THREADED STUD 1 SELECTED");
        
  //////////////////////////////////////////////////////////////       
//add thread 2 to the body 
      
        if  ((thread_2_exists == 1)&&(thread_2_type==0)&&(section_identity==section_with_thread_2))
            {
                //position the thread in the section                
                translate([0,0,section_base_height])
                //generate the thread
                
                if (thread_2_form == 0)
                {
                //ScrewThread(outer_diam, height, pitch=0, tooth_angle=30, tolerance=0.4, tip_height=0, tooth_height=0, tip_min_fract=0)
                ScrewThread( thread_2_metric_size,
                             thread_2_length,
                             thread_2_pitch,
                             thread_2_tooth_angle);
                }   
                else if (thread_2_form == 1)
                {
                trapezoidThread(
	thread_2_length,				    // axial length of the threaded rod
	thread_2_pitch,				        // axial distance from crest to crest
	thread_2_pitch_radius,			    // radial distance from center to mid-profile
	thread_2_tooth_angle,			    // angle between the two faces of the thread
	thread_2_RH,				        // true/false the thread winds clockwise / counterclockwise
	thread_2_steps_per_turn,			// number of slices to create per turn
	thread_2_clearance,			    // radial clearance, normalized to thread height
	thread_2_backlash,			    // axial clearance, normalized to pitch
	threadHeightToPitch=0.5,	// ratio between the height of the profile and the pitch
	profileRatio=0.5			// ratio between the lengths of the raised part of the profile and the pitch
                      );    
                }
                
            }
            
            
        else
            echo("SECTION",section_identity,"NO THREADED STUD 2 SELECTED");
              
//////////////////////////////////////////////////////////////       
//end of union 

        }   //end of union



//////////////////////////////////////////////////////////////       
// end of adding the bits together section 1
//////////////////////////////////////////////////////////////       
//start of deleting bits from section 1
//////////////////////////////////////////////////////////////       
 
   
        
        //remove section spindle bore
        
        if (sect_bore_exists==1)
            {
                
                remove_section_bore(    section_base_height,
                                        sect_bore_type,
                                        sect_bore_dia,
                                        sect_bore_height,
                                        sect_bore_width_of_flats
                                    );
            }
       else
            echo("SECTION",section_identity,"HAS NO SPINDLE BORE");
       
       //remove radius at top of bore
        
        if (sect_bore_exists==1 && top_inner_radius_exists==1&&section_identity==section_with_top_inner_radius)
            {
                
                remove_upper_inner_radius ( section_base_height,
                                            section_height,
                                            sect_bore_dia,
                                            top_inner_radius_size);
                
            }
        else
            echo("SECTION",section_identity,"HAS NO TOP INNER RADIUS");
       
        //remove radius at bottom of bore
        
        if (sect_bore_exists==1 && bot_inner_radius_exists==1&&section_identity==section_with_bot_inner_radius)
            {
                
               remove_lower_inner_radius (  section_base_height,
                                            section_height,
                                            sect_bore_dia,
                                            bot_inner_radius_size); 
            }
        else
            echo("SECTION",section_identity,"HAS NO bottom INNER RADIUS");

              
        //remove section lower recess
        
        if ((bottom_face_recess_exists==1)&&(section_identity==section_with_bottom_recess))
            {
                translate([0,0,section_base_height])
                cylinder(bottom_face_recess_depth ,d1=bottom_face_recess_dia,d2=bottom_face_recess_dia,center=false);
            }
        else
            echo("SECTION",section_identity,"HAS NO LOWER RECESS");
        
        //remove section upper recess
        
        if ((top_face_recess_exists==1)&&(section_identity==section_with_top_recess))
            {
                translate([0,0,section_base_height+section_height-top_face_recess_depth
])
                cylinder(top_face_recess_depth, d1=top_face_recess_dia, d2=top_face_recess_dia, center=false);
            }
        else
            echo("SECTION",section_identity,"HAS NO UPPER RECESS");
       
      
     //remove section 1 internal keyway
        
        if ((internal_keyway_exists==1)&&(section_identity==section_with_internal_key))
            {
                translate([0,0,section_base_height])
                internal_key_sub(   sect_bore_dia,
                                    section_height,
                                    number_of_int_keyway,
                                    internal_key_offset,
                                    internal_key_width,
                                    internal_key_height);
            }
        else
            echo("SECTION",section_identity,"HAS NO INTERNAL KEYWAY"); 
      
      
        //remove section 1 external keyway
        
        if ((external_keyway_exists==1)&&(section_identity==section_with_external_key))
            {
                translate([0,0,section_base_height])
                external_key_sub(   section_diameter,
                                    section_height,
                                    number_of_ext_keyway,
                                    external_key_offset,
                                    external_key_width,
                                    external_key_height);
            }
        else
            echo("SECTION",section_identity,"HAS NO EXTERNAL KEYWAY");        
        
        //remove section 1 internal spline
        
        if ((internal_spline_exists==1)&&(internal_spline_type==1)&&(section_identity>=section_with_internal_spline))
            {
                translate([0,0,section_base_height])
                internal_spline_sub(sect_bore_dia,
                                    section_height,
                                    number_of_int_splines,
                                    internal_spline_offset,
                                    internal_spline_width,
                                    internal_spline_depth);
            }
        else
            echo("SECTION",section_identity,"HAS NO NEGATIVE INTERNAL SPLINES");      
        
        //remove section 1 external spline
        
        if ((external_spline_exists==1)&&(external_spline_type==1)&&(section_identity==section_with_external_spline))
            {
                translate([0,0,section_base_height])
                external_spline_sub(section_diameter,
                                    section_height,
                                    number_of_ext_splines,
                                    external_spline_offset,
                                    external_spline_width,
                                    external_spline_depth);
            }
        else
            echo("SECTION",section_identity,"HAS NO NEGATIVE EXTERNAL SPLINES");         
        
        
        
        
        //remove parts section 1 spindle clamp
        
        
        
        if ((bottom_spindle_clamp_exists==1)&&(section_identity==section_with_bottom_spindle_clamp))
            {

                spindle_clamp(  section_base_height,
                                section_height,
                                section_diameter,
                                bot_spin_clamp_split_type,
                                bot_spin_clamp_split_width,
                                bot_spin_clamp_fixing_height,
                                bot_spin_clamp_fixing_radius,
                                bot_spin_clamp_head_offset,
                                bot_spin_clamp_nut_offset,
                                bot_spin_clamp_rotate,
                                bot_spin_clamp_screw_bore_size,
                                bot_spin_clamp_screw_head_size,
                                bot_spin_clamp_hex_nut_size
                                );
               
            }
        else
            echo("SECTION",section_identity,"IS NOT A BOTTOM SPINDLE CLAMP");   
  
  
        
        if ((top_spindle_clamp_exists==1)&&(section_identity==section_with_top_spindle_clamp))
            {
    
                spindle_clamp(  section_base_height,
                                section_height,
                                section_diameter,
                                top_spin_clamp_split_type,
                                top_spin_clamp_split_width,
                                top_spin_clamp_fixing_height,
                                top_spin_clamp_fixing_radius,
                                top_spin_clamp_head_offset,
                                top_spin_clamp_nut_offset,
                                top_spin_clamp_rotate,
                                top_spin_clamp_screw_bore_size,
                                top_spin_clamp_screw_head_size,
                                top_spin_clamp_hex_nut_size
                                );
               
            }
        else
            echo("SECTION",section_identity,"IS NOT A TOP SPINDLE CLAMP");   
     
        //indented radial scale in union area
        if  ((radial_graduations_exist == 1)&&(scale_type==1 )&&(section_identity==section_with_radial_graduations))
            {
                translate([0,0,0.1])
                radial_ticks(section_base_height,
                             section_diameter,
                             section_height);
            }
        else
            echo("SECTION",section_identity,"HAS NO NEGATIVE RADIAL GRADUATIONS");   
     
       //indented axial scale in union area
        if  ((axial_graduations_exist == 1)&&(scale_type_ax==1 )&&(section_identity==section_with_axial_graduations))
            {
                translate([0,0,0.1])
                axial_ticks(section_base_height,
                            section_diameter,
                            section_height);
            }
        else
            echo("SECTION",section_identity,"HAS NO NEGATIVE AXIAL GRADUATIONS");
        
//////////////////////////////////////////////////////////////       
//remove negative cams from the surface
//////////////////////////////////////////////////////////////       

        if  ((switch_cams_1_exist == 1)&&
             (switch_cams_1_type==0 )&&(section_identity==section_with_sw_cams_1))
            {
                generate_cams(section_base_height,
                              section_diameter,
                              switch_cams_1_type,
                              switch_cams_1_qty,
                              switch_cams_1_spacing,
                              switch_cams_1_width,
                              switch_cams_1_height,    
                              switch_cams_1_major_dia,
                              switch_cams_1_neg_cam_dia,
                              switch_cams_1_rot_offset,
                              switch_cams_1_base_height);
            }
        else
            echo("SECTION",section_identity,"NO NEGITIVE CAM SET 1"); 
 
        if  ((switch_cams_2_exist == 1)&&
             (switch_cams_2_type==0 )&&(section_identity==section_with_sw_cams_2))
            {
                generate_cams(section_base_height,
                              section_diameter,
                              switch_cams_2_type,
                              switch_cams_2_qty,
                              switch_cams_2_spacing,
                              switch_cams_2_width,
                              switch_cams_2_height,    
                              switch_cams_2_major_dia,
                              switch_cams_2_neg_cam_dia,
                              switch_cams_2_rot_offset,
                              switch_cams_2_base_height);
            }
        else
            echo("SECTION",section_identity,"NO NEGITIVE CAM SET 2"); 
                
        //recessed axial text
            
            if((axial_text_exist ==1)&&(axial_text_type==2)&&(section_identity==section_with_axial_text))
            {
                translate([0,0,0.1])// added because sometimes indented text does not cut surface
                add_axial_text( section_base_height,
                                section_diameter,
                                axial_text_type,
                                axial_text_height,
                                axial_text_height_trim,
                                axial_text_to_print,
                                axial_number_of_strings,
                                axial_text_within_angle,
                                axial_text_direction,
                                axial_distance_from_base,
                                axial_character_height,
                                axial_text_rotation_angle,
                                axial_text_start_rotation,
                                axial_text_font);
            }
            else
                echo("SECTION",section_identity,"HAS NO NEGATIVE AXIAL TEXT"); 
   
       if((radial_text_exist ==1)&&(radial_text_type==2)&&(section_identity==section_with_radial_text))
            {
                translate([0,0,0.1])// added because sometimes indented text does not cut surface
                add_radial_text(    section_base_height,
                                    section_height,
                                    radial_text_type,
                                    radial_text_height,
                                    radial_text_to_print,
                                    radial_number_of_strings,
                                    radial_text_within_angle,
                                    radial_text_direction,
                                    radial_distance_from_centre,
                                    radial_character_height,
                                    radial_text_rotation_angle,
                                    radial_text_start_rotation,
                                    radial_text_font);
            }
        else
            echo("SECTION",section_identity,"HAS NO NEGATIVE RADIAL TEXT"); 
    
/////////////////////////////////////////////////////////
//remove internal circular groove

        if ((internal_circ_groove_exist ==1)&&(section_identity==section_with_circular_groove))
        {
              circ_internal_groove( section_base_height,
                                    section_height,
                                    section_diameter,
                                    sect_bore_dia
                                  );  
        }
        else
            echo("SECTION",section_identity,"HAS NO INTERNAL CIRCULAR GROOVE"); 
 
        
/////////////////////////////////////////////////////////
//remove internal rectangular groove

        if ((internal_rect_groove_exist ==1)&&(section_identity==section_with_rectangular_groove))
        {
              rect_internal_groove( section_base_height,
                                    section_height,
                                    section_diameter,
                                    sect_bore_dia
                                  );  
        }
        else
            echo("SECTION",section_identity,"HAS NO INTERNAL RECTANGULAR GROOVE"); 
        
        




       
/////////////////////////////////////////////////////////
//remove weight saving
        
        if ((weight_saving_exists ==1)&&(section_identity==section_with_weight_saving))
        {
          translate([0,0,section_base_height])
          weight_reduction  (section_height,
                             section_diameter
                            );  
            
        }
        else
            echo("SECTION",section_identity,"HAS NO WEIGHT SAVING");     
       
       
        

/////////////////////////////////////////////////////////
//knurling outside face

        if ((knurling_exists ==1)&&(section_identity==section_with_knurling))
        {
            translate([0,0,section_base_height])
            knurl_outside_surface(  section_diameter,
                                    number_knurl_starts,
                                    total_knurl_angle,
                                    knurling_depth,
                                    knurling_width,
                                    knurling_base_height
                            ); 
            
        }
        else
            echo("SECTION",section_identity,"HAS NO KNURLING"); 

/////////////////////////////////////////////////////////
//put a drive hole in base of body
        
        
        // puts a hole in the base of the body section

        if ((base_drive_exists ==1)&&(section_identity==section_with_base_drive))
        {
            // base driver 0=allen key
            if  (base_driver_type ==0)
            {
              translate([0,0,section_base_height])
              allen_key_in_base();  
            }
            
            // base driver 1=pozi drive
            else if  (base_driver_type ==1)
            {
                translate([0,0,section_base_height])
                pozi_drive( );   
            }
            
            // base driver 2=screwdriver slot
            else if  (base_driver_type ==2)
            {
                translate([0,0,section_base_height])
                screw_drive( );   
            }
            
            // base driver 3=triangular key
            else if  (base_driver_type ==3)
            {
                translate([0,0,section_base_height])
                triangular_drive( );
            }
            
            // base driver 4=square key
            else if  (base_driver_type ==4)
            {
                translate([0,0,section_base_height])
                square_drive ( );
            }
            
            // base driver 5=torx driver
            else if  (base_driver_type ==5)
            {
                translate([0,0,section_base_height])
                torx_drive();      
            }
            
            // base driver 6=nut housing
            else if  (base_driver_type ==6)
            {
                    translate([0,0,section_base_height])
                    nut_housing_in_base();
      
            }
        }
        else
            echo("SECTION",section_identity,"HAS NO HOLE IN BASE"); 

/////////////////////////////////////////////////////////
//puts axial and radial fixings in body
            
//axial fixing 1    
        if ((axial_fixing_1_exists==1)&&(section_identity==section_with_axial_fixing_1)&&(axial_fixing_1_pcd_is_square!=1))
            {
                    axial_fixing(section_base_height,
                                    section_height,
                                    axial_fixing_1_pcd,
                                    axial_fixing_1_qty,
                                    axial_metric_thread_1_size,
                                    axial_fixing_1_depth,
                                    axial_fixing_1_offset,
                                    axial_cbore_1_exists,
                                    axial_cbore_1_depth);
            }
        else if ((axial_fixing_1_exists==1)&&(section_identity==section_with_axial_fixing_1)&&(axial_fixing_1_pcd_is_square==1))
            {
                    axial_fixing_square(section_base_height,
                                        section_height,
                                        axial_fixing_1_pcd,
                                        axial_fixing_1_qty,
                                        axial_metric_thread_1_size,
                                        axial_fixing_1_depth,
                                        axial_fixing_1_offset,
                                        axial_cbore_1_exists,
                                        axial_cbore_1_depth);
            } 
        
        else
            echo("SECTION",section_identity,"HAS NO AXIAL FIXING 1 HOLES"); 
      
//axial fixing 2    
        if ((axial_fixing_2_exists==1)&&(section_identity==section_with_axial_fixing_2)&&(axial_fixing_2_pcd_is_square!=1))
            {
                    axial_fixing(section_base_height,
                                    section_height,
                                    axial_fixing_2_pcd,
                                    axial_fixing_2_qty,
                                    axial_metric_thread_2_size,
                                    axial_fixing_2_depth,
                                    axial_fixing_2_offset,
                                    axial_cbore_2_exists,
                                    axial_cbore_2_depth);
            }
        else if ((axial_fixing_2_exists==1)&&(section_identity==section_with_axial_fixing_2)&&(axial_fixing_2_pcd_is_square==1))
            {
                    axial_fixing_square(section_base_height,
                                        section_height,
                                        axial_fixing_1_pcd,
                                        axial_fixing_1_qty,
                                        axial_metric_thread_1_size,
                                        axial_fixing_1_depth,
                                        axial_fixing_1_offset,
                                        axial_cbore_1_exists,
                                        axial_cbore_1_depth);
            } 
        
        else
            echo("SECTION",section_identity,"HAS NO AXIAL FIXING 2 HOLES");
       
        
//radial fixing 1
        
        if ((radial_fixing_exists==1)&&(section_identity==section_with_radial_fixing))
            {
                    radial_fixing(  section_base_height,
                                    section_diameter
                                  );
            }
        else
            echo("SECTION",section_identity,"HAS NO RADIAL FIXING HOLES"); 

//////////////////////////////////////////////////////////////       
//remove threaded hole 1 from the body 
      
        if  ((thread_1_exists == 1)&&(thread_1_type==1)&&(section_identity==section_with_thread_1))
            {
                //position the thread in the section                
                translate([0,0,section_base_height+thread_1_base_height])
                
                if (thread_1_form == 0) //metric thread
                {
                //generate the thread
                //ScrewHole(outer_diam, height, position=[x,y,z], rotation=[y,x,z], pitch=0, tooth_angle=30, tolerance=0.4, tooth_height=0)
                ScrewHole(  thread_1_metric_size,
                            thread_1_length, 
                            [0,0,0], 
                            [0,0,0], 
                            thread_1_pitch);
                }
                
                else if (thread_1_form == 1) //trapazoidal thread
                {
                trapezoidThreadNegativeSpace(
	thread_1_length,				    // axial length of the threaded rod
	thread_1_pitch,				        // axial distance from crest to crest
	thread_1_pitch_radius,			    // radial distance from center to mid-profile
	thread_1_tooth_angle,			    // angle between the two faces of the thread
	thread_1_RH,				        // true/false the thread winds clockwise / counterclockwise
	thread_1_steps_per_turn,			// number of slices to create per turn
	thread_1_clearance,			    // radial clearance, normalized to thread height
	thread_1_backlash,			    // axial clearance, normalized to pitch
	thread_1_countersunk,			    // depth of 45 degree chamfered entries, normalized to pitch
	threadHeightToPitch=0.5,	// ratio between the height of the profile and the pitch
	profileRatio=0.5			// ratio between the lengths of the raised part of the profile and the pitch
		);    
                }
                
            }
        else
            echo("SECTION",section_identity,"NO THREADED HOLE 1 SELECTED");
  
 
        
//////////////////////////////////////////////////////////////       
//remove threaded hole 2 from the body 
      
        if  ((thread_2_exists == 1)&&(thread_2_type==1)&&(section_identity==section_with_thread_2))
            {
                //position the thread in the section                
                translate([0,0,section_base_height+thread_2_base_height])
                //generate the thread
                //ScrewHole(outer_diam, height, position=[x,y,z], rotation=[y,x,z], pitch=0, tooth_angle=30, tolerance=0.4, tooth_height=0)
                ScrewHole(  thread_2_metric_size,
                            thread_2_length, 
                            [0,0,0], 
                            [0,0,0], 
                            thread_2_pitch);
            }
        else
            echo("SECTION",section_identity,"NO THREADED HOLE 2 SELECTED");       


/////////////////////////////////////////////////////////
//end of subtractions
/////////////////////////////////////////////////////////
        
    }    //////ADDITIONS NOT SUBTRACTIONS BELOW THIS LINE (end of differance bracket)  

    
    
    //after building section 1 then subtracting all the relevant bits here is where we add some bits back on
    //mainly for putting bosses back in after recesses have been carved out
    
    
    
//////////////////////////////////////////////////////////////       
//add sticky out internal spline to the build 

       
        if ((internal_spline_exists==1)&&(internal_spline_type==2)&&(section_identity>=section_with_internal_spline)&&(body_exist!=0))
            {
                translate([0,0,section_base_height])
                internal_spline_add(sect_bore_dia,
                                    section_height,
                                    number_of_int_splines,
                                    internal_spline_offset,
                                    internal_spline_width,
                                    internal_spline_depth);
            }
        else
            echo("SECTION",section_identity,"HAS NO POSITIVE INTERNAL SPLINES");

        
  ///adds solid spokes to selected section
    
    if ((spoke_bars_exist == 1)&&(section_with_spoke_bar==section_identity))
        {   
            difference()
            {
                union()
                {
                //add the spokes
                    translate([0,0,section_base_height + spoke_bar_z_base_pos])
                    generate_spokes();
                }
                // remove anything if required
            }
            
        }
         
        else
        {
            echo("SECTION",section_identity,"ADDITION OF SPOKES DOES NOT EXIST");
        }
        
          
    
  ///adds lower boss to selected section
    
    if ((lower_boss_exists == 1)&&(section_with_lower_boss==section_identity))
        {   
            difference()
            {
                union()
                {
                    translate([0,0,section_base_height])
                    cylinder(lower_boss_height,d1=lower_boss_bot_dia,d2=lower_boss_top_dia);
                    
                }
                    remove_section_bore (section_base_height,
                                        sect_bore_type,
                                        sect_bore_dia,
                                        sect_bore_height,
                                        sect_bore_width_of_flats
                                        );
            }
            
        }
         
        else
        {
            echo("SECTION",section_identity,"ADDITION OF LOWER BOSS DOES NOT EXIST");
        }
        
        
  ///adds upper boss to selected section
        
        
        if ((upper_boss_exists == 1)&&(section_with_upper_boss==section_identity))
        {   
            difference()
            {
                union()
                {
                    translate([0,0,section_base_height+section_height-top_face_recess_depth])
                    cylinder(upper_boss_height,d1=upper_boss_bot_dia,d2=upper_boss_top_dia);  
                    
                }
                    remove_section_bore (section_base_height,
                                        sect_bore_type,
                                        sect_bore_dia,
                                        sect_bore_height,
                                        sect_bore_width_of_flats
                                        );
            }
            
        }
         
        else
        {
            echo("SECTION",section_identity,"ADDITION OF UPPER BOSS DOES NOT EXIST");
        }

}   //end of build_section module

///////////////////////////////////////////////////
// module to remove bore
////////////////////////////////////////////////////////

module remove_section_bore (section_base_height,
                            sect_bore_type,
                            sect_bore_dia,
                            sect_bore_height,
                            sect_bore_width_of_flats
                           )
{
    if (sect_bore_type==1)
    { 
        translate([0,0,section_base_height])
        cylinder(sect_bore_height,d1=sect_bore_dia,d2=sect_bore_dia,center=false);
                
     }
     else if (sect_bore_type==2)
     {
        translate([0,0,section_base_height])
        linear_extrude(height=sect_bore_height)
     {
        difference()
            { 
             //draw a 2D circle on centre axis
             circle(d=sect_bore_dia);        
             //draw a square and deduct from circle to blank one side
             translate([-sect_bore_width_of_flats*1.5,-sect_bore_dia/2]) 
             square([sect_bore_width_of_flats,sect_bore_dia]);
             //draw a square and deduct from circle to blank other side
             translate([sect_bore_width_of_flats/2,-sect_bore_dia/2]) 
             square([sect_bore_width_of_flats,sect_bore_dia]);
             }
     }
     }             
     else if (sect_bore_type==4)
     {
            translate([0,0,section_base_height+section_height/2])
            cube([sect_bore_dia,sect_bore_dia,section_height],center=true);
        
     }
     else 
            generate_polygon(section_base_height,sect_bore_type,section_height+.01,sect_bore_dia);      
        
}

///////////////////////////////////////////////
// module to build upper chamfer
////////////////////////////////////////////////////////
 
module add_upper_chamfer(   base_height,
                            section_height,
                            top_cham_1_thick,
                            top_cham_1_top_dia,
                            top_cham_1_bot_dia,
                            top_cham_2_thick,
                            top_cham_2_top_dia,
                            top_cham_2_bot_dia)
{ 
                   //top top chamfer
                    translate([0,0,(base_height+section_height-top_cham_1_thick)])
                    cylinder(top_cham_1_thick,d1=top_cham_1_bot_dia, d2=top_cham_1_top_dia, center=false);
                    //top bottom chamfer
                    translate([0,0,(base_height+section_height-top_cham_1_thick-top_cham_2_thick)])
                    cylinder(top_cham_2_thick, d1=top_cham_2_bot_dia, d2=top_cham_2_top_dia, center=false);
   
}

///////////////////////////////////////////////
// module to build lower chamfer
////////////////////////////////////////////////////////

module add_lower_chamfer(   base_height,
                            section_height,
                            bot_cham_1_thick,
                            bot_cham_1_top_dia,
                            bot_cham_1_bot_dia,
                            bot_cham_2_thick,
                            bot_cham_2_top_dia,
                            bot_cham_2_bot_dia)
{ 
                   //bot top chamfer
                    translate([0,0,(base_height+bot_cham_2_thick)])
                    cylinder(bot_cham_1_thick,d1=bot_cham_1_bot_dia, d2=bot_cham_1_top_dia, center=false);
                    //bot bottom chamfer
                    translate([0,0,(base_height)])
                    cylinder(bot_cham_2_thick, d1=bot_cham_2_bot_dia, d2=bot_cham_2_top_dia, center=false);
   
}
///////////////////////////////////////////////
// module to build upper radius
////////////////////////////////////////////////////////

module add_upper_radius (base_height,
                         section_height,
                         top_rad_1_thick,
                         top_rad_1_base_dia,
                         top_rad_2_thick,
                         top_rad_2_base_dia)
{
    //form the upper section of the upper radius
    //position the rings overall height
    translate([0,0,(base_height+section_height-top_rad_1_thick)])
    
    //extrude the ring
    rotate_extrude()
    
    //move the radius quadrant away from centre
    translate([top_rad_1_base_dia/2-top_rad_1_thick,0])  
    //subtract the squares from the circle to leave a quadrant
        
    difference()
        {
            
            //draw a 2D circle on centre axis
            circle(top_rad_1_thick);
            //draw a square and deduct from circle
            translate([-2*top_rad_1_thick,0])  
            square(top_rad_1_thick*2);  
            //draw an square and deduct from circle
            translate([-top_rad_1_thick,-2*top_rad_1_thick])  
            square(top_rad_1_thick*2);            
            
        }

         // fill in the centre section
        translate([0,0,(base_height+section_height-top_rad_1_thick)])
        cylinder (top_rad_1_thick,d1=top_rad_1_base_dia-2*top_rad_1_thick,d2=top_rad_1_base_dia-2*top_rad_1_thick);
 
             
    //form the lower section of the upper radius
    //position the rings overall height
    translate([0,0,(base_height+section_height-top_rad_1_thick)])
    
    //extrude the ring
    rotate_extrude()
    //invert the ring
    rotate([180,0,0])
    
    //move the radius quadrant away from centre
    translate([top_rad_2_base_dia/2-top_rad_2_thick,0])  
    //subtract the squares from the circle to leave a quadrant
    difference()
        {
            
            //draw a 2D circle on centre axis
            circle(top_rad_2_thick);
            //draw a square and deduct from circle
            translate([-2*top_rad_2_thick,0])  
            square(top_rad_2_thick*2);  
            //draw an square and deduct from circle
            translate([-top_rad_2_thick,-2*top_rad_2_thick])  
            square(top_rad_2_thick*2);            
            
        }
              
        // fill in the centre section
        translate([0,0,(base_height+section_height-top_rad_1_thick-top_rad_2_thick)])
        cylinder (top_rad_2_thick,d1=top_rad_2_base_dia-2*top_rad_2_thick,d2=top_rad_2_base_dia-2*top_rad_2_thick);
                
}

///////////////////////////////////////////////
// module to build lower radius
////////////////////////////////////////////////////////

module add_lower_radius (base_height,
                         section_height,
                         bot_rad_1_thick,
                         bot_rad_1_base_dia,
                         bot_rad_2_thick,
                         bot_rad_2_base_dia)
{
    //form the upper section of the lower radius
    //position the rings overall height
    translate([0,0,(base_height+bot_rad_2_thick)])
    
    //extrude the ring
    rotate_extrude()
    
    //move the radius quadrant away from centre
    translate([bot_rad_1_base_dia/2-bot_rad_1_thick,0])  
    //subtract the squares from the circle to leave a quadrant
    difference()
        {
            
            //draw a 2D circle on centre axis
            circle(bot_rad_1_thick);
            //draw a square and deduct from circle
            translate([-2*bot_rad_1_thick,0])  
            square(bot_rad_1_thick*2);  
            //draw an square and deduct from circle
            translate([-bot_rad_1_thick,-2*bot_rad_1_thick])  
            square(bot_rad_1_thick*2);            
            
        }
              
        // fill in the centre section
        translate([0,0,(base_height+bot_rad_1_thick)])
        cylinder (bot_rad_1_thick,d1=bot_rad_1_base_dia-2*bot_rad_1_thick,d2=bot_rad_1_base_dia-2*bot_rad_1_thick);
 
  
  
    //form the lower section of the lower radius
    //position the rings overall height
    translate([0,0,(base_height+bot_rad_2_thick)])
    
    //extrude the ring
    rotate_extrude()
    //invert the ring
    rotate([180,0,0])
    
    //move the radius quadrant away from centre
    translate([bot_rad_2_base_dia/2-bot_rad_2_thick,0])  
    //subtract the squares from the circle to leave a quadrant
    difference()
        {
            
            //draw a 2D circle on centre axis
            circle(bot_rad_2_thick);
            //draw a square and deduct from circle
            translate([-2*bot_rad_2_thick,0])  
            square(bot_rad_2_thick*2);  
            //draw an square and deduct from circle
            translate([-bot_rad_2_thick,-2*bot_rad_2_thick])  
            square(bot_rad_2_thick*2);            
            
        }
              
        // fill in the centre section
        translate([0,0,(base_height)])
        cylinder (bot_rad_2_thick,d1=bot_rad_2_base_dia-2*bot_rad_2_thick,d2=bot_rad_2_base_dia-2*bot_rad_2_thick);
               
}

///////////////////////////////////////////////
// module to remove upper inner radius
////////////////////////////////////////////////////////

  module remove_upper_inner_radius (    base_height,
                                        section_height,
                                        bore_dia,
                                        top_inner_radius_size)
{
    //form the upper section of the upper radius
    //position the rings overall height
    translate([0,0,(base_height+section_height-top_inner_radius_size)])
    
    //extrude the ring
    rotate_extrude()
    
    //move the radius quadrant away from centre
    translate([(bore_dia/2)+top_inner_radius_size-0.01,0])
    
    //subtract the squares from the circle to leave a quadrant
    difference()
        {
            //position the quadrant
            rotate(90)
            //draw a 2D square on centre axis
            square(top_inner_radius_size); 
            //draw an circle and deduct from square
            circle(top_inner_radius_size);
        }
}

///////////////////////////////////////////////
// module to remove lower inner radius
////////////////////////////////////////////////////////


  module remove_lower_inner_radius (    base_height,
                                        section_height,
                                        bore_dia,
                                        bot_inner_radius_size)
{
    //form the upper section of the upper radius
    //position the rings overall height
    translate([0,0,bot_inner_radius_size])
    
    //extrude the ring
    rotate_extrude()
    
    //move the radius quadrant away from centre
    translate([(bore_dia/2)+bot_inner_radius_size-0.01,0])
    
    //subtract the squares from the circle to leave a quadrant
    difference()
        {
            //position the quadrant
            rotate(180)
            //draw a 2D square on centre axis
            square(bot_inner_radius_size); 
            //draw an circle and deduct from square
            circle(bot_inner_radius_size);
        }
}

////////////////////////////////////////////////////////////////////////////
// module to manufactur a polygon other than cylinder and cubeoid

module generate_polygon (base_height,number_sides,section_height,points_diameter )
{
  translate([0,0,base_height])  
  linear_extrude(section_height)
  circle(points_diameter/2,$fn=number_sides);     
}

///////////////////////////////////////////////
// module to build a spindle clamp
// this would be one section of the build
// it uses the section as the body to subtract the fixings from
////////////////////////////////////////////////////////

module spindle_clamp(   base_height,
                        section_height,
                        section_dia,
                        clamp_split_type,
                        clamp_split_width,
                        fixing_height,
                        fixing_radius,
                        head_offset,
                        nut_offset,
                        rotate_fixings,
                        screw_bore_size,
                        screw_head_size,
                        hex_nut_size
                        )
	{
        rotate([0,0,rotate_fixings])
       { 
        //gap between clamping halves
        translate([clamp_split_type*section_dia/2,0,base_height+(section_height/2)])
        {
        cube([section_dia,clamp_split_width,section_height],true);
        } 

        //bores for the screw thread
        if (clamp_split_type==0) //diameter split use 2 fixings
        {
            for(i=[1:2])
            rotate([(i*180),90,90])
            translate([-base_height-fixing_height,fixing_radius,-section_dia/2])
            cylinder(section_dia,d1=ThreadToClearanceHole(screw_bore_size),d2=ThreadToClearanceHole(screw_bore_size),true); 
        
            //bores for the screw heads
            for(i=[1:2])
            rotate([(i*180),90,90])
            translate([-base_height-fixing_height,fixing_radius,head_offset])
            cylinder((section_dia/2),d1=ThreadToCapHeadDiameter(screw_head_size),d2=    ThreadToCapHeadDiameter(screw_head_size),true); 

            //bores for the fixing nuts
            for(i=[1:2])
            rotate([(i*180),270,90])
            translate([base_height+fixing_height,fixing_radius,nut_offset])
            cylinder((section_dia/2),d1=(ThreadToHexHeadAcrossFlats(hex_nut_size)/cos(30)),d2=(ThreadToHexHeadAcrossFlats(hex_nut_size)/cos(30)),$fn=6);
        }
        else    //radius split use 1 fixing
        {
            rotate([180,90,90])
            translate([-base_height-fixing_height,fixing_radius,-section_dia/2])
            cylinder(section_dia,d1=ThreadToClearanceHole(screw_bore_size),d2=ThreadToClearanceHole(screw_bore_size),true); 
        
            //bores for the screw heads
            rotate([180,90,90])
            translate([-base_height-fixing_height,fixing_radius,head_offset])
            cylinder((section_dia/2),d1=ThreadToCapHeadDiameter(screw_head_size),d2=    ThreadToCapHeadDiameter(screw_head_size),true); 

            //bores for the fixing nuts
            rotate([180,270,90])
            translate([base_height+fixing_height,fixing_radius,nut_offset])
            cylinder((section_dia/2),d1=(ThreadToHexHeadAcrossFlats(hex_nut_size)/cos(30)),d2=(ThreadToHexHeadAcrossFlats(hex_nut_size)/cos(30)),$fn=6);
        }

	}
}   


////////////////////////////////////////////////////////////////////////////
// module to insert grub screw fixing holes for radial shaft location
////////////////////////////////////////////////////////

module radial_fixing(   base_height,
                        section_diameter)
	{
        translate([0,0,base_height+radial_fixing_centre_height])
        for(i=[0:radial_fixing_qty-1])
        {
            rotate([0,90,i*(360/radial_fixing_qty)+radial_fixing_hole_offset])
            {
                // add the screw bore hole
                cylinder(section_diameter/2,d1=ThreadToClearanceHole(radial_fixing_diameter),d2=ThreadToClearanceHole(radial_fixing_diameter));
                    
                // add the screw head counterbore
                if (radial_cbore_exists == 1)
                {
                    translate([0,0,section_diameter/2-radial_cbore_depth])
                    cylinder(radial_cbore_depth,d1=ThreadToCapHeadDiameter(radial_fixing_diameter),d2=ThreadToCapHeadDiameter(radial_fixing_diameter));
                }
                
            }   
        }//end loop
        
   translate([0,0,0])
        for(i=[0:radial_fixing_qty-1])
        {
            rotate([0,0,i*(360/radial_fixing_qty)+radial_fixing_hole_offset])
            {

   // add the nut locating slot
                if (radial_fixing_nut_exists == 1)
                {
                    rotate([0,0,0])
                    translate([radial_fixing_nut_from_centre,
                                -ThreadToHexHeadAcrossFlats(radial_fixing_diameter)/2,
                                0])
                    
                    cube([NutThickness(radial_fixing_diameter),ThreadToHexHeadAcrossFlats(radial_fixing_diameter),radial_fixing_centre_height+(HexAcrossFlatsToPointsDia(radial_fixing_diameter)/2)],false);    
                }    
 
                
            }   
        }//end loop 
        
        
    } //end module
                    
////////////////////////////////////////////////////////////////////////////
//module to insert through pulley fixings on PCD 
////////////////////////////////////////////////////////
    
module axial_fixing(base_height,
                    sect_height,
                    axial_fixing_pcd,
                    axial_fixing_qty,
                    axial_metric_thread_size,
                    axial_fixing_depth,
                    axial_fixing_offset,
                    axial_cbore_exists,
                    axial_cbore_depth                  
                    )
{
    for(i=[1:axial_fixing_qty])
    {
        rotate([0,0,i*(360/axial_fixing_qty)+axial_fixing_offset])
        {
            // add the screw bore hole
            translate([0,axial_fixing_pcd/2,base_height])
            cylinder(axial_fixing_depth,d1=ThreadToClearanceHole(axial_metric_thread_size),d2=ThreadToClearanceHole(axial_metric_thread_size));
            
            // add the screw head counterbore
            if (axial_cbore_exists == 1)
            {
                translate([0,axial_fixing_pcd/2,base_height+sect_height-axial_cbore_depth])
                cylinder(axial_cbore_depth,d1=ThreadToCapHeadDiameter(axial_metric_thread_size),d2=ThreadToCapHeadDiameter(axial_metric_thread_size));
            }
        }
    }
} 
 
////////////////////////////////////////////////////////////////////////////
//module to insert through pulley fixings on square pitch
////////////////////////////////////////////////////////
    
module axial_fixing_square(base_height,
                            sect_height,
                            axial_fixing_pcd,
                            axial_fixing_qty,
                            axial_metric_thread_size,
                            axial_fixing_depth,
                            axial_fixing_offset,
                            axial_cbore_exists,
                            axial_cbore_depth)
{
        square_PCD=(axial_fixing_pcd/2)/sin(45)/2;
    
    for(i=[1:4])
    {
        rotate([0,0,i*(360/4)+axial_fixing_offset])
        {
            // add the screw bore hole
            translate([0,square_PCD,base_height])
            cylinder(axial_fixing_depth,d1=ThreadToClearanceHole(axial_metric_thread_size),d2=ThreadToClearanceHole(axial_metric_thread_size));
            
            // add the screw head counterbore
            if (axial_cbore_exists == 1)
            {
                translate([0,square_PCD,base_height+sect_height-axial_cbore_depth])
                cylinder(axial_cbore_depth,d1=ThreadToCapHeadDiameter(axial_metric_thread_size),d2=ThreadToCapHeadDiameter(axial_metric_thread_size));
            }
        }
    }
} 
 

////////////////////////////////////////////////////////////////////////////
//module to insert negative internal keyways on bore
////////////////////////////////////////////////////////

module internal_key_sub( bore_dia,
                         section_length,
                         number_of_int_keyway,
                         internal_key_offset,
                         internal_key_width,
                         internal_key_height)
	{
        
        for(i=[1:number_of_int_keyway])
            
            rotate([0,0,i*(360/number_of_int_keyway)+internal_key_offset])
            translate([0,(bore_dia/2)+(internal_key_height/2)-1,section_length/2])
            cube([internal_key_width,internal_key_height,section_length],true);  
	}
    
////////////////////////////////////////////////////////////////////////////
//module to insert negative ezternal keyways on section
////////////////////////////////////////////////////////
 

module external_key_sub( body_dia,
                         section_length,
                         number_of_ext_keyway,
                         external_key_offset,
                         external_key_width,
                         external_key_height)
	{
        
        for(i=[1:number_of_ext_keyway])
            
            rotate([0,0,i*(360/number_of_ext_keyway)+external_key_offset])
            translate([0,(body_dia/2)-(external_key_height/2),section_length/2])
            cube([external_key_width,external_key_height+0.1,section_length],true);  
	}    

////////////////////////////////////////////////////////////////////////////
//module to insert negative internal splines on bore
////////////////////////////////////////////////////////
    
module internal_spline_sub( bore_dia,
                            section_length,
                            number_of_int_splines,
                            internal_spline_offset,
                            internal_spline_width,
                            internal_spline_depth)
	{
        
        for(i=[1:number_of_int_splines])
            
            rotate([0,0,i*(360/number_of_int_splines)+internal_spline_offset])   
            translate([0,(bore_dia/2)+(internal_spline_depth/2)-1,external_spline_length/2])
            cube([internal_spline_width,internal_spline_depth,external_spline_length],true);     
        
        
	}
    
    
////////////////////////////////////////////////////////////////////////////
//module to insert positive internal splines on bore
////////////////////////////////////////////////////////
    
module internal_spline_add( bore_dia,
                            section_length,
                            number_of_int_splines,
                            internal_spline_offset,
                            internal_spline_width,
                            internal_spline_depth)
	{

    for(i=[1:number_of_int_splines])
            rotate([0,0,i*(360/number_of_int_splines)+internal_spline_offset])   
            translate([0,(bore_dia/2-(internal_spline_depth/2)),section_length/2])
            cube([internal_spline_width,internal_spline_depth,section_length],true);
        
	}       
  
////////////////////////////////////////////////////////////////////////////
//module to insert negative external splines on body
////////////////////////////////////////////////////////
    
module external_spline_sub( body_dia,
                            section_length,
                            number_of_ext_splines,
                            ext_spline_offset,
                            ext_spline_width,
                            ext_spline_depth)
	{
    for(i=[1:number_of_ext_splines])
            
            rotate([0,0,i*(360/number_of_ext_splines)+ext_spline_offset])   
            translate([0,(body_dia/2)-(ext_spline_depth/2),external_spline_length/2])
            cube([ext_spline_width,ext_spline_depth+0.1,external_spline_length],true);     
        
	}   
    
////////////////////////////////////////////////////////////////////////////
//module to insert positive external splines on body
////////////////////////////////////////////////////////
    
module external_spline_add( body_dia,
                            section_length,
                            number_of_ext_splines,
                            ext_spline_offset,
                            ext_spline_width,
                            ext_spline_depth)
	{
    for(i=[1:number_of_ext_splines])
            //translate slightly shortened because cube sits flat on circumferance
            //ie its 2.5 instead of 2
            rotate([0,0,i*(360/number_of_ext_splines)+ext_spline_offset])   
            translate([0,(body_dia/2+(ext_spline_depth/2.5)),section_length/2])
            cube([ext_spline_width,ext_spline_depth,section_length],true);
	}       
  
    
////////////////////////////////////////////////////////////////////////////
//module to draw the main body of a section
//provides the height dependant on section, chamfers and radius selected
////////////////////////////////////////////////////////

module draw_body_middle(    section_identity,
                            body_exist,
                            base_height,
                            section_height,
                            section_diameter,
    
                            top_cham_exists,
                            top_cham_1_top_thick,
                            top_cham_1_top_dia,
                            top_cham_1_bot_dia,
                            top_cham_2_bot_thick,
                            top_cham_2_top_dia,
                            top_cham_2_bot_dia,
    
                            bot_cham_exists,
                            bot_cham_1_top_thick,
                            bot_cham_1_top_dia,
                            bot_cham_1_bot_dia,
                            bot_cham_2_bot_thick,
                            bot_cham_2_top_dia,
                            bot_cham_2_bot_dia,
                            
                            
                            top_rad_exists,
                            top_rad_top_thick,
                            top_rad_1_major_dia,
                            top_rad_bot_thick,
                            top_rad_2_major_dia,
                            
                            bot_rad_exists,
                            bot_rad_top_thick,
                            bot_rad_1_major_dia,
                            bot_rad_bot_thick,
                            bot_rad_2_major_dia,

                            
                            top_inner_radius_exists,
                            section_with_top_inner_radius,
                            top_inner_radius_size,
                            bot_inner_radius_exists,
                            section_with_bot_inner_radius,
                            bot_inner_radius_size,
                        )
 {
     
    if ((body_exist==1)&&(top_cham_exists !=1)&&(bot_cham_exists!=1)&&(top_rad_exists !=1)&&(bot_rad_exists!=1)                            )                
        {
        //1 draw core body
        //body only
            translate([0,0,base_height])
            cylinder(section_height, d1=section_diameter, d2=section_diameter, center=false);
   
        }
              
    else if ((body_exist==1)&&(top_cham_exists ==1)&&(bot_cham_exists!=1)&&(top_rad_exists !=1)&&(bot_rad_exists!=1))
        {
        //2 draw core body
        //top chamfer + body
         
        //body    
            translate([0,0,base_height])
            cylinder((section_height - top_cham_1_top_thick - top_cham_2_bot_thick), d1=section_diameter, d2=section_diameter, center=false);                   
            
        }
        
    else if ((body_exist==1)&&(top_cham_exists !=1)&&(bot_cham_exists==1)&&(top_rad_exists !=1)&&(bot_rad_exists!=1))
        {
        //3 draw core body
        //body + bottom chamfer
            translate([0,0,base_height+bot_cham_1_thick+bot_cham_2_thick])
            cylinder((section_height-bot_cham_1_thick-bot_cham_2_thick), d1=section_diameter, d2=section_diameter, center=false);                
                                                         
        }
        
    else if ((body_exist==1)&&(top_cham_exists ==1)&&(bot_cham_exists==1)&&(top_rad_exists !=1)&&(bot_rad_exists!=1))
        {
        //4 draw core body
        //top chamfer + body + bottom chamfer
            translate([0,0,base_height+bot_cham_1_top_thick+bot_cham_2_bot_thick])
            cylinder((section_height- top_cham_1_top_thick - top_cham_2_bot_thick - bot_cham_1_top_thick - bot_cham_2_bot_thick),
                    d1=section_diameter, d2=section_diameter, center=false);
        }
        
    else if ((body_exist==1)&&(top_cham_exists !=1)&&(bot_cham_exists!=1)&&(top_rad_exists ==1)&&(bot_rad_exists!=1))
        {
        //5 draw core body
        //top radius + body
            translate([0,0,base_height])
            cylinder((section_height-top_rad_top_thick-top_rad_bot_thick),
                    d1=section_diameter, d2=section_diameter, center=false);
        }       
        
    else if ((body_exist==1)&&(top_cham_exists !=1)&&(bot_cham_exists==1)&&(top_rad_exists ==1)&&(bot_rad_exists!=1))
        {
        //6 draw core body
        //top radius + body + bottom chamfer
            translate([0,0,base_height+bot_cham_1_thick+bot_cham_2_thick])
            cylinder((section_height-top_rad_top_thick-top_rad_bot_thick-bot_cham_1_top_thick-bot_cham_2_bot_thick),
                    d1=section_diameter, d2=section_diameter, center=false);
        }    
        
        else if ((body_exist==1)&&(top_cham_exists !=1)&&(bot_cham_exists!=1)&&(top_rad_exists !=1)&&(bot_rad_exists==1))
        {
        //7 draw core body
        //body + bottom radius
            translate([0,0,base_height+bot_rad_top_thick+bot_rad_bot_thick])
            cylinder((section_height-bot_rad_top_thick-bot_rad_bot_thick),
                    d1=section_diameter, d2=section_diameter, center=false);
        }
        
        else if ((body_exist==1)&&(top_cham_exists ==1)&&(bot_cham_exists!=1)&&(top_rad_exists !=1)&&(bot_rad_exists==1))
        {
        //8 draw core body
        //top chamfer + body + bottom radius
            translate([0,0,base_height+bot_rad_top_thick+bot_rad_bot_thick])
            cylinder((section_height-top_cham_1_thick-top_cham_2_thick-bot_rad_top_thick-bot_rad_bot_thick),
                    d1=section_diameter, d2=section_diameter, center=false);
        }
        
        else if ((body_exist==1)&&(top_cham_exists !=1)&&(bot_cham_exists!=1)&&(top_rad_exists ==1)&&(bot_rad_exists==1))
        {
        //9 draw core body
        //top radius + body + bottom radius
            translate([0,0,base_height+bot_rad_top_thick+bot_rad_bot_thick])
            cylinder((section_height-top_rad_top_thick-top_rad_bot_thick-bot_rad_top_thick-bot_rad_bot_thick),
                    d1=section_diameter, d2=section_diameter, center=false);
        }
        
        else
            echo("SECTION",section_identity,"CIRCULAR BODY BUILD ERROR");
              
}

///////////////////////////////////////////////////////////
//module for adding axial graduations    
////////////////////////////////////////////////////////
        
module axial_ticks(base_height,
                   section_dia,
                   section_height)
{
    //large graduations
    if(scale_type_ax==1)
    {  
        for (i=[0:number_major_ticks_ax-1])
            rotate(360/number_major_ticks_ax*i)
            translate([section_dia/2-major_scale_depth_ax,-major_scale_width_ax/2,base_height+section_height-major_scale_length_ax-major_dist_from_top])
            cube([major_scale_depth_ax,major_scale_width_ax,major_scale_length_ax]);
            }           
           
    if(scale_type_ax==2)
    {
        for (i=[0:number_major_ticks_ax-1])
            rotate(360/number_major_ticks_ax*i)
            translate([section_dia/2,-major_scale_width_ax/2,base_height+section_height-major_scale_length_ax-major_dist_from_top])
            cube([major_scale_depth_ax,major_scale_width_ax,major_scale_length_ax]);
    }

    //medium graduations
            
    if(scale_type_ax==1)
    {        
        for(i=[0:number_major_ticks_ax*medium_ticks_between-1])
            rotate(360/number_major_ticks_ax/medium_ticks_between_ax*i)
            translate([section_dia/2-medium_scale_depth_ax,-medium_scale_width_ax/2,base_height+section_height-medium_scale_length_ax-medium_dist_from_top])
            cube([medium_scale_depth_ax,medium_scale_width_ax,medium_scale_length_ax]);
     }
     
     if(scale_type_ax==2)
     {
         for(i=[0:number_major_ticks_ax*medium_ticks_between_ax-1])
            rotate(360/number_major_ticks_ax/medium_ticks_between*i)
			translate([section_dia/2,-medium_scale_width_ax/2,base_height+section_height-medium_scale_length_ax-medium_dist_from_top])
            cube([medium_scale_depth_ax,medium_scale_width_ax,medium_scale_length_ax]);
     }
     
     //small graduations
            
     if(scale_type_ax==1)
     {
        for (i=[0:number_major_ticks_ax*minor_ticks_between_ax-1])
            rotate(360/number_major_ticks_ax/minor_ticks_between_ax*i)
            translate([section_dia/2-minor_scale_depth_ax,-minor_scale_width_ax/2,base_height+section_height-minor_scale_length_ax-minor_dist_from_top])
            cube([minor_scale_depth_ax,minor_scale_width_ax,minor_scale_length_ax]);
     }
     
     if(scale_type_ax==2)
     {
        for (i=[0:number_major_ticks_ax*minor_ticks_between_ax-1])
            rotate(360/number_major_ticks_ax/minor_ticks_between_ax*i)
            translate([section_dia/2,-minor_scale_width_ax/2,base_height+section_height-minor_scale_length_ax-minor_dist_from_top])
            cube([minor_scale_depth_ax,minor_scale_width_ax,minor_scale_length_ax]);
     }
  
}


///////////////////////////////////////////////////////////
//module for adding the graduations    
////////////////////////////////////////////////////////
   
module radial_ticks(base_height,
                    section_dia,
                    section_height)
{
    //large graduations
    if(scale_type==1)
    {  
        for (i=[0:number_major_ticks-1])
            rotate(360/number_major_ticks*i)
            translate([section_dia/2-major_scale_length-major_dist_from_edge,-major_scale_width/2,base_height+section_height-major_scale_depth])
            cube([major_scale_length,major_scale_width,major_scale_depth]);
    }           
           
    if(scale_type==2)
    {
        for (i=[0:number_major_ticks-1])
            rotate(360/number_major_ticks*i)
            translate([section_dia/2-major_scale_length-major_dist_from_edge,-major_scale_width/2,base_height+section_height])
            cube([major_scale_length,major_scale_width,major_scale_depth]);
    }

    //medium graduations
            
    if(scale_type==1)
    {        
        for(i=[0:number_major_ticks*medium_ticks_between-1])
            rotate(360/number_major_ticks/medium_ticks_between*i)
            translate([section_dia/2-medium_scale_length-medium_dist_from_edge,-medium_scale_width/2,base_height+section_height-medium_scale_depth])
            cube([medium_scale_length,medium_scale_width,medium_scale_depth]);
     }
     
     if(scale_type==2)
     {
         for(i=[0:number_major_ticks*medium_ticks_between-1])
            rotate(360/number_major_ticks/medium_ticks_between*i)
			translate([section_dia/2-medium_scale_length-medium_dist_from_edge,-medium_scale_width/2,base_height+section_height])
            cube([medium_scale_length,medium_scale_width,medium_scale_depth]);
     }
     
     //small graduations
            
     if(scale_type==1)
     {
        for (i=[0:number_major_ticks*minor_ticks_between-1])
            rotate(360/number_major_ticks/minor_ticks_between*i)
            translate([section_dia/2-minor_scale_length-minor_dist_from_edge,-minor_scale_width/2,base_height+section_height-minor_scale_depth])
            cube([minor_scale_length,minor_scale_width,minor_scale_depth]);
     }
     
     if(scale_type==2)
     {
        for (i=[0:number_major_ticks*minor_ticks_between-1])
            rotate(360/number_major_ticks/minor_ticks_between*i)
            translate([section_dia/2-minor_scale_length-minor_dist_from_edge,-minor_scale_width/2,base_height+section_height])
            cube([minor_scale_length,minor_scale_width,minor_scale_depth]);
     }
  
}

///////////////////////////////////////////////////////////
//module for adding axial graduations    
////////////////////////////////////////////////////////
        
module generate_cams(section_base_height,
                     section_diameter,
                     switch_cams_type,
                     switch_cams_qty,
                     switch_cams_spacing,
                     switch_cams_width,
                     switch_cams_height,
                     switch_cams_major_dia,
                     switch_cams_neg_cam_dia,
                     switch_cams_rot_offset,
                     switch_cams_base)
{
    
    // positive cams triangular polygon having its points on circumferance
    polygon_base_height = sqrt((section_diameter/2*section_diameter/2) - 
                               (switch_cams_width/2*switch_cams_width/2));
   
   cam_points = [ [0,polygon_base_height], //centre base point
                  [switch_cams_width/2,polygon_base_height], //rhs base
                  [0,switch_cams_major_dia/2], //centre top
                  [-switch_cams_width/2,polygon_base_height] ];
    
    if(switch_cams_type==1)// positive cams
    {
        translate ([0,0,section_base_height+switch_cams_base])
        for (i=[0:switch_cams_qty-1])
        {
            rotate((switch_cams_spacing*i)+switch_cams_rot_offset)
            linear_extrude(switch_cams_height)  
            polygon (cam_points);  
        }
    }           
           
    else // negitive cams
    {
        translate ([0,0,section_base_height+switch_cams_base])
        for (i=[0:switch_cams_qty-1])
            rotate((switch_cams_spacing*i)+switch_cams_rot_offset)
            translate([switch_cams_neg_cam_dia/2,0,0])
            cylinder(d=switch_cams_width,h=switch_cams_height);
    }
}
    
///////////////////////////////////////////////////////////
//module for adding text on a axial surface
///////////////////////////////////////////////////////////    
        
module add_axial_text  (base_height,
                        body_diameter,
                        text_type,
                        text_height,
                        text_height_trim,
                        text_to_print,
                        number_of_strings,
                        text_within_angle,
                        text_direction,
                        distance_from_base,
                        character_height,
                        text_rotation_angle,
                        initial_twist,
                        font)
{
    if(axial_text_type==1)//protrude
    {
        //align text with polygon faces   
        rotate(text_direction*(text_within_angle/number_of_strings/2))
            { 
                for    (i=[0:number_of_strings-1])
                    rotate((text_direction*(text_within_angle/number_of_strings*(i)))+initial_twist) 
                    {
                        translate([body_diameter/2-text_height_trim,0, base_height+distance_from_base])
                        rotate([0,90,0]) 
                        linear_extrude(height = text_height)
                        rotate([0,0,text_rotation_angle]) 
                        text (text_to_print[i],
                        font=font,
                        halign="center",
                        size=character_height);
                    }
            }
    }
       
  if(axial_text_type==2)//sunk in
    {
        //align text with polygon faces   
        rotate(text_direction*(text_within_angle/number_of_strings/2))
        { 
            for    (i=[0:number_of_strings-1])
                    rotate((text_direction*(text_within_angle/number_of_strings*(i)))+initial_twist) 
                    {
                        translate([body_diameter/2-text_height,0, base_height+distance_from_base])
                        rotate([0,90,0]) 
                        linear_extrude(height = text_height)                
                        rotate([0,0,text_rotation_angle])                
                        text (text_to_print[i],               
                        font=font,
                        halign="center",
                        size=character_height);
                    }
        }
    }
}

///////////////////////////////////////////////////////////
//module for adding text on a radial surface
///////////////////////////////////////////////////////////    
       
module add_radial_text( base_height, 
                        body_height,
                        text_type,
                        text_height,
                        text_to_print,
                        number_of_strings,
                        text_within_angle,
                        text_direction,
                        distance_from_centre,
                        character_height,
                        text_rotation_angle,
                        initial_twist,
                        font)
{
    if(text_type==1)//protrude
    { 
     for    (i=[0:number_of_strings-1])
            rotate((text_direction*(text_within_angle/number_of_strings*(i)))+initial_twist) 
            {
                translate([distance_from_centre,0, base_height+body_height])
                linear_extrude(height = text_height)
                rotate([0,0,text_rotation_angle]) 
                text (text_to_print[i],
                font=font,
                halign="center",
                size=character_height);
                
        }
    }
       
  if(text_type==2)//sunk in
    { 
     for    (i=[0:number_of_strings-1])
            rotate((text_direction*(text_within_angle/number_of_strings*(i)))+initial_twist) 
            {
                translate([distance_from_centre,0, base_height+body_height-text_height])
                linear_extrude(height = text_height)
                rotate([0,0,text_rotation_angle]) 
                text (text_to_print[i],
                font=font,
                halign="center",
                size=character_height);
            }
    }
}

////////////////////////////////////////////////////////
//module for adding an internal circular groove inside body
////////////////////////////////////////////////////////

module circ_internal_groove(base_height,
                            body_height,
                            body_diameter,
                            cylinder_bore
                            )
{
            //take the bits away
            for (i=[0:number_of_circ_grooves-1])
            {   
            //move the groove up in height    
            translate([0,0,base_height+circ_groove_from_base-circ_groove_diameter/2
                        + distance_between_circ_groove*i])
            {
            //extrude the ring
            rotate_extrude()
            //move the circle from the centre
            translate([circ_groove_from_centre,circ_groove_diameter/2])
            //draw a 2D circle on centre axis
            circle(circ_groove_diameter/2);
             
            //fill the doughnut
            translate([0,0,base_height+circ_groove_from_base])
            cylinder(circ_groove_diameter-circ_groove_diameter,circ_groove_from_centre,circ_groove_from_centre);
            }
        }
}

////////////////////////////////////////////////////////
//module for adding an internal rectangular groove inside body
////////////////////////////////////////////////////////


module rect_internal_groove(base_height,
                            body_height,
                            body_diameter,
                            cylinder_bore
                            )
{
            //take the bits away
    
    
           for (i=[0:number_of_rect_grooves-1])
            {   
            //move the groove up in height    
            translate([0,0,base_height+rect_groove_from_base-rect_groove_width/2
                        + distance_between_rect_groove*i])

            //extrude the ring
            rotate_extrude()
            //move the rectangle from the centre
            translate([rect_groove_from_centre,0,0])
            //draw a 2D rectangular on centre axis
            square([rect_groove_depth,rect_groove_width]);
            }   
             
}

////////////////////////////////////////////////////////
//module for adding solid spokes to the body
////////////////////////////////////////////////////////

module generate_spokes()
{
               for (i=[0:number_of_spoke_bars-1])
               {
                   //spokes are round
                   if (spoke_bar_shape == 0)
                   {
                    rotate(total_spoke_bar_angle/number_of_spoke_bars*i +initial_spoke_bar_offset)
                    translate([spoke_bar_dist_from_centre,0,0])
                    rotate ([0,90-spoke_bar_tilt_up_angle,0])
                    cylinder(h=spoke_bar_length, d=spoke_bar_dia);
                   }
                   //spokes are rectangular
                   else if (spoke_bar_shape ==1)
                   {
                    rotate(total_spoke_bar_angle/number_of_spoke_bars*i +initial_spoke_bar_offset)
                    translate([spoke_bar_dist_from_centre,0,spoke_bar_height/2])
                    rotate ([0,90-spoke_bar_tilt_up_angle,0])
                    translate([0,0,spoke_bar_length/2])
                    cube([spoke_bar_height,spoke_bar_width,spoke_bar_length], center=true);
                   }
                }
}

////////////////////////////////////////////////////////
//module for adding slotted holes into the body
////////////////////////////////////////////////////////


module circular_slots(  number_of_slots,
                        total_slots_angle,
                        initial_slot_offset,
                        individual_slot_angle,
                        slot_diameter,
                        slot_distance_from_centre)
    
{
            
               for (i=[0:number_of_slots-1])
               {
                    //insert one hole to create slot start curve
                    rotate(total_slots_angle/number_of_slots*i +initial_slot_offset)
                    translate([slot_distance_from_centre,0,0])
                    cylinder(max_body_height,d1=slot_diameter,d2=slot_diameter);
                    //insert another hole to create slot end curve
                    rotate((total_slots_angle/number_of_slots*i)+individual_slot_angle+initial_slot_offset)
                    translate([slot_distance_from_centre,0,0])
                    cylinder(max_body_height,d1=slot_diameter,d2=slot_diameter);
                    //insert the slot between the two holes
                    //rotate to the next slot
                    rotate(total_slots_angle/number_of_slots*i+initial_slot_offset)
                    //extrude the ring
                    rotate_extrude(angle=individual_slot_angle)
                    //move the circle from the centre
                    translate([slot_distance_from_centre-slot_diameter/2,0,0])
                    //draw a 2D rectangular on centre axis
                    square([slot_diameter,max_body_height]);
            
                }
        
}

////////////////////////////////////////////////////////
//module for adding spokes into the body
////////////////////////////////////////////////////////

module circular_spokes(     number_of_spoke,
                            total_spoke_angle,
                            initial_spoke_offset,
                            individual_spoke_angle,
                            spoke_inner_corner_radius,
                            spoke_outer_corner_radius,
                            spoke_inner_distance_from_centre,
                            spoke_outer_distance_from_centre
                            )
    
{
            
               for (i=[0:number_of_spoke-1])
               {
                   
                   hull()
                    {
                    //insert inside first cylinder
                    rotate(total_spoke_angle/number_of_spoke*i +initial_spoke_offset)
                    translate([spoke_inner_distance_from_centre,0,0])
                    cylinder(max_body_height,r1=spoke_inner_corner_radius,r2=spoke_inner_corner_radius);
                   
                    //insert inside second cylinder
                    rotate((total_spoke_angle/number_of_spoke*i)+individual_spoke_angle+initial_spoke_offset)
                    translate([spoke_inner_distance_from_centre,0,0])
                    cylinder(max_body_height,r1=spoke_inner_corner_radius,r2=spoke_inner_corner_radius);
                        
                    //round off the inner radius
                    //rotate to the next slot
                    //rotate(total_spoke_angle/number_of_spoke*i+initial_spoke_offset)
                    //extrude the ring
                    //rotate_extrude(angle=individual_spoke_angle)
                    //move the circle from the centre
                    //translate([spoke_inner_distance_from_centre,0,0])
                    //draw a 2D rectangular on centre axis
                    //square([spoke_inner_corner_radius*2,spoke_depth]);    
                        
                   
                   //insert inside first cylinder
                    rotate(total_spoke_angle/number_of_spoke*i +initial_spoke_offset)
                    translate([spoke_outer_distance_from_centre,0,0])
                    cylinder(max_body_height,r1=spoke_outer_corner_radius,r2=spoke_outer_corner_radius);
                   
                    //insert inside second cylinder
                    rotate((total_spoke_angle/number_of_spoke*i)+individual_spoke_angle+initial_spoke_offset)
                    translate([spoke_outer_distance_from_centre,0,0])
                    cylinder(max_body_height,r1=spoke_outer_corner_radius,r2=spoke_outer_corner_radius);
                    
            
                    //round off the outer radius
                    //rotate to the next slot
                    rotate(total_spoke_angle/number_of_spoke*i+initial_spoke_offset)
                    //extrude the ring
                    rotate_extrude(angle=individual_spoke_angle)
                    //move the circle from the centre
                    translate([spoke_outer_distance_from_centre-spoke_outer_corner_radius,0,0])
                    //draw a 2D rectangular on centre axis
                    square([spoke_outer_corner_radius*2,max_body_height]);
            
                    }
                }
}

////////////////////////////////////////////////////////
//module for adding weight reduction
////////////////////////////////////////////////////////

module weight_reduction( body_height,
                         body_diameter  
                       )
{
 //calculations

// centre of weight saving area
PCD_weight_saving =(circle_inner_limit+((circle_outer_limit-circle_inner_limit)/2));
echo ("PCD OF CUT OUTS",PCD_weight_saving);

// size of weight saving cut out
cut_out_circ_dia =((circle_outer_limit-circle_inner_limit)/2)*0.8;
echo ("CUT OUT DIA",cut_out_circ_dia);

// circumferance of weight saving PCD
circumfrance_of_PCD = (PCD_weight_saving * 3.1416);
echo ("CIRCUMFERANCE OF PCD",circumfrance_of_PCD);

//calculates distance between cut outs
space_between_circs=cut_out_circ_dia*weigh_saving_ratio;
echo ("BETWEEN CIRCS SPACE",space_between_circs);

// rounded number of holes
number_of_holes = floor(circumfrance_of_PCD/(space_between_circs+cut_out_circ_dia));
echo ("NUMBER HOLES",number_of_holes);

//calculates depth of recesses
recess_depth = (body_height*(cut_area_reduction/100))/2;
echo ("REDUCTION ON RECESS",recess_depth);   
    
    difference()
    {
        union()
        {
            //remove the weight saving holes
            
            for (i=[1:number_of_holes])
            rotate(360/number_of_holes*i)
            {
                translate([PCD_weight_saving/2,0,0])
                cylinder(body_height,d1=cut_out_circ_dia,d2=cut_out_circ_dia);    
            }
               
            //remove the weight saving recesses
            //lower
            translate([0,0,-0.01])
            cylinder(recess_depth,d1=circle_outer_limit,d2=circle_outer_limit);
            //upper
            translate([0,0,body_height-recess_depth])
            cylinder(recess_depth+0.01,d1=circle_outer_limit,d2=circle_outer_limit);
               
        }
            //add the centres back in
            cylinder(recess_depth,d1=circle_inner_limit,d2=circle_inner_limit);
        
            translate([0,0,body_height-recess_depth])
            cylinder(recess_depth,d1=circle_inner_limit,d2=circle_inner_limit);         
    }      
}
   

////////////////////////////////////////////////////////
//module for adding knurling to outside face
////////////////////////////////////////////////////////

module knurl_outside_surface(   body_diameter,
                                number_knurl_starts,
                                total_knurl_angle,
                                knurling_depth,
                                knurling_width,
                                knurling_base_height)
    
    {
            
            //remove the knurling
            
               for (i=[0:number_knurl_starts-1])
               {
                    //left hand flute
                    translate([0,0,knurling_base_height])
                    linear_extrude(height = knurling_width, twist =total_knurl_angle )
                    rotate(360/number_knurl_starts*i)
                    translate([body_diameter/2-knurling_depth,0,0])
                    rotate(-45)
                    square(10);
                    //right hand flute
                    translate([0,0,knurling_base_height])
                    linear_extrude(height = knurling_width, twist =-total_knurl_angle )
                    rotate(360/number_knurl_starts*i)
                    translate([body_diameter/2-knurling_depth,0,0])
                    rotate(-45)
                    square(10);
                }
    }

////////////////////////////////////////////////////////
//module for adding allen key hole in base
////////////////////////////////////////////////////////

module allen_key_in_base(   )
    {
        linear_extrude(allen_key_depth)
        circle(d=(ThreadToAllenHeadAcrossFlats(allen_key_bolt_size)/cos(30)),$fn=6);     

    }

////////////////////////////////////////////////////////
//module for adding pozi drive hole in base
////////////////////////////////////////////////////////
    
module pozi_drive(  )
    
    {
                
        for (i=[0:4-1])
        {
            rotate(360/4*i)
             
            // move triangle slot into position
            translate([-pozi_slot_width/2,pozi_slot_width/2,pozi_slot_length/sin(26.5)-0.01])
            //turn the triangle slot upright
            rotate([0,90,0])
            //generate a pozi triangle
            linear_extrude(pozi_slot_width)
                   
            polygon(points=[[pozi_slot_length/sin(26.5)-pozi_slot_depth,0],
                            [(pozi_slot_length/sin(26.5)),0],
                            [(pozi_slot_length/sin(26.5)),pozi_slot_length],
                            [pozi_slot_length/sin(26.5)-pozi_slot_depth,
                            (tan(26.5)*((pozi_slot_length/sin(26.5))-pozi_slot_depth))]
                            ]  );            
        }
        translate([-pozi_slot_width/2,-pozi_slot_width/2,0])
        cube([pozi_slot_width,pozi_slot_width,pozi_slot_depth],center=false);
     }
     
////////////////////////////////////////////////////////
//module for adding screwdriver slot in base 
////////////////////////////////////////////////////////
     
module screw_drive( )
    
    {
                
        for (i=[0:number_of_slots-1])
        {
            rotate(360/number_of_slots*i)
            translate([0,0,screw_slot_depth/2])
            cube([screw_slot_width,screw_slot_length,screw_slot_depth],center=true);                      
        }

    }
    
////////////////////////////////////////////////////////
//module for adding triangular drive in base     
////////////////////////////////////////////////////////
    
module triangular_drive( )
    {
        linear_extrude(triangle_slot_depth)
        circle(d=triangle_slot_points_dia,$fn=3);
    }
 
////////////////////////////////////////////////////////
//module for adding square drive in base     
////////////////////////////////////////////////////////
 
 module square_drive ()
    {
        translate([0,0,square_slot_depth/2])
        cube([square_slot_width,square_slot_width,square_slot_depth],center=true);          
    }
    
////////////////////////////////////////////////////////
//module for adding torx drive in base     
////////////////////////////////////////////////////////
    
//small gap between start and end of radius because of construction method ??

module torx_drive()
{
    difference()
    {
        union()
        {
        //remove the shape to create the big radius   
        for (i=[0:6-1])
            {
                rotate(360/6*i)   
                hull()
                {   
                translate([((TorxDiameter(torx_size)*0.72)/2)+((TorxDiameter(torx_size)*0.175)),0,0])
                cylinder(torx_depth,r=TorxDiameter(torx_size)*0.175);
                cylinder(torx_depth,r=0.1);   
                }
            }
        //remove shape with small radius    
        for (i=[0:6-1])
            {
                rotate(360/6*i+30)
                hull()
                {
                cylinder(torx_depth,r=TorxDiameter(torx_size)*0.175);
                translate([((TorxDiameter(torx_size)/2)-(TorxDiameter(torx_size)*0.1)),0,0])
                cylinder(torx_depth,r=TorxDiameter(torx_size)*0.1);
                }    
            }
        }
        //add cylinder back in for big radius
        for (i=[0:6-1])
        {
            rotate(360/6*i)   
            translate([((TorxDiameter(torx_size)*0.72)/2)+((TorxDiameter(torx_size)*0.175)),0,0])
            cylinder(torx_depth,r=TorxDiameter(torx_size)*0.175);
        } 
    }
}    
         
    
////////////////////////////////////////////////////////
//module for adding nut housing in base
////////////////////////////////////////////////////////

module nut_housing_in_base()
    {
        translate([0,0,0])  
        linear_extrude(nut_housing_depth)
        circle(d=(ThreadToHexHeadAcrossFlats(nut_thread_size)/cos(30)),$fn=6);     

    }    

//////////////////////////////////////////////////////////////////
// makes holes in rectangular, circular or single positions
//////////////////////////////////////////////////////////////////
module holes_from_array ( first_hole,
                          last_hole)
{
            for (i=[first_hole:last_hole])
            {//start index
                if (hole [i][4] == 1) //rectangular array exists
                {
                    translate ([ hole[i][25],hole[i][26],hole[i][27] ])
                    rotate    ([ hole[i][28],hole[i][29],hole[i][30] ])
                    make_rect_array_holes ( hole[i][4],    //rect array exist
                                            hole[i][5],    //rect array x qty
                                            hole[i][6],    //rect array x pitch
                                            hole[i][7],    //rect array y qty
                                            hole[i][8],    //rect array y pitch
                                            hole[i][9],    //rect array outside only   
                                            hole[i][0],    //thread diameter
                                            hole[i][1],    //hole length
                                            hole[i][2],    //hole exists
                                            hole[i][3],    //actual or screw size
                                            hole[i][13],   //recess exists
                                            hole[i][14],   //recess type
                                            hole[i][15],   //counterbore depth
                                            hole[i][16],   //counterbore height offset
                                            hole[i][17],   //nut retainer exists
                                            hole[i][18],   //nut hole depth
                                            hole[i][19],   //nut hole height offset
                                            hole[i][20],   //thread exists
                                            hole[i][21],   //thread pitch
                                            hole[i][22],   //thread depth
                                            hole[i][23],   //thread angle
                                            hole[i][24] ); //thread height offset)
                    
                }
                
                else if (hole [i][10] == 1) // circular aray exists
                {
                    translate ([ hole[i][25],hole[i][26],hole[i][27] ])
                    rotate    ([ hole[i][28],hole[i][29],hole[i][30] ])
                    make_circ_array_holes(  hole[i][10],   //circ array exist
                                            hole[i][11],   //circ qty of holes
                                            hole[i][12],   //circ PCD of holes
                                            hole[i][0],    //thread diameter
                                            hole[i][1],    //hole length
                                            hole[i][2],    //hole exists
                                            hole[i][3],    //actual or screw size
                                            hole[i][13],   //recess exists
                                            hole[i][14],   //recess type
                                            hole[i][15],   //counterbore depth
                                            hole[i][16],   //counterbore height offset
                                            hole[i][17],   //nut retainer exists
                                            hole[i][18],   //nut hole depth
                                            hole[i][19],   //nut hole height offset
                                            hole[i][20],   //thread exists
                                            hole[i][21],   //thread pitch
                                            hole[i][22],   //thread depth
                                            hole[i][23],   //thread angle
                                            hole[i][24] ); //thread height offset));
                    
                }
                
                else // hole is a single entity
                {
                    translate ([ hole[i][25],hole[i][26],hole[i][27] ])
                    rotate    ([ hole[i][28],hole[i][29],hole[i][30] ])
                    make_single_hole( hole[i][0],    //thread diameter
                                      hole[i][1],    //hole length
                                      hole[i][2],    //hole exists
                                      hole[i][3],    //actual or screw size
                                      hole[i][13],   //recess exists
                                      hole[i][14],   //recess type
                                      hole[i][15],   //counterbore depth
                                      hole[i][16],   //counterbore height offset
                                      hole[i][17],   //nut retainer exists
                                      hole[i][18],   //nut hole depth
                                      hole[i][19],   //nut hole height offset
                                      hole[i][20],   //thread exists
                                      hole[i][21],   //thread pitch
                                      hole[i][22],   //thread depth
                                      hole[i][23],   //thread angle
                                      hole[i][24] ); //thread height offset
                   
                }
                
            }//end index
}

   //////////////////////////////////////////////////////////////////
   // make a rectangular array of holes
   //////////////////////////////////////////////////////////////////
   
    module make_rect_array_holes(rect_exist,    //rect array exist
                                 x_qty,         //rect array x qty
                                 x_pitch,       //rect array x pitch
                                 y_qty,         //rect array y qty
                                 y_pitch,       //rect array y pitch
                                 outer_only,    //rect array outside only   
                                 dia,           //thread diameter
                                 height,        //hole depth
                                 hole_exist,    //hole exists
                                 hole_type,     //actual or screw size
                                 recess_exist,  //recess exists
                                 recess_type,   //recess type
                                 cbore_depth,   //counterbore depth
                                 cbore_offset,  //counterbore height offset
                                 nut_exist,     //nut retainer exists
                                 nut_depth,     //nut hole depth
                                 nut_offset,    //nut hole height offset
                                 thread_exist,  //thread exists
                                 thread_pitch,  //thread pitch
                                 thread_depth,  //thread depth
                                 thread_angle,  //thread angle
                                 thread_offset) //thread height offset
   {
       
    translate ([0,y_pitch/2,0])
    for(y=[0:y_qty-1])
    {
        translate ([0,y_pitch*y-((y_pitch*(y_qty))/2),0])
        rotate ([0,0,0])
        for(x=[0:x_qty-1])
        {
            if (  (outer_only==1)  &&
                     (  
                     ((x==(0))||(x==(x_qty-1)))  ||  ((y==(0))||(y==(y_qty-1)))  
                     ) 
               )
            {
                translate ([x_pitch*x-((x_pitch*(x_qty-1))/2),0,0])
                rotate ([0,0,0])
                make_single_hole(dia,            //thread diameter
                                 height,         //hole height
                                 hole_exist,     //hole exists
                                 hole_type,      //actual or screw size
                                 recess_exist,   //recess exists
                                 recess_type,    //recess type
                                 cbore_depth,    //counterbore depth
                                 cbore_offset,   //counterbore height offset
                                 nut_exist,      //nut retainer exists
                                 nut_depth,      //nut hole depth
                                 nut_offset,     //nut hole height offset
                                 thread_exist,   //thread exists
                                 thread_pitch,   //thread pitch
                                 thread_depth,   //thread depth
                                 thread_angle,   //thread angle
                                 thread_offset); //thread height offset   

            }
            
            else if (outer_only==0)
            {
                translate ([x_pitch*x-((x_pitch*(x_qty-1))/2),0,0])
                rotate ([0,0,0])
                make_single_hole(dia,            //thread diameter
                                 height,         //hole height
                                 hole_exist,     //hole exists
                                 hole_type,      //actual or screw size
                                 recess_exist,   //recess exists
                                 recess_type,    //recess type
                                 cbore_depth,    //counterbore depth
                                 cbore_offset,   //counterbore height offset
                                 nut_exist,      //nut retainer exists
                                 nut_depth,      //nut hole depth
                                 nut_offset,     //nut hole height offset
                                 thread_exist,   //thread exists
                                 thread_pitch,   //thread pitch
                                 thread_depth,   //thread depth
                                 thread_angle,   //thread angle
                                 thread_offset); //thread height offset   
                    
            }
        }   
    }        
}

    
   
   
   //////////////////////////////////////////////////////////////////
   // make a circular array of holes
   //////////////////////////////////////////////////////////////////
   
    module make_circ_array_holes(circ_exist,    //circ array exist
                                 circ_qty,      //circ array qty
                                 pcd,           //circ PCD
                                 dia,           //thread diameter
                                 height,        //hole depth
                                 hole_exist,    //hole exists
                                 hole_type,     //actual or screw size
                                 recess_exist,  //recess exists
                                 recess_type,   //recess type
                                 cbore_depth,   //counterbore depth
                                 cbore_offset,  //counterbore height offset
                                 nut_exist,     //nut retainer exists
                                 nut_depth,     //nut hole depth
                                 nut_offset,    //nut hole height offset
                                 thread_exist,  //thread exists
                                 thread_pitch,  //thread pitch
                                 thread_depth,  //thread depth
                                 thread_angle,  //thread angle
                                 thread_offset) //thread height offset)
   {
    for(i=[0:circ_qty-1])
    {
        rotate ([0,0,i*(360/circ_qty)])
        translate ([pcd/2,0,0])
        make_single_hole(dia,            //thread diameter
                         height,         //hole height
                         hole_exist,     //hole exists
                         hole_type,      //actual or screw size
                         recess_exist,   //recess exists
                         recess_type,    //recess type
                         cbore_depth,    //counterbore depth
                         cbore_offset,   //counterbore height offset
                         nut_exist,      //nut retainer exists
                         nut_depth,      //nut hole depth
                         nut_offset,     //nut hole height offset
                         thread_exist,   //thread exists
                         thread_pitch,   //thread pitch
                         thread_depth,   //thread depth
                         thread_angle,   //thread angle
                         thread_offset); //thread height offset           
  
    }
      
   } 
   
   
   //////////////////////////////////////////////////////////////////
   // make a single hole as defined
   //////////////////////////////////////////////////////////////////
   
    module make_single_hole( dia,           //thread diameter
                             height,        //hole height
                             hole_exist,    //hole exists
                             hole_type,     //actual or screw size
                             recess_exist,  //recess exists
                             recess_type,   //recess type
                             cbore_depth,   //counterbore depth
                             cbore_offset,  //counterbore height offset
                             nut_exist,     //nut retainer exists
                             nut_depth,     //nut hole depth
                             nut_offset,    //nut hole height offset
                             thread_exist,  //thread exists
                             thread_pitch,  //thread pitch
                             thread_depth,  //thread depth
                             thread_angle,  //thread angle
                             thread_offset) //thread height offset
   {
    if (hole_exist == 1)
    {
        if (hole_type == 1) //1=clearance
        {    
            cylinder( d = ThreadToClearanceHole(dia), h = height + 0.01);
        }
        else // 0=actual size
        {
            cylinder( d = dia, h = height + 0.01);
        }
    }
       
    if ( recess_exist==1)
    {
        if (recess_type == 1) //counter bore
        {
        translate ([0,0,height - cbore_depth + cbore_offset ])
        cylinder( d = ThreadToCapHeadDiameter(dia),h = cbore_depth + 0.01);
        }
        else //countersink
        {
            cylinder((dia/2)*tan(45),
                      d1=dia*2,
                      d2=dia);   
        }
    }
    
    if ( nut_exist==1)
    {
        translate ([0,0,nut_offset ])
        cylinder( d = HexAcrossFlatsToPointsDia(dia),h = nut_depth + 0.01,$fn=6);        
        
    }
    
    if ( thread_exist==1)
    {
        translate ([0,0,thread_offset ])
        // ScrewHole(outer_diam, height, position=[0,0,0], rotation=[0,0,0], pitch=0, tooth_angle=30, tolerance=0.4, tooth_height=0)
        ScrewHole(dia, thread_depth, [0, 0, 0], [0,0,0], thread_pitch);   
    }
       
} 
   
////////////////////////////////////////////////////////////////////////////
//function to return torx major diameter from torx size
////////////////////////////////////////////////////////////////////////////

function TorxDiameter(diameter) =
  (diameter <= 100) ?
    lookup(diameter, [
      [6, 1.75],
      [8, 2.4],
      [10, 2.8],
      [15, 3.35],
      [20, 3.95],
      [25, 4.5],
      [30, 5.6],
      [40, 6.75],
      [45, 7.93],
      [50, 8.95],
      [55, 11.35],
      [60, 13.45],
      [70, 15.7],
      [80, 17.75],
      [90, 20.2],
      [100, 22.4]
      ]) : return;
   
/*    
////////////////////////////////////////////////////////////////////////////
//function to return across flats dimention for metric bolts
// Provides standard metric hex head widths across the flats from thread size.

function ThreadToClearanceHole(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.05],
      [2.5, 2.6],
      [3, 3.1],
      [3.5, 3.6],
      [4, 4.1],
      [5, 5.1],
      [6, 6.1],
      [7, 7.2],
      [8, 8.2],
      [10, 10.2],
      [12, 12.2],
      [14, 14.25],
      [16, 16.25],
      [18, 18.25],
      [20, 20.25],
      [22, 22.25],
      [24, 24.25],
      [27, 27.25],
      [30, 30.5],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 64.75]
        ]) : return;
 
*/
////////////////////////////////////////////////////////////////////////////
//function to return across flats dimention for metric bolts
// Provides standard metric hex head widths across the flats from thread size.

function ThreadToHexHeadAcrossFlats(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 4],
      [2.5, 5],
      [3, 5.5],
      [3.5, 6],
      [4, 7],
      [5, 8],
      [6, 10],
      [7, 11],
      [8, 13],
      [10, 16],
      [12, 18],
      [14, 21],
      [16, 24],
      [18, 27],
      [20, 30],
      [22, 34],
      [24, 36],
      [27, 41],
      [30, 46],
      [33, 50],
      [36, 55],
      [39, 60],
      [42, 65],
      [48, 75],
      [52, 80],
      [56, 85],
      [60, 90],
      [64, 95]
        ]) : return;
 

////////////////////////////////////////////////////////////////////////////
//function to return across points dimention for metric bolts
// Provides standard metric hex head widths across points from across flats size.


function HexAcrossFlatsToPointsDia(diameter) =
  ThreadToHexHeadAcrossFlats(diameter) / cos(30);

 
////////////////////////////////////////////////////////////////////////////
//function to return across flats dimention for metric Allen head sockets
// Provides standard metric hex Allen head widths across the flats from thread size.
 
function ThreadToAllenHeadAcrossFlats(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 1.5],
      [2.5, 2],
      [3, 2.5],
      [3.5, 3],
      [4, 3],
      [5, 4],
      [6, 5],
      [7, 5],
      [8, 6],
      [10, 8],
      [12, 10],
      [14, 12],
      [16, 14],
      [18, 15],
      [20, 17],
      [22, 18],
      [24, 19],
      [27, 20],
      [30, 22],
      [33, 24],
      [36, 27],
      [39, 30],
      [42, 32],
      [48, 36],
      [52, 36],
      [56, 41],
      [60, 42],
      [64, 46]
        ]) : return;
  
 

////////////////////////////////////////////////////////////////////////////
//function to return across points dimention for metric Allen head sockets
// Provides standard metric hex Allen head diameters across points from across flats size.


function AllenAcrosFlatsToPointsDia(diameter) =
  ThreadToAllenHeadAcrossFlats(diameter) / cos(30);
  
  
////////////////////////////////////////////////////////////////////////////
//function to return standard metric nut thickness from thread size

function NutThickness(height) =
  (height <= 64) ?
    lookup(height, [
      [2, 1.6],
      [2.5, 2],
      [3, 2.4],
      [3.5, 2.8],
      [4, 3.2],
      [5, 4.7],
      [6, 5.2],
      [7, 6.0],
      [8, 6.8],
      [10, 8.4],
      [12, 10.8],
      [14, 12.8],
      [16, 14.8],
      [18, 15.8],
      [20, 18.0],
      [22, 21.1],
      [24, 21.5],
      [27, 23.8],
      [30, 25.6],
      [33, 28.7],
      [36, 31.0],
      [42, 34],
      [48, 38],
      [56, 45],
      [64, 51]
        ]) : return;
  
 ////////////////////////////////////////////////////////////////////////////
//function to return clearance hole size for a metric thread

 
function ThreadToClearanceHole(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.05],
      [2.5, 2.6],
      [3, 3.1],
      [3.5, 3.6],
      [4, 4.1],
      [5, 5.1],
      [6, 6.1],
      [7, 7.2],
      [8, 8.2],
      [10, 10.2],
      [12, 12.2],
      [14, 14.25],
      [16, 16.25],
      [18, 18.25],
      [20, 20.25],
      [22, 22.25],
      [24, 24.25],
      [27, 27.25],
      [30, 30.5],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 64.75]
        ]) : return;
        
        
////////////////////////////////////////////////////////////////////////////
//function to return cap screw head diameter clearance hole size

 
function ThreadToCapHeadDiameter(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 3.5],
      [2.5, 2.6],
      [3, 5.5],
      [3.5, 3.6],
      [4, 7.0],
      [5, 8.5],
      [6, 10.0],
      [7, 11.5],
      [8, 13.0],
      [10, 16.0],
      [12, 18.0],
      [14, 21.0],
      [16, 24.0],
      [18, 27.0],
      [20, 30.0],
      [22, 33.0],
      [24, 36.0],
      [27, 40.0],
      [30, 45.0],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 60.75]
        ]) : return;
  
 ////////////////////////////////////////////////////////////////////////////
//function to return cap screw head diameter clearance hole size

 
function ThreadToCapHeadDepth(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.0],
      [2.5, 2.5],
      [3, 3.0],
      [3.5, 3.5],
      [4, 4.0],
      [5, 5.0],
      [6, 6.0],
      [7, 7.0],
      [8, 8.0],
      [10, 10.0],
      [12, 12.0],
      [14, 14.0],
      [16, 16.0],
      [18, 18.0],
      [20, 20.0],
      [22, 22.0],
      [24, 24.0],
      [27, 27.0],
      [30, 30.0],
      [33, 33.0],
      [36, 36.0],
      [39, 39.0],
      [42, 42.0],
      [45, 45.0],
      [48, 48.0],
      [52, 52.0],
      [56, 56.0],
      [60, 60.0],
      [64, 60.0]
        ]) : return;
  
 
////////////////////////////////////////////////////////////////////////////
//END MODULE SECTION 
////////////////////////////////////////////////////////////////////////////
