/* this is a script for making boxes and lids

Version 1 DESIGN STARTED 01-08-2020
Version 2 added cable clamps and grips
          added second set of backplate pads
          added additional box bosses 3 & 4 JMC 03-07-2021
Version 3 added full polygon to box and lid hollow
          added pads for lid, previously only on box JMC 22-07-2021
Version 4 nut rotation, nut retaining slot added
          hole exist,type,dia,length part of array shuffled about JMC 29-10-2021 
Version 5 radial holes option added to circ array JMC 02-11-21
Version 6 x-y offset moves added to mesh, added for collet boxes
          JMC 24-01-2023 ,   base done others to sort
Version 7 second set of box and lid pillers added JMC 14-2-23

AUTHOR JOHN MCPHEE. 

Generally this script is for generating a box with mating lid.
The box and lid can be previewed / rendered seperatly or together and
can be translated together to view the finished article.
The box has various options and features enabled by a "xxxxxx_exist" parameters. After
which detailed parameters need entered. The parameters are hopefully self explanatry. 

*/

///////////////////////////////////////////////////////////////
//future addition to consider
///////////////////////////////////////////////////////////////

//add locks or cathes
//add more hinge types. eg ball clip no pins
//1/4 inch 20tpi unc camera mount
//mesh type add 2=import 2d drg, use surface or import ??
//can fan grill shapes be imported
//



///////////////////////////////////////////////////////////////
//problems to sort at some point
///////////////////////////////////////////////////////////////

//rounded box and lid not working properly or was that in blocks
//box_top_radius_exists = 1;//puts radius on bottom ok also lid bottom???
//also puts radius on box inside bottom

///////////////////////////////////////////////////////////////
//include libraries
///////////////////////////////////////////////////////////////

use <mounting_blocks.scad>;    //used for bespoke shapes
use <terminal_blocks.scad>;    //used for cable terminations
use <my_threads.scad>;         //used for threaded holes

///////////////////////////////////////////////////////////////
//define resolution
///////////////////////////////////////////////////////////////

$fn=30;

///////////////////////////////////////////////////////////////
//PARAMETERS FOR POSITIONING PARTS FOR PRINTING
///////////////////////////////////////////////////////////////

box_print = 1;                      //show box for printing
lid_print = 0;                      //show lid for printing
lid_base_print_seperation = 170;     //distance between box and lid
show_assembled = 0;                 //shows lid on top of box
assembled_split = 20;               //when assembled hovers lid above box
flip_lid_for_print = 0;             //turns lid over if better for printing

///////////////////////////////////////////////////////////////
//IF HINGES ARE REQUIRED DEFINE FIRSY BEFORE STAGE BUILDS
///////////////////////////////////////////////////////////////

//common variables
flange_hinge_exists = 0;
hinge_joint_position = 50;//where 2 hinges meet from centre

//box hinge variables

box_hinge_type = 0; //0=screw end 1=nut end
box_hinge_length = 20;
box_hinge_width = 15;
box_hinge_shaft_height = box_hinge_width/2;
box_hinge_screw_size = 4;
box_hinge_cbore_length = 4;
box_hinge_rot_x =0;
box_hinge_rot_y = 0;
box_hinge_rot_z = 0;
box_hinge_pos_x = 0;
box_hinge_pos_y = 0;
box_hinge_pos_z = 0;

//lid hinge variables

lid_hinge_type = 1; //0=screw end 1=nut end
lid_hinge_length = 15;
lid_hinge_width = 15;
lid_hinge_shaft_height =lid_hinge_width/2;
lid_hinge_screw_size = 4;
lid_hinge_cbore_length = 4;
lid_hinge_rot_x = 0;
lid_hinge_rot_y = 0;
lid_hinge_rot_z = 0;
lid_hinge_pos_x = 0;
lid_hinge_pos_y = 0;
lid_hinge_pos_z = 0;

// angled side hinges
side_hinges_exist = 0;
box_side_hinge_height = 10;
lid_side_hinge_height = 10;
side_hinge_length = 130;
side_hinge_radius = 5;
side_hinge_center_dist = 5;
side_hinge_pin_dia = 3;
side_hinge_end_cap_dia = 8;
side_hinge_end_cap_depth = 4;
side_hinge_box_finger_qty = 3;
side_hinge_finger_width = side_hinge_length/((side_hinge_box_finger_qty*2)-1);
side_hinge_finger_clearance = 0.5;

////////////////////////////////////////////////////////////////
//PARAMETERS FOR STAGE 1 OF THE BOX AND LID BUILD
//this defines the size and hollow shape of box and lid 
////////////////////////////////////////////////////////////////

//define the box size
box_length = 190;
box_width = 110;
box_height = 50;
box_corner_radius = 2;
box_top_radius_exists = 0;//puts radius on bottom ok also lid bottom???

//define the lid size
lid_length = 74;//box_length;//changed for split lid
lid_width = box_width-((box_hinge_shaft_height+(box_hinge_width/2))*flange_hinge_exists);
lid_height = 8;
lid_corner_radius = box_corner_radius;
lid_top_radius_exists = 0;


//define the box hollow

//box simple hollow has inside same shape as outside
box_simple_hollow_exists = 1;
box_hollow_wall_thickness = 4; //used if hollow polygon not used, also used in clamps and recesses
box_base_thickness = 4;

// box simple polygon has a fixed hollow as defined by parameters
box_hollow_simple_polygon_exists = 0;  //0=simple hollow 1=defined polygon

//if a simple polygon hollow is defined the following explains the parameters
//declare the internal shape of the body, lid or plinth.
//uses a 16 pointed symetrical polygon having 7 dimentions then extrudes it
/*

layout of internal polygon to hollow out


                .i     .h                   
         .k     .j     .g       .f          
   m.  .l                         .e .d     
                 y+ |
              x- ---+--- x+                 
                 y- |
   n.  .o                         .b .c     
         .p     .q      .t      .a          
                .r      .s                  
                

 a = poly_x_mid,-poly_y_mid
 b = poly_x_far,-poly_y_near
 c = poly_x_ext,-poly_y_near
 d = poly_x_ext,poly_y_near
 e = poly_x_far,poly_y_near
 f = poly_x_mid,poly_y_mid
 g = poly_x_near,poly_y_mid
 h = poly_x_near,poly_y_far
 i = -poly_x_near,poly_y_far
 j = -poly_x_near,poly_y_mid
 k = -poly_x_mid,poly_y_mid
 l = -poly_x_far,poly_y_near
 m = -poly_x_ext,poly_y_near
 n = -poly_x_ext,-poly_y_near
 o = -poly_x_far,-poly_y_near
 p = -poly_x_mid,-poly_y_mid
 q = -poly_x_near,-poly_y_mid
 r = -poly_x_near,-poly_y_far
 s = poly_x_near,-poly_y_far
 t = poly_x_near,-poly_y_mid

*/

//define the box polygon hollowing out
box_poly_x_near = 20;
box_poly_x_mid = 24;
box_poly_x_far = 35;
box_poly_x_ext = 35;
box_poly_y_near = 18;
box_poly_y_mid = 27;
box_poly_y_far = 27;

// box full polygon has an open ended number of points as defined in array
box_hollow_full_polygon_exists = 0;  //0=simple hollow 1=full polygon


//define polygon mesh points, create your own polygon
box_hollow_full_poly_points = [ [35,22],[-39,22],[-39,28],[-41,28],[-41,22],[-43.1,22],
                                [-43.1,-22],[-41,-22],[-41,-28],[-39,-28],[-39,-22],[35,-22] ];

//box simple hollow has inside same shape as outside
lid_simple_hollow_exists = 1;
lid_hollow_wall_thickness = box_hollow_wall_thickness; //used if hollow polygon not used
lid_top_thickness = 2;

lid_hollow_simple_polygon_exists = 0; //0=simple hollow 1=defined polygon

// lid full polygon has an open ended number of points as defined in array
lid_hollow_full_polygon_exists = 0;  //0=simple hollow 1=full polygon

//define a plinth on the box
box_plinth_exists = 0;
box_plinth_thickness = 8;
box_plinth_clearance = 2;

//define a plinth on the lid
lid_plinth_exists = 0;
lid_plinth_thickness = 2.0;
lid_plinth_clearance = 1.0;

//define the lid polygon hollowing out
lid_poly_x_near = box_poly_x_near;
lid_poly_x_mid = box_poly_x_mid;
lid_poly_x_far = box_poly_x_far;
lid_poly_x_ext = box_poly_x_ext;
lid_poly_y_near = box_poly_y_near-
                ((box_hinge_shaft_height+(box_hinge_width/2))*flange_hinge_exists)/2;
lid_poly_y_mid = box_poly_y_mid-
               ((box_hinge_shaft_height+(box_hinge_width/2))*flange_hinge_exists)/2;
lid_poly_y_far = box_poly_y_far-
               ((box_hinge_shaft_height+(box_hinge_width/2))*flange_hinge_exists)/2;


               
////////////////////////////////////////////////////////////////
//PARAMETERS FOR STAGE 2 OF THE BOX AND LID BUILD
//
//if required add flange, pillers, dividers, pcb cradles, 
//backplate pads, filter holders,flange hinge, angled hinge
//
//then if required remove side recesses, sliding lid slots
//cutout 1-5,vent grills 1-5, fans 1-2, oring grooves,
//meshes to faces
////////////////////////////////////////////////////////

//define box top flange, puts a fixed flange on top of hollowed box
box_flange_exists = 0;
box_flange_thickness = 3;
box_flange_opening_X = 130;
box_flange_opening_Y = 50;
box_top_opening_pos_X = 0;
box_top_opening_pos_Y = 0;

//define lid top flange, puts a fixed flange on top of hollowed lid
lid_flange_exists = 0;
lid_flange_thickness = 5;
lid_flange_opening_X = 100;
lid_flange_opening_Y = 80;
lid_top_opening_pos_X = 0;
lid_top_opening_pos_Y = 0;

//define array of pillers to box
box_pillers_1_exist = 1;
box_pillers_1_outer_only = 0;
box_pillers_1_dia = 8;
box_pillers_1_length = box_height;
box_pillers_1_x_qty = 2;
box_pillers_1_y_qty = 2;
box_pillers_1_x_pitch = 66;
box_pillers_1_y_pitch = 102;
box_pillers_1_shift_x = 58;
box_pillers_1_shift_y = 0;
box_pillers_1_shift_z = 0;

//define array of pillers to box
box_pillers_2_exist = 1;
box_pillers_2_outer_only = 0;
box_pillers_2_dia = 8;
box_pillers_2_length = box_height;
box_pillers_2_x_qty = 2;
box_pillers_2_y_qty = 2;
box_pillers_2_x_pitch = 108;
box_pillers_2_y_pitch = 102;
box_pillers_2_shift_x = -37;
box_pillers_2_shift_y = 0;
box_pillers_2_shift_z = 0;


//define pillers to lid for spindle
lid_pillers_1_exist = 1;
lid_pillers_1_outer_only = 0;
lid_pillers_1_dia = 8;
lid_pillers_1_length = lid_height;
lid_pillers_1_x_qty = 2;
lid_pillers_1_y_qty = 2;
lid_pillers_1_x_pitch = 66;
lid_pillers_1_y_pitch = 102;
lid_pillers_1_shift_x = 0;
lid_pillers_1_shift_y = 0;
lid_pillers_1_shift_z = 0;

//define pillers to lid for motor
lid_pillers_2_exist = 0;
lid_pillers_2_outer_only = 0;
lid_pillers_2_dia = 8;
lid_pillers_2_length = lid_height;
lid_pillers_2_x_qty = 2;
lid_pillers_2_y_qty = 2;
lid_pillers_2_x_pitch = 108;
lid_pillers_2_y_pitch = 102;
lid_pillers_2_shift_x = -37;
lid_pillers_2_shift_y = 0;
lid_pillers_2_shift_z = 0;

//define box seperation dividers, box only feature
box_divider_exist = 0;
box_divider_x_qty = 7;
box_divider_x_pitch = (2*box_poly_x_ext)/(box_divider_x_qty+1);
box_divider_x_length = box_poly_y_far*2;
box_divider_x_thickness = 2;
box_divider_x_height = 60;
box_divider_y_qty = 2;
box_divider_y_pitch = (2*box_poly_y_far)/(box_divider_y_qty+1);
box_divider_y_length = box_poly_x_ext*2;
box_divider_y_thickness = 2;
box_divider_y_height = 60;

//define pcb holding cradles, box only feature

pcb_cradle_exists = 0;
pcb_cradle_qty = 3;
pcb_cradle_pitch = 20;
pcb_board_length = 70;
pcb_board_width = 55;
pcb_board_thickness = 2;
pcb_cradle_length = 80;
pcb_cradle_width = 60;
pcb_cradle_thickness = 6;
pcb_cradle_base_depth = 10;
pcb_cradle_opening_length = 65;
pcb_cradle_rot_x = 0;
pcb_cradle_rot_y = 0;
pcb_cradle_rot_z = 90;
pcb_cradle_pos_x = -30;
pcb_cradle_pos_y = 0;
pcb_cradle_pos_z = 0;

//define set of two box mounting pads, front
box_mount_1_pads_exist = 0;
box_mount_1_pads_length = 4;
box_mount_1_pads_width = 6;
box_mount_1_pads_height = 5;
box_mount_1_pads_x_qty = 1;
box_mount_1_pads_x_pitch = 0;
box_mount_1_pads_y_qty = 2;
box_mount_1_pads_y_pitch = 25;
box_mount_1_pads_x_move = -25;
box_mount_1_pads_y_move = 0;

//define set of four box mounting pads, board fixing
box_mount_2_pads_exist = 0;
box_mount_2_pads_length = 7;
box_mount_2_pads_width = 7;
box_mount_2_pads_height = 5;
box_mount_2_pads_x_qty = 1;
box_mount_2_pads_x_pitch = 0;
box_mount_2_pads_y_qty = 1;
box_mount_2_pads_y_pitch = 0;
box_mount_2_pads_x_move = 28.5;
box_mount_2_pads_y_move = 0;

//define set of four lid mounting pads
lid_mount_1_pads_exist = 0;
lid_mount_1_pads_length = 4;
lid_mount_1_pads_width = 6;
lid_mount_1_pads_height = 6;
lid_mount_1_pads_x_qty = 1;
lid_mount_1_pads_x_pitch = 0;
lid_mount_1_pads_y_qty = 2;
lid_mount_1_pads_y_pitch = 25;
lid_mount_1_pads_x_move = -25;
lid_mount_1_pads_y_move = 0;

//define set of lid mounting pads
lid_mount_2_pads_exist = 0;
lid_mount_2_pads_length = 2;
lid_mount_2_pads_width = 50;
lid_mount_2_pads_height = 1;
lid_mount_2_pads_x_qty = 1;
lid_mount_2_pads_x_pitch = 0;
lid_mount_2_pads_y_qty = 1;
lid_mount_2_pads_y_pitch = 0;
lid_mount_2_pads_x_move = -42;
lid_mount_2_pads_y_move = 0;


//define box filter mountings
//box filter 1
box_filter_1_exists = 0;
box_filter_1_side = 3;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_filter_1_length = 30;
box_filter_1_width = 30;
box_filter_1_thickness = 5;
box_filter_1_wall_thickness = 2;
box_filter_1_retain_lip = 5;
box_filter_1_stop_thickness = 2;
box_filter_1_x_pos = 0;
box_filter_1_y_pos = 0;
box_filter_1_z_pos = 20;

//box filter 2
box_filter_2_exists = 0;
box_filter_2_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_filter_2_length = 30;
box_filter_2_width = 30;
box_filter_2_thickness = 5;
box_filter_2_wall_thickness = 2;
box_filter_2_retain_lip = 5;
box_filter_2_stop_thickness = 2;
box_filter_2_x_pos = 0;
box_filter_2_y_pos = 0;
box_filter_2_z_pos = 20;

//define lid filter mountings
//lid filter 1
lid_filter_1_exists = 0;
lid_filter_1_side = 1;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_filter_1_length = 20;
lid_filter_1_width = 20;
lid_filter_1_thickness = 3;
lid_filter_1_wall_thickness = 2;
lid_filter_1_retain_lip = 5;
lid_filter_1_stop_thickness = 2;
lid_filter_1_x_pos = 0;
lid_filter_1_y_pos = 0;
lid_filter_1_z_pos = 20;

//lid filter 2
lid_filter_2_exists = 0;
lid_filter_2_side = 2;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_filter_2_length = 20;
lid_filter_2_width = 20;
lid_filter_2_thickness = 3;
lid_filter_2_wall_thickness = 2;
lid_filter_2_retain_lip = 5;
lid_filter_2_stop_thickness = 2;
lid_filter_2_x_pos = 0;
lid_filter_2_y_pos = 0;
lid_filter_2_z_pos = 20;

//define the side recesses of the junction box and lid
//x positive side
box_x_pos_recess_exists = 0;
box_x_pos_recess_length = 40;
box_x_pos_recess_radius = 7;
box_x_pos_recess_move_y = 0;

//x negitive side
box_x_neg_recess_exists = 0;
box_x_neg_recess_length = 34;
box_x_neg_recess_radius = 8;
box_x_neg_recess_move_y = 0;

//y positive side
box_y_pos_recess_exists = 0;
box_y_pos_recess_length = 46;
box_y_pos_recess_radius = 6;
box_y_pos_recess_move_x = 0;

//y negitive side
box_y_neg_recess_exists = 0;
box_y_neg_recess_length = 46;
box_y_neg_recess_radius = 6;
box_y_neg_recess_move_x = 0;

//the box can have sliding lid or lids
//the box flange feature is used to cut the grooves into
//sliding lid details
sliding_lid_exists = 0;
sliding_lid_qty = 3;
sliding_lid_width = 35;
sliding_lid_pitch = (2*box_poly_x_ext)/(sliding_lid_qty);
sliding_lid_clearance = 0.5;

//sliding lid thumbhole parameters
sliding_lid_thumbhole_exists = 0;
sliding_lid_thumbhole_sphere_dia = 20;
sliding_lid_thumbhole_cube_size = 40;
sliding_lid_thumbhole_cube_height = 5;
sliding_lid_thumbhole_x_move = 0;
sliding_lid_thumbhole_y_move = -40;
sliding_lid_thumbhole_z_move = 2.5;
sliding_lid_thumbhole_rotation = 90;

//define cutouts in the box sides or base
//define box cutout number 1
box_cutout_1_exists = 0;
box_cutout_1_side = 2;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_cutout_1_x_qty = 1;
box_cutout_1_x_pitch = 0;
box_cutout_1_y_qty = 1;
box_cutout_1_y_pitch = 0;
box_cutout_1_x_dim = 55;
box_cutout_1_y_dim = 8;
box_cutout_1_depth = 2;
box_cutout_1_x_pos = 2;
box_cutout_1_y_pos = 0;
box_cutout_1_z_pos = 1;

//define box cutout number 2
box_cutout_2_exists = 0;
box_cutout_2_side = 4;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_cutout_2_x_qty = 1;
box_cutout_2_x_pitch = 0;
box_cutout_2_y_qty = 1;
box_cutout_2_y_pitch = 0;
box_cutout_2_x_dim = 20;
box_cutout_2_y_dim = 15;
box_cutout_2_depth = 10;
box_cutout_2_x_pos = -30;
box_cutout_2_y_pos = 0;
box_cutout_2_z_pos = -10;

//define box cutout number 3
box_cutout_3_exists = 0;
box_cutout_3_side = 3;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_cutout_3_x_qty = 3;
box_cutout_3_x_pitch = 30;
box_cutout_3_y_qty = 2;
box_cutout_3_y_pitch = 20;
box_cutout_3_x_dim = 20;
box_cutout_3_y_dim = 15;
box_cutout_3_depth = 10;
box_cutout_3_x_pos = 0;
box_cutout_3_y_pos = 0;
box_cutout_3_z_pos = 0;

//define box cutout number 4
box_cutout_4_exists = 0;
box_cutout_4_side = 4;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_cutout_4_x_qty = 3;
box_cutout_4_x_pitch = 30;
box_cutout_4_y_qty = 2;
box_cutout_4_y_pitch = 20;
box_cutout_4_x_dim = 20;
box_cutout_4_y_dim = 15;
box_cutout_4_depth = 10;
box_cutout_4_x_pos = 0;
box_cutout_4_y_pos = 0;
box_cutout_4_z_pos = 0;

//define box cutout number 5
box_cutout_5_exists = 0;
box_cutout_5_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_cutout_5_x_qty = 3;
box_cutout_5_x_pitch = 30;
box_cutout_5_y_qty = 2;
box_cutout_5_y_pitch = 20;
box_cutout_5_x_dim = 20;
box_cutout_5_y_dim = 15;
box_cutout_5_depth = 10;
box_cutout_5_x_pos = 0;
box_cutout_5_y_pos = 0;
box_cutout_5_z_pos = 0;

//define cutouts in the lid sides or base
//define lid cutout number 1 (d shell recess)
lid_cutout_1_exists = 0;
lid_cutout_1_side = 2;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_cutout_1_x_qty = 1;
lid_cutout_1_x_pitch = 0;
lid_cutout_1_y_qty = 1;
lid_cutout_1_y_pitch = 0;
lid_cutout_1_x_dim = 55;
lid_cutout_1_y_dim = 8;
lid_cutout_1_depth = 2;
lid_cutout_1_x_pos = 2;
lid_cutout_1_y_pos = 0;
lid_cutout_1_z_pos = 1;

//define lid cutout number 2 ( 12 0ff LED )
lid_cutout_2_exists = 0;
lid_cutout_2_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_cutout_2_x_qty = 1;
lid_cutout_2_x_pitch = 0;
lid_cutout_2_y_qty = 1;
lid_cutout_2_y_pitch = 0;
lid_cutout_2_x_dim = 6;
lid_cutout_2_y_dim = 32;
lid_cutout_2_depth = 15;
lid_cutout_2_x_pos = -17;
lid_cutout_2_y_pos = -2.0;
lid_cutout_2_z_pos = -0.5;

//define lid cutout number 3 ( led and switches )
lid_cutout_3_exists = 0;
lid_cutout_3_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_cutout_3_x_qty = 1;
lid_cutout_3_x_pitch = 0;
lid_cutout_3_y_qty = 1;
lid_cutout_3_y_pitch = 0;
lid_cutout_3_x_dim = 19;
lid_cutout_3_y_dim = 15;
lid_cutout_3_depth = 15;
lid_cutout_3_x_pos = 16;
lid_cutout_3_y_pos = -9;
lid_cutout_3_z_pos = -0.5;

//define lid cutout number 4 ( DIL switches )
lid_cutout_4_exists = 0;
lid_cutout_4_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_cutout_4_x_qty = 1;
lid_cutout_4_x_pitch = 0;
lid_cutout_4_y_qty = 1;
lid_cutout_4_y_pitch = 00;
lid_cutout_4_x_dim = 10;
lid_cutout_4_y_dim = 15;
lid_cutout_4_depth = 15;
lid_cutout_4_x_pos = 15;
lid_cutout_4_y_pos = -8;
lid_cutout_4_z_pos = -0.5;

//define lid cutout number 5
lid_cutout_5_exists = 0;
lid_cutout_5_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_cutout_5_x_qty = 3;
lid_cutout_5_x_pitch = 30;
lid_cutout_5_y_qty = 2;
lid_cutout_5_y_pitch = 20;
lid_cutout_5_x_dim = 20;
lid_cutout_5_y_dim = 15;
lid_cutout_5_depth = 10;
lid_cutout_5_x_pos = 0;
lid_cutout_5_y_pos = 0;
lid_cutout_5_z_pos = 0;

//define ventilation grills in sides of box
//define vent 1 grills
box_vents_1_exists = 1;
box_vents_1_side = 5; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_vents_1_x_qty = 1;
box_vents_1_x_pitch = 0;
box_vents_1_y_qty = 1;
box_vents_1_y_pitch = 0;
box_vents_1_width = 28.5;
box_vents_1_height = 6.5;
box_vents_1_depth = 10;
box_vents_1_radius = 3.25;
box_vents_1_x_pos= -71;
box_vents_1_y_pos= 0;
box_vents_1_z_pos= 0;

//define vent 2 grills
box_vents_2_exists = 1;
box_vents_2_side = 5; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_vents_2_x_qty = 1;
box_vents_2_x_pitch = 0;
box_vents_2_y_qty = 2;
box_vents_2_y_pitch = 81;
box_vents_2_width = 28.5;
box_vents_2_height = 6.5;
box_vents_2_depth = 10;
box_vents_2_radius = 3.25;
box_vents_2_x_pos = -1;
box_vents_2_y_pos= 0;
box_vents_2_z_pos= 0;

//define vent 3 grills
box_vents_3_exists = 0;
box_vents_3_side = 3; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_vents_3_x_qty = 4;
box_vents_3_x_pitch = 10;
box_vents_3_y_qty = 3;
box_vents_3_y_pitch = 12;
box_vents_3_width = 6;
box_vents_3_height = 8;
box_vents_3_depth = 10;
box_vents_3_radius = 2;
box_vents_3_x_pos= 0;
box_vents_3_y_pos= 0;
box_vents_3_z_pos= 0;

//define vent 4 grills
box_vents_4_exists = 0;
box_vents_4_side = 4; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_vents_4_x_qty = 6;
box_vents_4_x_pitch = 10;
box_vents_4_y_qty = 3;
box_vents_4_y_pitch = 12;
box_vents_4_width = 6;
box_vents_4_height = 6;
box_vents_4_depth = 10;
box_vents_4_radius = 2;
box_vents_4_x_pos= 0;
box_vents_4_y_pos= 0;
box_vents_4_z_pos= 0;

//define vent 5 grills
box_vents_5_exists = 0;
box_vents_5_side = 5; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_vents_5_x_qty = 5;
box_vents_5_x_pitch = 25;
box_vents_5_y_qty = 4;
box_vents_5_y_pitch = 20;
box_vents_5_width = 6;
box_vents_5_height = 10;
box_vents_5_depth = 10;
box_vents_5_radius = 2;
box_vents_5_x_pos= 0;
box_vents_5_y_pos= 0;
box_vents_5_z_pos= 0;

//define ventilation grills in sides of lid
//define vent 1 grills
lid_vents_1_exists = 0;
lid_vents_1_side = 1; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_vents_1_x_qty = 3;
lid_vents_1_x_pitch = 10;
lid_vents_1_y_qty = 3;
lid_vents_1_y_pitch = 15;
lid_vents_1_width = 6;
lid_vents_1_height = 10;
lid_vents_1_depth = 6;
lid_vents_1_radius = 2;
lid_vents_1_x_pos= 0;
lid_vents_1_y_pos= 0;
lid_vents_1_z_pos= 0;

//define vent 2 grills
lid_vents_2_exists = 0;
lid_vents_2_side = 2; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_vents_2_x_qty = 3;
lid_vents_2_x_pitch = 10;
lid_vents_2_y_qty = 4;
lid_vents_2_y_pitch = 15;
lid_vents_2_width = 6;
lid_vents_2_height = 10;
lid_vents_2_depth = 6;
lid_vents_2_radius = 2;
lid_vents_2_x_pos= 0;
lid_vents_2_y_pos= 0;
lid_vents_2_z_pos= 0;

//define vent 3 grills
lid_vents_3_exists = 0;
lid_vents_3_side = 3; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_vents_3_x_qty = 7;
lid_vents_3_x_pitch = 10;
lid_vents_3_y_qty = 3;
lid_vents_3_y_pitch = 19;
lid_vents_3_width = 6;
lid_vents_3_height = 10;
lid_vents_3_depth = 6;
lid_vents_3_radius = 2;
lid_vents_3_x_pos= 0;
lid_vents_3_y_pos= 0;
lid_vents_3_z_pos= 0;

//define vent 4 grills
lid_vents_4_exists = 0;
lid_vents_4_side = 4; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_vents_4_x_qty = 9;
lid_vents_4_x_pitch = 10;
lid_vents_4_y_qty = 3;
lid_vents_4_y_pitch = 15;
lid_vents_4_width = 6;
lid_vents_4_height = 10;
lid_vents_4_depth = 6;
lid_vents_4_radius = 2;
lid_vents_4_x_pos= 0;
lid_vents_4_y_pos= 0;
lid_vents_4_z_pos= 0;

//define vent 5 grills
lid_vents_5_exists = 0;
lid_vents_5_side = 5; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_vents_5_x_qty = 5;
lid_vents_5_x_pitch = 10;
lid_vents_5_y_qty = 5;
lid_vents_5_y_pitch = 10;
lid_vents_5_width = 6;
lid_vents_5_height = 8;
lid_vents_5_depth = 6;
lid_vents_5_radius = 2;
lid_vents_5_x_pos= 0;
lid_vents_5_y_pos= 0;
lid_vents_5_z_pos= 0;



//define box fan mounting cut outs
//box fan 1
box_fan_1_exists = 0;
box_fan_1_side = 1; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_fan_1_dia = 30;
box_fan_1_fix_dia = 4;
box_fan_1_fixing_square = 25;
box_fan_1_depth = 10;
box_fan_1_x_pos = 0;
box_fan_1_y_pos = 0;
box_fan_1_z_pos = 0;

//box fan 2
box_fan_2_exists = 0;
box_fan_2_side = 2; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_fan_2_dia = 30;
box_fan_2_fix_dia = 4;
box_fan_2_fixing_square = 25;
box_fan_2_depth = 15;
box_fan_2_x_pos = 0;
box_fan_2_y_pos = 0;
box_fan_2_z_pos = 0;

//define lid fan mounting cut outs
//lid fan 1
lid_fan_1_exists = 0;
lid_fan_1_side = 1; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_fan_1_dia = 20;
lid_fan_1_fix_dia = 3;
lid_fan_1_fixing_square = 18;
lid_fan_1_depth = 15;
lid_fan_1_x_pos = 0;
lid_fan_1_y_pos = 0;
lid_fan_1_z_pos = 0;

//lid fan 2
lid_fan_2_exists = 0;
lid_fan_2_side = 2; //1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_fan_2_dia = 20;
lid_fan_2_fix_dia = 3;
lid_fan_2_fixing_square = 18;
lid_fan_2_depth = 15;
lid_fan_2_x_pos = 0;
lid_fan_2_y_pos = 0;
lid_fan_2_z_pos = 0;


// o_ring_insertion
// defines an "o" ring groove between box and lid
// if side recesses are selected the x pos or neg steps are added

o_ring_selected = 0;    // 0 or 1
o_ring_add_x_pos_corner = 1; // 0 or 1
o_ring_add_x_neg_corner = 1; // 0 or 1
o_ring_x_outer = 115;
o_ring_x_inner = 95;
o_ring_y_outer = 60;
o_ring_y_inner = 50;

//o ring radius at corners and diameter of rubber o ring
o_ring_corner_radius = 3;
o_ring_diameter = 2;

//define any mesh work on the x positive face of the box
box_mesh_x_pos_side_exists = 0;
box_mesh_x_pos_offset_req = 1;
box_mesh_x_pos_type = 4; //0=circles,3 up = polygon sides 1&2 invalid
box_mesh_x_pos_dia = 8;
box_mesh_x_pos_rotation = 45;
box_mesh_x_pos_horz_qty = 7;
box_mesh_x_pos_horz_pitch = 12;
box_mesh_x_pos_vert_qty = 5;
box_mesh_x_pos_vert_pitch = 9;
box_mesh_x_pos_depth = 10;
box_mesh_x_pos_depth_offset = 0; //so pattern can have bottom
box_mesh_x_pos_x_shift = 0;
box_mesh_x_pos_y_shift = 0;

//define any mesh work on the x negitive face of the box
box_mesh_x_neg_side_exists = 0;
box_mesh_x_neg_offset_req = 0;
box_mesh_x_neg_type = 7; //0=circles,3 up = polygon sides 1&2 invalid
box_mesh_x_neg_dia = 8;
box_mesh_x_neg_rotation = 0;
box_mesh_x_neg_horz_qty = 7;
box_mesh_x_neg_horz_pitch = 12;
box_mesh_x_neg_vert_qty = 5;
box_mesh_x_neg_vert_pitch = 9;
box_mesh_x_neg_depth = 10;
box_mesh_x_neg_depth_offset = 0; //so pattern can have bottom
box_mesh_x_neg_x_shift = 0;
box_mesh_x_neg_y_shift = 0;

//define any mesh work on the y positive face of the box
box_mesh_y_pos_side_exists = 0;
box_mesh_y_pos_offset_req = 1;
box_mesh_y_pos_type = 2; //0=circles,1=polygon,2=dxf file, 3 up = polygon sides
box_mesh_y_pos_dia = 8;
box_mesh_y_pos_rotation = 0;
box_mesh_y_pos_horz_qty = 7;
box_mesh_y_pos_horz_pitch = 12;
box_mesh_y_pos_vert_qty = 5;
box_mesh_y_pos_vert_pitch = 9;
box_mesh_y_pos_depth = 10;
box_mesh_y_pos_depth_offset = 0; //so pattern can have bottom
box_mesh_y_pos_x_shift = 0;
box_mesh_y_pos_y_shift = 0;


//define any mesh work on the y negitive face of the box
box_mesh_y_neg_side_exists = 0;
box_mesh_y_neg_offset_req = 0;
box_mesh_y_neg_type = 3; //0=circles,3 up = polygon sides 1&2 invalid
box_mesh_y_neg_dia = 3;
box_mesh_y_neg_rotation = 0;
box_mesh_y_neg_horz_qty = 7;
box_mesh_y_neg_horz_pitch = 12;
box_mesh_y_neg_vert_qty = 5;
box_mesh_y_neg_vert_pitch = 9;
box_mesh_y_neg_depth = 10;
box_mesh_y_neg_depth_offset = 0; //so pattern can have bottom
box_mesh_y_neg_x_shift = 0;
box_mesh_y_neg_y_shift = 0;


//define any mesh work on the bottom face of the box
box_mesh_base_side_exists = 0;
box_mesh_base_offset_req = 0;
box_mesh_base_type = 0; //0=circles,3 up = polygon sides 1&2 invalid
box_mesh_base_dia = 13;
box_mesh_base_rotation = 0;
box_mesh_base_horz_qty = 6;
box_mesh_base_horz_pitch = 20;
box_mesh_base_vert_qty = 2;
box_mesh_base_vert_pitch = 24;
box_mesh_base_depth = 15;
box_mesh_base_depth_offset = 3; //so pattern can have bottom
box_mesh_base_x_shift = 0;
box_mesh_base_y_shift = 3;

//define any mesh work on the standard lid
lid_top_mesh_exists = 0;
lid_top_mesh_offset_req = 0;
lid_top_mesh_type = 6; //0=circles,3 up = polygon sides 1&2 invalid
lid_top_mesh_dia = 6;
lid_top_mesh_rotation = 0;
lid_top_mesh_x_qty = 12;
lid_top_mesh_x_pitch = 8;
lid_top_mesh_y_qty = 8;
lid_top_mesh_y_pitch = 8;
lid_top_mesh_x_pos = 0;
lid_top_mesh_y_pos = 0;
lid_top_mesh_depth = 5;
lid_top_mesh_depth_offset = 2; //so pattern can have bottom
lid_mesh_base_x_shift = 0;
lid_mesh_base_y_shift = 5;

//define any mesh work on the sliding lids
sliding_lid_mesh_exists = 0;
sliding_lid_mesh_offset_req = 0;
sliding_lid_mesh_type = 6; //0=circles,3 up = polygon sides 1&2 invalid
sliding_lid_mesh_dia = 6;
sliding_lid_mesh_rotation = 0;
sliding_lid_mesh_x_qty = 12;
sliding_lid_mesh_x_pitch = 8;
sliding_lid_mesh_y_qty = 8;
sliding_lid_mesh_y_pitch = 8;
sliding_lid_mesh_y_pos = -5;
sliding_lid_mesh_depth = 10;
sliding_lid_mesh_depth_offset = 0; //so pattern can have bottom
sliding_lid_mesh_base_x_shift = 0;
sliding_lid_mesh_base_y_shift = 5;

//define polygon mesh points
mesh_poly_points = [ [0,0],[1,1],[2,1],[2,3],[1,3],[-1,2],[-2,1],[-2,0] ];

//define a dxf file for mesh shape
dxf_mesh_file = "test.dxf";


////////////////////////////////////////////////////////////////
//PARAMETERS FOR STAGE 3 OF THE BOX AND LID BUILD
//
//if required add entry bosses, import .stl files, 
//positive text, mounting fixtures, terminal strips
//
//then if required remove negative text, coin slot
//holes:- clearance, counterbore, nut retainer, threaded
////////////////////////////////////////////////////////

//define box entry boss 1 parameters
box_boss_1_exists =0;
box_boss_1_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_boss_1_length = 8;//y axis
box_boss_1_width = 45; //z axis
box_boss_1_height = 3; //x axis
box_boss_1_radius = 0;
box_boss_1_x_pos = 39;
box_boss_1_y_pos = 0;
box_boss_1_z_pos = -3;

//define box entry boss 2 parameters
box_boss_2_exists =0;
box_boss_2_side = 2;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_boss_2_length = 35;
box_boss_2_width = 6;
box_boss_2_height = 8;
box_boss_2_radius = 0;
box_boss_2_x_pos = 0;
box_boss_2_y_pos = 0;
box_boss_2_z_pos = -9;

//define box entry boss 3 parameters
box_boss_3_exists =0;
box_boss_3_side = 3;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_boss_3_length = 62;//x axis
box_boss_3_width = 6; //z axis
box_boss_3_height = 8; //y axis
box_boss_3_radius = 0;
box_boss_3_x_pos = 0;
box_boss_3_y_pos = 0;
box_boss_3_z_pos = -9;

//define box entry boss 4 parameters
box_boss_4_exists =0;
box_boss_4_side = 4;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
box_boss_4_length = 62;
box_boss_4_width = 6;
box_boss_4_height = 8;
box_boss_4_radius = 0;
box_boss_4_x_pos = 0;
box_boss_4_y_pos = 0;
box_boss_4_z_pos = -9;

//define lid entry boss 1 parameters
lid_boss_1_exists =0;
lid_boss_1_side = 5;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_boss_1_length = 10;
lid_boss_1_width = 10;
lid_boss_1_height = 11;
lid_boss_1_radius = 5;
lid_boss_1_x_pos = 25;
lid_boss_1_y_pos = 37.5;
lid_boss_1_z_pos = 0;

//define lid entry boss 2 parameters
lid_boss_2_exists =0;
lid_boss_2_side = 4;//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
lid_boss_2_length = 30;
lid_boss_2_width = 20;
lid_boss_2_height = 15;
lid_boss_2_radius = 4;
lid_boss_2_x_pos = 0;
lid_boss_2_y_pos = 0;
lid_boss_2_z_pos = 0;

//define box cable clamp entry 1 parameters
box_clamp_1_exists =0;
box_clamp_1_side = 1; //1=xpos.2=xneg.3=ypos.4=yneg.
box_clamp_1_length = 7;
box_clamp_1_width = 20;
box_clamp_1_height = 7;
box_clamp_1_grip_height = 6;
box_clamp_1_radius = 2;
box_clamp_1_cable_size =3;
box_clamp_1_screw_hole =2;
box_clamp_1_grip_hole =3.5;
box_clamp_1_x_pos = 0;
box_clamp_1_y_pos = 0;
box_clamp_1_z_pos = 0;

//define box cable clamp entry 2 parameters
box_clamp_2_exists =0;
box_clamp_2_side = 2; //1=xpos.2=xneg.3=ypos.4=yneg.
box_clamp_2_length = 8;
box_clamp_2_width = 20;
box_clamp_2_height = 10;
box_clamp_2_grip_height = 6;
box_clamp_2_radius = 3;
box_clamp_2_cable_size =3;
box_clamp_2_screw_hole =2;
box_clamp_2_grip_hole =3.5;
box_clamp_2_x_pos = 0;
box_clamp_2_y_pos = 0;
box_clamp_2_z_pos = 0;


//import a .stl file to add an external component
//place xxx.stl in same directory as this scsd file

//box import 1
box_import_1_stl_exists =0; //0 or 1
box_import_1_stl_file = "TreeOfLife_40mm_Fan_guard.stl";
//size the import
box_import_1_scale_x = 2;
box_import_1_scale_y = 2;            
box_import_1_scale_z = 2;            
//orient the imported stl file
box_import_1_rot_x = 0;
box_import_1_rot_y = 0;
box_import_1_rot_z = 0;
//position the imported stl file
box_import_1_pos_x = 0;
box_import_1_pos_y = 0;
box_import_1_pos_z = 60;

//lid import 1
lid_import_1_stl_exists =0; //0 or 1
lid_import_1_stl_file = "TreeOfLife_40mm_Fan_guard.stl";
//size the import
lid_import_1_scale_x = 2;
lid_import_1_scale_y = 2;            
lid_import_1_scale_z = 2;            
//orient the imported stl file
lid_import_1_rot_x = 0;
lid_import_1_rot_y = 0;
lid_import_1_rot_z = 0;
//position the imported stl file
lid_import_1_pos_x = 0;
lid_import_1_pos_y = 0;
lid_import_1_pos_z = 60;

//print some text on box x positive face

box_text_x_pos_exists = 0;
box_text_x_pos_type = 0; // 0= raised 1= sunken
box_text_x_pos_string = ["text xpos1",
                         "text xpos2",
                         "text xpos3",
                         "text xpos4",
                         "text xpos5",
                         "text xpos6"];
box_text_x_pos_size = 10;
box_text_x_pos_font = "DejaVu Sans      Mono:style=Bold";
box_text_x_pos_direction = "ltr"; //"ltr"(left-to-right),"rtl"(right-to-left),
                                  //"ttb"(top-to-bottom), "btt"(bottom-to-top)
box_text_x_pos_halign = "center";  // "left", "center", "right"
box_text_x_pos_valign = "center";  // "left", "center", "right"
box_text_x_pos_spacing = 1.0;
box_text_x_pos_no_of_lines = 5; // 1-6 or no of strings in array
box_text_x_pos_line_spacing = 10;
box_text_x_pos_extrude_height = 2;
box_text_x_pos_rotation_x = 0;
box_text_x_pos_rotation_y = 0;
box_text_x_pos_rotation_z = 0;
box_text_x_pos_position_x = 0;
box_text_x_pos_position_y = 0;
box_text_x_pos_position_z = 0;

//print some text on box x negitive face

box_text_x_neg_exists = 0;
box_text_x_neg_type = 1; // 0= raised 1= sunken
box_text_x_neg_string = ["text xneg1",
                         "text xneg2",
                         "text xneg3",
                         "text xneg4",
                         "text xneg5",
                         "text xneg6"];
box_text_x_neg_size = 10;
box_text_x_neg_font = "DejaVu Sans      Mono:style=Bold";
box_text_x_neg_direction = "ltr"; //"ltr"(left-to-right),"rtl"(right-to-left),
                                  //"ttb"(top-to-bottom), "btt"(bottom-to-top)
box_text_x_neg_halign = "center";  // "left", "center", "right"
box_text_x_neg_valign = "center";  // "left", "center", "right"
box_text_x_neg_spacing = 1.0;
box_text_x_neg_no_of_lines = 5; // 1-6 or no of strings in array
box_text_x_neg_line_spacing = 10;
box_text_x_neg_extrude_height = 2;
box_text_x_neg_rotation_x = 0;
box_text_x_neg_rotation_y = 0;
box_text_x_neg_rotation_z = 0;
box_text_x_neg_position_x = 0;
box_text_x_neg_position_y = 0;
box_text_x_neg_position_z = 0;

//print some text on box y positive face

box_text_y_pos_exists = 0;
box_text_y_pos_type = 1; // 0= raised 1= sunken
box_text_y_pos_string = ["text ypos1",
                         "text ypos2",
                         "text ypos3",
                         "text ypos4",
                         "text ypos5",
                         "text ypos6"];
box_text_y_pos_size = 10;
box_text_y_pos_font = "Arial:style=Bold";
box_text_y_pos_direction = "ltr"; //"ltr"(left-to-right),"rtl"(right-to-left),
                                  //"ttb"(top-to-bottom), "btt"(bottom-to-top)
box_text_y_pos_halign = "center";  // "left", "center", "right"
box_text_y_pos_valign = "center";  // "left", "center", "right"
box_text_y_pos_spacing = 1.0;
box_text_y_pos_no_of_lines = 5; // 1-6 or no of strings in array
box_text_y_pos_line_spacing = 10;
box_text_y_pos_extrude_height = 2;
box_text_y_pos_rotation_x = 0;
box_text_y_pos_rotation_y = 0;
box_text_y_pos_rotation_z = 0;
box_text_y_pos_position_x = 0;
box_text_y_pos_position_y = 0;
box_text_y_pos_position_z = 0;

//print some text on box y negitive face

box_text_y_neg_exists = 0;
box_text_y_neg_type = 1; // 0= raised 1= sunken
box_text_y_neg_string = ["text yneg1",
                         "text yneg2",
                         "text yneg3",
                         "text yneg4",
                         "text yneg5",
                         "text yneg6"];
box_text_y_neg_size = 10;
box_text_y_neg_font = "DejaVu Sans      Mono:style=Bold";
box_text_y_neg_direction = "ltr"; //"ltr"(left-to-right),"rtl"(right-to-left),
                                  //"ttb"(top-to-bottom), "btt"(bottom-to-top)
box_text_y_neg_halign = "center";  // "left", "center", "right"
box_text_y_neg_valign = "center";  // "left", "center", "right"
box_text_y_neg_spacing = 1.0;
box_text_y_neg_no_of_lines = 5; // 1-6 or no of strings in array
box_text_y_neg_line_spacing = 10;
box_text_y_neg_extrude_height = 2;
box_text_y_neg_rotation_x = 0;
box_text_y_neg_rotation_y = 0;
box_text_y_neg_rotation_z = 0;
box_text_y_neg_position_x = 0;
box_text_y_neg_position_y = 0;
box_text_y_neg_position_z = 0;

//print some text on box base

box_text_base_exists = 0;
box_text_base_type = 0; // 0= raised 1= sunken
box_text_base_string =  ["1.0  1.5  2.0  2.5  3.0  3.5",
                         "4.0  4.5  5.0  5.5  6.0  7.0",
                         ];
box_text_base_size = 5;
box_text_base_font = "DejaVu Sans      Mono:style=Bold";
box_text_base_direction = "ltr"; //"ltr"(left-to-right),"rtl"(right-to-left),
                                  //"ttb"(top-to-bottom), "btt"(bottom-to-top)
box_text_base_halign = "center";  // "left", "center", "right"
box_text_base_valign = "center";  // "left", "center", "right"
box_text_base_spacing = 1.0;
box_text_base_no_of_lines = 5; // 1-6 or no of strings in array
box_text_base_line_spacing = 24;
box_text_base_extrude_height = 1.0;
box_text_base_rotation_x = 0;
box_text_base_rotation_y = 0;
box_text_base_rotation_z = 0;
box_text_base_position_x = 0;
box_text_base_position_y = -44;
box_text_base_position_z = 0;

//print some text on lid

lid_text_exists = 0;
lid_text_type = 1; // 0= raised 1= sunken
lid_text_to_print_1 = ["ER 11",
                      ];
lid_text_size = 15;
lid_text_font = "Arial:style=Black";
lid_text_direction = "ltr";  //"ltr"(left-to-right),"rtl"(right-to-left),
                               //"ttb"(top-to-bottom), "btt"(bottom-to-top) 
lid_text_halign = "center";  // "left", "center", "right"
lid_text_valign = "center";  // "left", "center", "right"
lid_text_spacing = 1;
lid_text_no_of_lines = 3; // 1-6 or no of strings in array
lid_text_line_spacing =8;
lid_text_extrude_height =2;
lid_text_rotation_x = 180;
lid_text_rotation_y = 0;
lid_text_rotation_z = 0;
lid_text_position_x = 0;
lid_text_position_y = -5;
lid_text_position_z = 0;

//print some text on sliding lid

sliding_lid_text_exists = 0;
sliding_lid_text_type = 0; // 0= raised 1= sunken
sliding_lid_text_to_print = [["text","on","lid1"],
                            ["text","on","lid2"],
                            ["text","on","lid3"],
                            ["text","on","lid4"],
                            ["text","on","lid5"],
                            ["text","on","lid6"]];
                        
sliding_lid_text_size = 4;
sliding_lid_text_font = "DejaVu Sans      Mono:style=Bold";
sliding_lid_text_direction = "ltr";  //"ltr"(left-to-right),"rtl"(right-to-left),
                               //"ttb"(top-to-bottom), "btt"(bottom-to-top) 
sliding_lid_text_halign = "center";  // "left", "center", "right"
sliding_lid_text_valign = "center";  // "left", "center", "right"
sliding_lid_text_spacing = 1;
sliding_lid_text_no_of_lines = 3; // 1-6 or no of strings in array
sliding_lid_text_line_spacing =6;
sliding_lid_text_extrude_height =2;
sliding_lid_text_rotation_x = 0;
sliding_lid_text_rotation_y = 0;
sliding_lid_text_rotation_z = 90;
sliding_lid_text_position_x = -6;
sliding_lid_text_position_y = 0;
sliding_lid_text_position_z = 0;


// cant find a way of indirectly calling modules by name
// so done using if statement
// these are all fixed size dedicated objects, no parameters
//
//1 calls "generate_motor_mount()"
//2 calls "3_axis_nano_board()"
//3 calls "small_limit_switch()"

//import an object from library
import_fixture_1_exists = 0;
fixture_1 = 1; //pick from list
fixture_1_pos_x = 25;
fixture_1_pos_y = -37.5;
fixture_1_pos_z = 5;
fixture_1_rot_x = 0;
fixture_1_rot_y = 0;
fixture_1_rot_z = -45;

//import an object from library
import_fixture_2_exists = 0;
fixture_2 = 2; //pick from list
fixture_2_pos_x = -25;
fixture_2_pos_y = 0;
fixture_2_pos_z = 0;
fixture_2_rot_x = 0;
fixture_2_rot_y = 0;
fixture_2_rot_z = -90;

//import an object from library
import_fixture_3_exists = 0;
fixture_3 = 3; //pick from list
fixture_3_pos_x = 24;
fixture_3_pos_y = 25;
fixture_3_pos_z = 18;
fixture_3_rot_x = 0;
fixture_3_rot_y = 0;
fixture_3_rot_z = 180;

//import an object from library
import_fixture_4_exists = 0;
fixture_4 = 4; //pick from list
fixture_4_pos_x = 0;
fixture_4_pos_y = 0;
fixture_4_pos_z = 0;
fixture_4_rot_x = 0;
fixture_4_rot_y = 0;
fixture_4_rot_z = 0;

//import an object from library
import_fixture_5_exists = 0;
fixture_5 = 5; //pick from list
fixture_5_pos_x = 0;
fixture_5_pos_y = 0;
fixture_5_pos_z = 0;
fixture_5_rot_x = 0;
fixture_5_rot_y = 0;
fixture_5_rot_z = 0;

/////////////////////////////////////////////////////////
//add a terminal block
/////////////////////////////////////////////////////////

term_block_exists = 0;
term_block_type = "screw_terminal_strip";
//"stud_terminal_strip","screw_terminal_strip"
term_block_rot_x = 0;
term_block_rot_y = 0;
term_block_rot_z = 0;
term_block_pos_x = 0;
term_block_pos_y = 0;
term_block_pos_z = 0;

////////////////////////////////////////////////////////
//parameters for stud type terminal block
////////////////////////////////////////////////////////

//define the size of the hex head bolt to be used for stud
stud_thread_size = 3.0;

//define the clearance from the base to the end of the hex bolt
stud_head_clearance = 1;

// define the base size of the stud termination block
stud_base_length = 60;
stud_base_width = 20;
stud_base_height = 10;

// declare barrier parameters for stud type connections
stud_qty = 5;
stud_fixing_y_shift = 0 ;
stud_barrier_spacing = 8.5;
stud_barrier_length = 12;
stud_barrier_width = 1.5;
stud_barrier_height = 6;
stud_barrier_x_move = 0;
stud_barrier_y_move = 0;

// declare fixing holes for mounting terminal rail to something
stud_location_exists = 0; //0 or 1

////////////////////////////////////////////////////////
//PARAMETERS FOR TERMINAL TYPE FIXING
////////////////////////////////////////////////////////

// define the base size of the stud termination block
strip_base_length = 0;              // base block length
strip_base_width= 0;                // base block width
strip_base_height= 0;               // base block height

//define the terminal strip
strip_width = 12;                   // width of terminal block
strip_height = 10;                  // height of terminal block
strip_qty = 4;                      // number of terminations
strip_spacing = 10;                 // spacing between terminations
strip_hole_dia = 3.8;               // screw terminal hole diameter
strip_hole_centres = 6;             // centre of terminal screw holes
strip_hole_depth = 6;               // depth of screw holes from terminal top
strip_barrel_width = 4.8;           // width of the termination hole
strip_barrel_cube_height = 3.4;     // height of rectangular section from cylinder middle
strip_barrel_from_plinth =1.5;      // height of the complete barrel hole from base


//add a bearing groove to the box

box_ball_groove_exists = 0;
box_ball_groove_size = 4;
box_ball_groove_dia = 60;
box_ball_groove_x_move = 25;
box_ball_groove_y_move = 0;
box_ball_groove_z_move = 5.0;

//add a bearing groove to the lid

lid_ball_groove_exists = 0;
lid_ball_groove_size = 4;
lid_ball_groove_dia = 60;
lid_ball_groove_x_move = 25;
lid_ball_groove_y_move = 0;
lid_ball_groove_z_move = 5.0;

//add a coin slot / bezel to lid
lid_coin_slot_exists = 0;
lid_coin_slot_length = 30;
lid_coin_slot_width = 60;
lid_coin_slot_surround = 5;
lid_coin_slot_lip = 2;
lid_coin_slot_x_move = -35;
lid_coin_slot_y_move = 0;
lid_coin_slot_rotation = 0;


/////////////////////////////////////////////////////////
//put holes where you want them
/////////////////////////////////////////////////////////
//define holes
/*


[0]  = [ hole exists: 0=no 1= exists ]
[1]  = [ hole size relates to metric screw: 0=actual 1=thread clearance ]
[2]  = [ size of hole in mm: number ]
[3]  = [ length of hole:number ]

[4]  = [ rectangular array of holes exist: 0=no 1=exists ]
[5]  = [ x axis qty of holes: number ]
[6]  = [ x axis pitch of holes: number ]
[7]  = [ y axis qty of holes: number ]
[8]  = [ y axis pitch of holes: number ]
[9]  = [ rectangular array outside holes only: 0= all holes 1=outside only ]

[10] = [ axial circular array of holes exist: 0=no 1=exists ]
[11] = [ radial circular array of holes exist: 0=no 1=exists ]
[12] = [ qty of holes in circular array: number ]
[13] = [ P.C.D. of circular array of holes: number ]
[13] = [ span of holes: number ]

[15] = [ recess exists on hole: 0=no 1=exists ]
[16] = [ recess type: 0=countersink 1=counterbore ]
[17] = [ counterbore depth: number ]
[18] = [ counterbore height offset: number ]
 
[19] = [ nut retainer hole exists on hole: 0=no 1=exists ]
[20] = [ nut retainer depth: number ]
[21] = [ nut retainer height offset: number ]
[22] = [ nut rotation around axis : number ]

[23] = [ thread on hole exists: 0=no 1=exists ]
[24] = [ pitch of thread: number ]
[25] = [ depth of thread: number ]
[26] = [ angle of thread: number ]
[27] = [ thread height offset: number ]

[28] = [ nut retaining slot exists: 0=no 1=exists ]
[29] = [ nut retaining slot x dimention: number ]
[30] = [ nut retaining slot y dimention: number ]
[31] = [ nut retaining slot z dimention: number ]
[32] = [ nut retaining slot height offset: number ]

[33] = [ translation in x direction: number ]
[34] = [ translation in y direction: number ]
[35] = [ translation in z direction: number ]
[36] = [ rotation in x plane: number ]
[37] = [ rotation in y plane: number ]
[38] = [ rotation in z plane: number ]

*/
//define box holes,holes_from_array decodes holes
box_holes_exist = 1;
box_first_hole = 0;
box_last_hole = 7;
//define lid holes
lid_holes_exist = 1;
lid_first_hole = 10;
lid_last_hole = 11;

hole =
[ //start array

// hole       rect           circ         recess   nut         thread      slot        position

//hole 0 spindle clearance hole
[1,0,50,10,    0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0, 0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  40,0,0,0,0,0],
//hole 1 spindle to box fixing holes
[1,1,6,10,     1,1,0,2,78,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  40,0,0,0,0,0],
//hole 2 fixing holes to plate
[1,1,6,10,     1,2,160,2,80,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 3 pulley to control box fixings 1
[1,1,3,10,     1,2,165,2,30,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,-50,30,90,0,0],
//hole 4 pulley to control box fixing 2
[1,1,3,10,     1,1,0,2,30,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  10,-50,30,90,0,0],
//hole 5 earth cable access hole
[1,0,10,10,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  50,-50,30,90,0,0],
//hole 6 front lid fixings
[1,0,2,15,     1,2,66,2,102,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  58,0,37,0,0,0],
//hole 7 rear lid fixings
[1,0,2,15,     1,2,108,2,102,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  -37,0,37,0,0,0],
//hole 8
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 9
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 10 small lid to box fixing holes
[1,0,3,15,     1,2,66,2,102,0,   0,0,0,0,0,   1,1,4,-11,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 11 spindle clearanc hole
[1,0,26,5,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  18,0,0,0,0,0],
//hole 12
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 13
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 14
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 15
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 16
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 17
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 18
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 19
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 20
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],
//hole 21
[0,0,0,0,     0,0,0,0,0,0,   0,0,0,0,0,   0,0,0,0,  0,0,0,0,   0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,0],

]; //end array , add more if required


/////////////////////////////////////////////////////////
//run the rendering
/////////////////////////////////////////////////////////

if (lid_print==1)
{    
    if (show_assembled==1)
    {    

        difference()
            {
            union()
                {
                    if (sliding_lid_exists==0)
                    {
                        //draw the junction box lid
                        translate([0,-((lid_hinge_shaft_height+(lid_hinge_width/2))*flange_hinge_exists)/2,
                                    box_height + lid_height + assembled_split])
                        rotate ([180,0,0])   
                        lid_stage_3_build();
                    }
                    else
                    {
                        //draw the sliding box lid
                        translate([0,0,box_height + assembled_split-box_flange_thickness])
                        rotate ([0,0,0])   
                        sliding_lid_build();
                    }

                }
                //last final subtraction after everything built
                //nothing yet to do
            }
        }
    else
    {
        if (flip_lid_for_print==1)
        {
            difference()
                {
                union()
                    {
                        if (sliding_lid_exists==0)
                        {
                            //draw the junction box lid
                            translate([0,lid_base_print_seperation,lid_height])
                            rotate([180,0,0])
                            lid_stage_3_build();
                        }
                        else
                        {
                            //draw the sliding box lid
                            translate([0,lid_base_print_seperation,0])
                            rotate ([0,0,0])   
                            sliding_lid_build();
                        }

                    }
                    //last final subtraction after everything built
                //nothing yet to do
                }
        }
        else
        {
            difference()
                {
                union()
                    {
                        if (sliding_lid_exists==0)
                        {
                            //draw the  box lid
                            translate([0,lid_base_print_seperation,0])
                            rotate([0,0,0])    
                            lid_stage_3_build();
                        }
                        else
                        {
                            //draw the sliding box lid
                            translate([0,lid_base_print_seperation,0])
                            rotate([0,0,0])    
                            sliding_lid_build();
                        }
                    }
                    
                    //last final subtraction after everything built
                    //nothing yet to do
                }    
        }    
    }
}

//draw the junction box base
if ((box_print==1)||(show_assembled==1))
{    
    translate([0,0,0])
    box_stage_3_build();
}  




//////////////////////////////////////////////////////////////
// build the basic box shape and hollow out
//////////////////////////////////////////////////////////////

module  box_stage_1_build()
{
    difference()
    {
        union()
        {
            //draw the box base 
            rounded_box(box_length,
                        box_width,
                        box_height,
                        box_corner_radius,
                        box_top_radius_exists);
            
            //add plinth if required
            
            if (box_plinth_exists == 1)
            {
                if (box_hollow_simple_polygon_exists == 1)
                {
                    translate([0,0,box_height])
                    internal_shape(box_poly_x_near-box_plinth_clearance,
                                    box_poly_x_mid-box_plinth_clearance,
                                    box_poly_x_far-box_plinth_clearance,
                                    box_poly_x_ext-box_plinth_clearance,
                                    box_poly_y_near-box_plinth_clearance,
                                    box_poly_y_mid-box_plinth_clearance,
                                    box_poly_y_far-box_plinth_clearance, 
                                    box_plinth_thickness);
                }
                else if (box_hollow_full_polygon_exists == 1)
                {
                    
                    
                    
                }
                else //simple plinth shape
                {
                    translate([0,0,box_height+0.1])
                    rounded_box(box_length-(box_hollow_wall_thickness*2)-box_plinth_clearance,
                                box_width-(box_hollow_wall_thickness*2)-box_plinth_clearance,
                                box_plinth_thickness,
                                box_corner_radius,
                                0);   
                }

            }
           

        }//end of union
            
            //remove bits from the box
            
            //hollow out the box
            if (box_simple_hollow_exists==1)
            {
                if(box_hollow_simple_polygon_exists==1)
                {
                    translate([0,0,box_base_thickness])
                    internal_shape( box_poly_x_near,
                                    box_poly_x_mid,
                                    box_poly_x_far,
                                    box_poly_x_ext,
                                    box_poly_y_near,
                                    box_poly_y_mid,
                                    box_poly_y_far,
                                    (box_height-box_base_thickness));
                }
                
                else if (box_hollow_full_polygon_exists == 1)
                {
                    translate([0,0,box_base_thickness])
                    linear_extrude(box_height-box_base_thickness+0.05)  
                    polygon (box_hollow_full_poly_points);  
                }
                
                else //basic rounded hollow
                {
                    translate([0,0,box_base_thickness+0.1])
                    rounded_box(box_length-box_hollow_wall_thickness*2,
                                box_width-box_hollow_wall_thickness*2,
                                box_height-box_base_thickness,
                                box_corner_radius,
                                box_top_radius_exists);
                }
            }
            
            //remove clearance on box plinth for lid pillers
            if ((box_plinth_exists == 1) && (lid_pillers_1_exist==1))
            {
                translate([lid_pillers_1_shift_x,lid_pillers_1_shift_y,box_height])    
                cylinder_array(lid_pillers_1_dia +box_plinth_clearance,
                               box_plinth_thickness+0.15,
                               lid_pillers_1_x_qty,
                               lid_pillers_1_y_qty,
                               lid_pillers_1_x_pitch,
                               lid_pillers_1_y_pitch,
                               lid_pillers_1_outer_only);
   
                
            }
            
            //remove clearance on box plinth for lid pillers
            if ((box_plinth_exists == 1) && (lid_pillers_2_exist==1))
            {
                translate([lid_pillers_2_shift_x,lid_pillers_2_shift_y,box_height])    
                cylinder_array(lid_pillers_2_dia +box_plinth_clearance,
                               box_plinth_thickness+0.15,
                               lid_pillers_2_x_qty,
                               lid_pillers_2_y_qty,
                               lid_pillers_2_x_pitch,
                               lid_pillers_2_y_pitch,
                               lid_pillers_2_outer_only);
   
                
            }
        }//end difference  
}

////////////////////////////////////////////////////////
//box stage 2 build
////////////////////////////////////////////////////////

module box_stage_2_build()
{

        difference()
        {
            union()
            {
                //draw the box base 
                box_stage_1_build();
                
                //add a lip to the box top
                if (box_flange_exists == 1)
                {
                translate([0,0,box_height-box_flange_thickness])    
                flange_plate(box_length,
                             box_width,
                             box_flange_thickness,
                             box_corner_radius,
                             box_flange_opening_X,
                             box_flange_opening_Y,
                             box_top_opening_pos_X,
                             box_top_opening_pos_Y);
                }
                
                //add an array of pillers to the box
                if (box_pillers_1_exist == 1)
                {
                translate([box_pillers_1_shift_x,box_pillers_1_shift_y,
                            box_pillers_1_shift_z+(box_corner_radius*box_top_radius_exists)])    
                cylinder_array(box_pillers_1_dia,
                               box_pillers_1_length-(box_corner_radius*box_top_radius_exists),
                               box_pillers_1_x_qty,
                               box_pillers_1_y_qty,
                               box_pillers_1_x_pitch,
                               box_pillers_1_y_pitch,
                               box_pillers_1_outer_only);
                }
   
                //add an array of pillers to the box
                if (box_pillers_2_exist == 1)
                {
                translate([box_pillers_2_shift_x,box_pillers_2_shift_y,
                            box_pillers_2_shift_z+(box_corner_radius*box_top_radius_exists)])    
                cylinder_array(box_pillers_2_dia,
                               box_pillers_2_length-(box_corner_radius*box_top_radius_exists),
                               box_pillers_2_x_qty,
                               box_pillers_2_y_qty,
                               box_pillers_2_x_pitch,
                               box_pillers_2_y_pitch,
                               box_pillers_2_outer_only);
                }
                
                
                //add box seperation dividers to box
                if (box_divider_exist == 1)
                {
                translate([0,0,0])    
                box_dividers(box_divider_x_qty,
                             box_divider_x_pitch,
                             box_divider_x_length,
                             box_divider_x_thickness,
                             box_divider_x_height,
                             box_divider_y_qty,
                             box_divider_y_pitch,
                             box_divider_y_length,
                             box_divider_y_thickness,
                             box_divider_y_height);
                }
              
                //add an array of PCB mounting slots
                if (pcb_cradle_exists == 1)
                {
                translate([pcb_cradle_pos_x,pcb_cradle_pos_y,pcb_cradle_pos_z])
                rotate ([pcb_cradle_rot_x,pcb_cradle_rot_y,pcb_cradle_rot_z])
                pcb_cradle_array (pcb_cradle_qty,
                                  pcb_cradle_pitch,
                                  pcb_board_length,
                                  pcb_board_width,
                                  pcb_board_thickness,
                                  pcb_cradle_length,
                                  pcb_cradle_width,
                                  pcb_cradle_thickness,
                                  pcb_cradle_base_depth,
                                  pcb_cradle_opening_length);
                }

                
               

                //add box backplate pads 1 to bottom
                if (box_mount_1_pads_exist == 1)
                {
                translate([0,0,0])    
                base_pads(box_mount_1_pads_length,
                               box_mount_1_pads_width,
                               box_mount_1_pads_height,
                               box_mount_1_pads_x_qty,
                               box_mount_1_pads_x_pitch,
                               box_mount_1_pads_y_qty,
                               box_mount_1_pads_y_pitch,
                               box_mount_1_pads_x_move,
                               box_mount_1_pads_y_move);
                }
                
                //add box backplate pads 2 to bottom
                if (box_mount_2_pads_exist == 1)
                {
                translate([0,0,0])    
                base_pads(box_mount_2_pads_length,
                               box_mount_2_pads_width,
                               box_mount_2_pads_height,
                               box_mount_2_pads_x_qty,
                               box_mount_2_pads_x_pitch,
                               box_mount_2_pads_y_qty,
                               box_mount_2_pads_y_pitch,
                               box_mount_2_pads_x_move,
                               box_mount_2_pads_y_move);
                }
                
                //add sheet filter holder 1 to one of the box sides
                if (box_filter_1_exists == 1)
                {
                    translate ([box_filter_1_x_pos,box_filter_1_y_pos,box_filter_1_z_pos])
                    make_filter(box_filter_1_side,
                                box_filter_1_length,
                                box_filter_1_width,
                                box_filter_1_thickness,
                                box_filter_1_wall_thickness,
                                box_filter_1_retain_lip,
                                box_filter_1_stop_thickness);
                }
                 
                //add sheet filter holder 2 to one of the box sides
                if (box_filter_2_exists == 1)
                {
                    translate ([box_filter_2_x_pos,box_filter_2_y_pos,box_filter_2_z_pos])
                    make_filter(box_filter_2_side,
                                box_filter_2_length,
                                box_filter_2_width,
                                box_filter_2_thickness,
                                box_filter_2_wall_thickness,
                                box_filter_2_retain_lip,
                                box_filter_2_stop_thickness);
                    
                } //end if exist 
                
            //remove fan 1 from the box
            if (box_fan_1_exists == 1)
            {
                translate([box_fan_1_x_pos,box_fan_1_y_pos,box_fan_1_z_pos])    
                fan_cutout (box_height,
                            box_fan_1_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                            box_fan_1_dia,
                            box_fan_1_fix_dia,
                            box_fan_1_fixing_square,
                            box_fan_1_depth);
            }
            
            //remove fan 2 from the box
            if (box_fan_2_exists == 1)
            {
                translate([box_fan_2_x_pos,box_fan_2_y_pos,box_fan_2_z_pos])    
                fan_cutout (box_height,
                            box_fan_2_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                            box_fan_2_dia,
                            box_fan_2_fix_dia,
                            box_fan_2_fixing_square,
                            box_fan_2_depth);
            } 
                
                //add hinge block to the box top
                if (flange_hinge_exists == 1)
                {
                    //shift the hinge where you want it
                    translate([box_hinge_pos_x,
                    (box_width/2)-(box_hinge_width/2)+box_hinge_pos_y,
                    box_hinge_pos_z+box_height])
                    rotate ([box_hinge_rot_x,box_hinge_rot_y,box_hinge_rot_z])
                    //position the hinge on the box
                    hinge_block(box_hinge_type,
                                box_hinge_length,
                                box_hinge_width,
                                box_hinge_shaft_height,
                                box_hinge_screw_size,
                                box_hinge_cbore_length,
                                hinge_joint_position);
                }
                
                //add angled side hinge to box
                if (side_hinges_exist ==1)
                {
                    translate([0,box_width/2+side_hinge_center_dist,box_height])
                    rotate ([0,0,0])
                    box_side_hinge();
                }
                
            }//end of union
            
            // remove the x positive side recess
            if (box_x_pos_recess_exists==1)
            {
                translate([box_length/2,box_x_pos_recess_move_y,-0.1])
                rotate([0,0,90])
                rounded_slot(box_x_pos_recess_length,
                             box_height,
                             box_x_pos_recess_radius);
            }
                
            // remove the x negative side recess
            if (box_x_neg_recess_exists==1)
            {
                translate([-box_length/2,box_x_neg_recess_move_y,-0.1])
                rotate([0,0,90])
                rounded_slot(box_x_pos_recess_length,
                             box_height,
                             box_x_neg_recess_radius);
            } 
            
            // remove the y positive side recess
            if (box_y_pos_recess_exists==1)
            {
                translate([box_y_pos_recess_move_x,box_width/2,-0.1])
                rotate([0,0,0])
                rounded_slot(box_y_pos_recess_length,
                             box_height,
                             box_y_pos_recess_radius);
            }
                
            // remove the y negative side recess
            if (box_y_neg_recess_exists==1)
            {
                translate([box_y_neg_recess_move_x,-box_width/2,-0.1])
                rotate([0,0,0])
                rounded_slot(box_y_neg_recess_length,
                             box_height,
                             box_y_neg_recess_radius);
            }

            if (sliding_lid_exists==1)
            {
                for(i=[0:sliding_lid_qty-1])
                {
                translate([sliding_lid_pitch*i-((sliding_lid_pitch*(sliding_lid_qty-1))/2),
                           -((box_width/2)-box_poly_y_far)/2,box_height-box_flange_thickness])
                rotate([0,0,0])
                sliding_lid_shape((box_width/2)+box_poly_y_far,
                                  sliding_lid_width,
                                  box_flange_thickness);
                }   
            } 
                
            


            //remove box cutout 1
            if (box_cutout_1_exists==1)
            {
               translate([box_cutout_1_x_pos,box_cutout_1_y_pos,box_cutout_1_z_pos])
               make_cutout (box_height,
                            box_cutout_1_side,
                            box_cutout_1_x_qty,
                            box_cutout_1_x_pitch,
                            box_cutout_1_y_qty,
                            box_cutout_1_y_pitch,
                            box_cutout_1_x_dim,
                            box_cutout_1_y_dim,
                            box_cutout_1_depth);  
            }
             
            //remove box cutout 2
            if (box_cutout_2_exists==1)
            {
               translate([box_cutout_2_x_pos,box_cutout_2_y_pos,box_cutout_2_z_pos])
               make_cutout (box_height,
                            box_cutout_2_side,
                            box_cutout_2_x_qty,
                            box_cutout_2_x_pitch,
                            box_cutout_2_y_qty,
                            box_cutout_2_y_pitch,
                            box_cutout_2_x_dim,
                            box_cutout_2_y_dim,
                            box_cutout_2_depth);  
            }
           
             //remove box cutout 3
            if (box_cutout_3_exists==1)
            {
               translate([box_cutout_3_x_pos,box_cutout_3_y_pos,box_cutout_3_z_pos])
               make_cutout (box_height,
                            box_cutout_3_side,
                            box_cutout_3_x_qty,
                            box_cutout_3_x_pitch,
                            box_cutout_3_y_qty,
                            box_cutout_3_y_pitch,
                            box_cutout_3_x_dim,
                            box_cutout_3_y_dim,
                            box_cutout_3_depth); 
            }           

              //remove box cutout 4
            if (box_cutout_4_exists==1)
            {
               translate([box_cutout_4_x_pos,box_cutout_4_y_pos,box_cutout_4_z_pos])
               make_cutout (box_height,
                            box_cutout_4_side,
                            box_cutout_4_x_qty,
                            box_cutout_4_x_pitch,
                            box_cutout_4_y_qty,
                            box_cutout_4_y_pitch,
                            box_cutout_4_x_dim,
                            box_cutout_4_y_dim,
                            box_cutout_4_depth); 
            } 
            
            //remove box cutout 5
            if (box_cutout_5_exists==1)
            {
               translate([box_cutout_5_x_pos,box_cutout_5_y_pos,box_cutout_5_z_pos])
               make_cutout (box_height,
                            box_cutout_5_side,
                            box_cutout_5_x_qty,
                            box_cutout_5_x_pitch,
                            box_cutout_5_y_qty,
                            box_cutout_5_y_pitch,
                            box_cutout_5_x_dim,
                            box_cutout_5_y_dim,
                            box_cutout_5_depth);  
            }
            
 
            //remove box vents 1
            if (box_vents_1_exists==1)
            {
               translate([box_vents_1_x_pos,box_vents_1_y_pos,box_vents_1_z_pos])
               make_grills (box_height,
                            box_vents_1_side,
                            box_vents_1_x_qty,
                            box_vents_1_x_pitch,
                            box_vents_1_y_qty,
                            box_vents_1_y_pitch,
                            box_vents_1_width,
                            box_vents_1_height,
                            box_vents_1_depth,
                            box_vents_1_radius);  
            }
            
             //remove box vents 2
            if (box_vents_2_exists==1)
            {
               translate([box_vents_2_x_pos,box_vents_2_y_pos,box_vents_2_z_pos])
               make_grills (box_height,
                            box_vents_2_side,
                            box_vents_2_x_qty,
                            box_vents_2_x_pitch,
                            box_vents_2_y_qty,
                            box_vents_2_y_pitch,
                            box_vents_2_width,
                            box_vents_2_height,
                            box_vents_2_depth,
                            box_vents_2_radius);  
            }
            
            //remove box vents 3
            if (box_vents_3_exists==1)
            {
               translate([box_vents_3_x_pos,box_vents_3_y_pos,box_vents_3_z_pos])
               make_grills (box_height,
                            box_vents_3_side,
                            box_vents_3_x_qty,
                            box_vents_3_x_pitch,
                            box_vents_3_y_qty,
                            box_vents_3_y_pitch,
                            box_vents_3_width,
                            box_vents_3_height,
                            box_vents_3_depth,
                            box_vents_3_radius);  
            }

            //remove box vents 4
            if (box_vents_4_exists==1)
            {
               translate([box_vents_4_x_pos,box_vents_4_y_pos,box_vents_4_z_pos])
               make_grills (box_height,
                            box_vents_4_side,
                            box_vents_4_x_qty,
                            box_vents_4_x_pitch,
                            box_vents_4_y_qty,
                            box_vents_4_y_pitch,
                            box_vents_4_width,
                            box_vents_4_height,
                            box_vents_4_depth,
                            box_vents_4_radius);  
            }
         
            //remove box vents 5
            if (box_vents_5_exists==1)
            {
               translate([box_vents_5_x_pos,box_vents_5_y_pos,box_vents_5_z_pos])
               make_grills (box_height,
                            box_vents_5_side,
                            box_vents_5_x_qty,
                            box_vents_5_x_pitch,
                            box_vents_5_y_qty,
                            box_vents_5_y_pitch,
                            box_vents_5_width,
                            box_vents_5_height,
                            box_vents_5_depth,
                            box_vents_5_radius);  
            }
            
            if (o_ring_selected==1)
            {
                translate([0,0,box_height])
                o_ring_insertion();   
            }
 
            
            //put a mesh on the x positive face of box
            if (box_mesh_x_pos_side_exists==1)
            {
                   translate ([box_poly_x_ext+box_mesh_x_pos_depth_offset,0,box_height/2])
                    rotate ([90,0,90])                    
                make_mesh (box_mesh_x_pos_type, //0=circles,3 up = polygon sides 1&2 invalid
                           box_mesh_x_pos_dia,
                           box_mesh_x_pos_rotation,
                           box_mesh_x_pos_horz_qty,
                           box_mesh_x_pos_horz_pitch,
                           box_mesh_x_pos_vert_qty,
                           box_mesh_x_pos_vert_pitch,
                           box_mesh_x_pos_depth,
                           box_mesh_x_pos_offset_req);
            }
            
            //put a mesh on the x negitive face of box
            if (box_mesh_x_neg_side_exists==1)
            {
                   translate ([-box_poly_x_ext-box_mesh_x_neg_depth_offset,0,box_height/2])
                    rotate ([90,0,270])                    
                make_mesh (box_mesh_x_neg_type, //0=circles,3 up = polygon sides 1&2 invalid
                           box_mesh_x_neg_dia,
                           box_mesh_x_neg_rotation,
                           box_mesh_x_neg_horz_qty,
                           box_mesh_x_neg_horz_pitch,
                           box_mesh_x_neg_vert_qty,
                           box_mesh_x_neg_vert_pitch,
                           box_mesh_x_neg_depth,
                           box_mesh_x_neg_offset_req);
            }
            
            //put a mesh on the y positive face of box
            if (box_mesh_y_pos_side_exists==1)
            {
                   translate ([0,box_poly_y_far+box_mesh_y_pos_depth_offset,box_height/2])
                    rotate ([90,0,180])                    
                make_mesh (box_mesh_y_pos_type, //0=circles,3 up = polygon sides 1&2 invalid
                           box_mesh_y_pos_dia,
                           box_mesh_y_pos_rotation,
                           box_mesh_y_pos_horz_qty,
                           box_mesh_y_pos_horz_pitch,
                           box_mesh_y_pos_vert_qty,
                           box_mesh_y_pos_vert_pitch,
                           box_mesh_y_pos_depth,
                           box_mesh_y_pos_offset_req);
            }
            
            //put a mesh on the y negitive face of box
            if (box_mesh_y_neg_side_exists==1)
            {
                   translate ([0,-box_poly_y_far-box_mesh_y_neg_depth_offset,box_height/2])
                    rotate ([90,0,0])                    
                make_mesh (box_mesh_y_neg_type, //0=circles,3 up = polygon sides 1&2 invalid
                           box_mesh_y_neg_dia,
                           box_mesh_y_neg_rotation,
                           box_mesh_y_neg_horz_qty,
                           box_mesh_y_neg_horz_pitch,
                           box_mesh_y_neg_vert_qty,
                           box_mesh_y_neg_vert_pitch,
                           box_mesh_y_neg_depth,
                           box_mesh_y_neg_offset_req);

            }
            
            //put a mesh on the bottom face of box
            #if (box_mesh_base_side_exists==1)
            {
                   translate ([box_mesh_base_x_shift,box_mesh_base_y_shift,box_mesh_base_depth_offset])
                    rotate ([0,0,0])                    
                make_mesh (box_mesh_base_type, //0=circles,3 up = polygon sides 1&2 invalid
                           box_mesh_base_dia,
                           box_mesh_base_rotation,
                           box_mesh_base_horz_qty,
                           box_mesh_base_horz_pitch,
                           box_mesh_base_vert_qty,
                           box_mesh_base_vert_pitch,
                           box_mesh_base_depth,
                           box_mesh_base_offset_req);

            }
            
            //remove groove to take ball bearings
            if (box_ball_groove_exists == 1)
            {
                translate([box_ball_groove_x_move,box_ball_groove_y_move,box_ball_groove_z_move])
                {
                rotate_extrude()
                translate ([box_ball_groove_dia/2,0,0])
                circle (d=box_ball_groove_size);
                }
            }


        }//end difference
}//end module
    

////////////////////////////////////////////////////////
//box stage 3 build
//if required add entry boss 1-2, cable clamp, import stl file, 
//proud text 1-6 lines, import fixture 1-5, terminal strips
//
//then if required remove sunken text 1-6 lines,
//holes:-clearance, counterbored, nut holder, threaded 
////////////////////////////////////////////////////////

module box_stage_3_build()
{
        translate([0,0,0])
        rotate ([0,0,0])   
        difference()
            {
            union()
                {
                //import stage 2 build of the box
                box_stage_2_build();
                    
                //add stage 3 bits to go onto the box
                    
                // add entry boss 1 to box
                 if (box_boss_1_exists ==1)
                 {
                    translate ([box_boss_1_x_pos,box_boss_1_y_pos,box_boss_1_z_pos])
                    make_entry_boss(box_height,
                                    box_boss_1_side,
                                    box_boss_1_length,
                                    box_boss_1_width,
                                    box_boss_1_height,
                                    box_boss_1_radius); 
                 }
                 
                 // add entry boss 2 to box
                 if (box_boss_2_exists ==1)
                 {
                    translate ([box_boss_2_x_pos,box_boss_2_y_pos,box_boss_2_z_pos])
                    make_entry_boss(box_height,
                                    box_boss_2_side,
                                    box_boss_2_length,
                                    box_boss_2_width,
                                    box_boss_2_height,
                                    box_boss_2_radius); 
                 }
               
                // add entry boss 3 to box
                 if (box_boss_3_exists ==1)
                 {
                    translate ([box_boss_3_x_pos,box_boss_3_y_pos,box_boss_3_z_pos])
                    make_entry_boss(box_height,
                                    box_boss_3_side,
                                    box_boss_3_length,
                                    box_boss_3_width,
                                    box_boss_3_height,
                                    box_boss_3_radius); 
                 }
                 
                 // add entry boss 4 to box
                 if (box_boss_4_exists ==1)
                 {
                    translate ([box_boss_4_x_pos,box_boss_4_y_pos,box_boss_4_z_pos])
                    make_entry_boss(box_height,
                                    box_boss_4_side,
                                    box_boss_4_length,
                                    box_boss_4_width,
                                    box_boss_4_height,
                                    box_boss_4_radius); 
                 }
                 
                // add cable clamp 1 to box
                 if (box_clamp_1_exists ==1)
                 {
                    translate ([box_clamp_1_x_pos,box_clamp_1_y_pos,box_clamp_1_z_pos])
                    make_entry_clamp(box_clamp_1_side,
                                     box_clamp_1_length,
                                     box_clamp_1_width,
                                     box_clamp_1_height,
                                     box_clamp_1_grip_height,
                                     box_clamp_1_radius,
                                     box_clamp_1_cable_size,
                                     box_clamp_1_screw_hole,
                                     box_clamp_1_grip_hole); 
                 }
                 
                 // add entry clamp 2 to box
                 if (box_clamp_2_exists ==1)
                 {
                    translate ([box_clamp_2_x_pos,box_clamp_2_y_pos,box_clamp_2_z_pos])
                    make_entry_clamp(box_clamp_2_side,
                                     box_clamp_2_length,
                                     box_clamp_2_width,
                                     box_clamp_2_height,
                                     box_clamp_2_grip_height,
                                     box_clamp_2_radius,
                                     box_clamp_2_cable_size,
                                     box_clamp_2_screw_hole,
                                     box_clamp_2_grip_hole);
                 } 
                
                 
                // import .stl file to box 1
                if (box_import_1_stl_exists==1)
                {
                    translate([box_import_1_pos_x,box_import_1_pos_y,box_import_1_pos_z])
                    rotate([box_import_1_rot_x,box_import_1_rot_y,box_import_1_rot_z])
                    scale([box_import_1_scale_x,box_import_1_scale_y,box_import_1_scale_z])
                    import(box_import_1_stl_file);
                }
                
                //add some positive text to x positive face
                if ((box_text_x_pos_exists==1)&&(box_text_x_pos_type==0))
                {
                    translate ([box_length/2,0,box_height-box_text_x_pos_line_spacing])
                    rotate ([90,0,90])
                    box_text_x_pos_face();
                }
                
                //add some positive text to x negitive face
                if ((box_text_x_neg_exists==1)&&(box_text_x_neg_type==0))
                {
                    translate ([-box_length/2,0,box_height-box_text_x_neg_line_spacing])
                    rotate ([90,0,270])
                    box_text_x_neg_face();
                }
                
                //add some positive text to y positive face
                if ((box_text_y_pos_exists==1)&&(box_text_y_pos_type==0))
                {
                    translate ([0,box_width/2,box_height-box_text_y_pos_line_spacing])
                    rotate ([90,0,180])
                    box_text_y_pos_face();
                }
                
                //add some positive text to y negitive face
                if ((box_text_y_neg_exists==1)&&(box_text_y_neg_type==0))
                {
                    translate ([0,-box_width/2,box_height-box_text_y_neg_line_spacing])
                    rotate ([90,0,0])
                    box_text_y_neg_face();
                }
                
                //add some positive text to base
                if ((box_text_base_exists==1)&&(box_text_base_type==0))
                {
                    translate ([0,((box_text_base_line_spacing*box_text_base_no_of_lines)/2)-box_text_base_line_spacing/2,box_base_thickness])
                    rotate ([0,0,0])
                    box_text_base_face();
                }
                
                //add an object from library
                if (import_fixture_1_exists==1)
                {
                    translate([fixture_1_pos_x,fixture_1_pos_y,fixture_1_pos_z])
                    rotate ([fixture_1_rot_x,fixture_1_rot_y,fixture_1_rot_z])
                    {
                        if (fixture_1==1) { generate_motor_mount(); }
                        else if (fixture_1==2) { axis_nano_board(); }
                        else if (fixture_1==3) { small_limit_sw(); }
                        
                    }
                }
                
                //add an object from library
                if (import_fixture_2_exists==1)
                {
                    translate([fixture_2_pos_x,fixture_2_pos_y,fixture_2_pos_z])
                    rotate ([fixture_2_rot_x,fixture_2_rot_y,fixture_2_rot_z])
                    {
                        if (fixture_2==1) { generate_motor_mount(); }
                        else if (fixture_2==2) { axis_nano_board(); }
                        else if (fixture_2==3) { small_limit_sw(); }

                    }
                }
                
                //add an object from library
                if (import_fixture_3_exists==1)
                {
                    translate([fixture_3_pos_x,fixture_3_pos_y,fixture_3_pos_z])
                    rotate ([fixture_3_rot_x,fixture_3_rot_y,fixture_3_rot_z])
                    {
                        if (fixture_3==1) { generate_motor_mount(); }
                        else if (fixture_3==2) { axis_nano_board(); }
                        else if (fixture_3==3) { small_limit_sw(); }

                    }
                }
                
                //add an object from library
                if (import_fixture_4_exists==1)
                {
                    translate([fixture_4_pos_x,fixture_4_pos_y,fixture_4_pos_z])
                    rotate ([fixture_4_rot_x,fixture_4_rot_y,fixture_4_rot_z])
                    {
                        if (fixture_4==1) { generate_motor_mount(); }
                        else if (fixture_4==2) { axis_nano_board(); }
                        else if (fixture_4==3) { small_limit_sw(); }
                        
                    }
                }
                
                //add an object from library
                if (import_fixture_5_exists==1)
                {
                    translate([fixture_5_pos_x,fixture_5_pos_y,fixture_5_pos_z])
                    rotate ([fixture_5_rot_x,fixture_5_rot_y,fixture_5_rot_z])
                    {
                        if (fixture_5==1) { generate_motor_mount(); }
                        else if (fixture_5==2) { axis_nano_board(); }
                        else if (fixture_5==3) { small_limit_sw(); }
                        
                    }
                }
                
                
                //add a terminal strip
                translate([term_block_pos_x,term_block_pos_y,term_block_pos_z])
                rotate([term_block_rot_x,term_block_rot_y,term_block_rot_z])    
                if (term_block_exists==1)
                {
                    if (term_block_type == "stud_terminal_strip")
                    {
                        stud_terminal_strip(stud_thread_size,
                                            stud_head_clearance,
                                            stud_base_length,
                                            stud_base_width,
                                            stud_base_height,
                                            stud_qty,
                                            stud_fixing_y_shift,
                                            stud_barrier_spacing,
                                            stud_barrier_length,
                                            stud_barrier_width,
                                            stud_barrier_height,
                                            stud_barrier_x_move,
                                            stud_barrier_y_move,
                                            stud_location_exists);
                    }
                    
                    else if (term_block_type == "screw_terminal_strip")
                    {
                        screw_terminal_strip(strip_base_length,
                                            strip_base_width,
                                            strip_base_height,
                                            strip_qty,
                                            strip_spacing,
                                            strip_hole_dia,
                                            strip_hole_centres,
                                            strip_hole_depth,
                                            strip_width,
                                            strip_height,
                                            strip_barrel_width,
                                            strip_barrel_cube_height,
                                            strip_barrel_from_plinth);                      
                    }
                        
                    
                } 
 
                
                
                }//end union
 
                
                //subtract stage 3 bits of the box
                
                // remove cable clamp 1 hole through box
                 if (box_clamp_1_exists ==1)
                 {
                    translate ([box_clamp_1_x_pos,box_clamp_1_y_pos,box_clamp_1_z_pos])
                    make_clamp_hole(box_clamp_1_side,
                                    box_clamp_1_length,
                                    box_clamp_1_width,
                                    box_clamp_1_height,
                                    box_clamp_1_grip_height,
                                    box_clamp_1_radius,
                                    box_clamp_1_cable_size,
                                    box_clamp_1_screw_hole,
                                    box_clamp_1_grip_hole); 
                 }
                 
                // remove cable clamp 1 hole through box
                 if (box_clamp_2_exists ==1)
                 {
                    translate ([box_clamp_2_x_pos,box_clamp_2_y_pos,box_clamp_2_z_pos])
                    make_clamp_hole(box_clamp_2_side,
                                    box_clamp_2_length,
                                    box_clamp_2_width,
                                    box_clamp_2_height,
                                    box_clamp_2_grip_height,
                                    box_clamp_2_radius,
                                    box_clamp_2_cable_size,
                                    box_clamp_2_screw_hole,
                                    box_clamp_2_grip_hole);
                 } 
                //add some negitive text
                if ((box_text_x_pos_exists==1)&&(box_text_x_pos_type==1))
                {
                   translate ([box_length/2-box_text_x_pos_extrude_height,0,box_height-box_text_x_pos_line_spacing])
                    rotate ([90,0,90])                    
                    box_text_x_pos_face();
                }
                
                //add some negitive text to x negitive face
                if ((box_text_x_neg_exists==1)&&(box_text_x_neg_type==1))
                {
                    translate ([-box_length/2+box_text_x_neg_extrude_height,0,box_height-box_text_x_neg_line_spacing])
                    rotate ([90,0,270])
                    box_text_x_neg_face();
                }
                
                //add some negitive text to y positive face
                if ((box_text_y_pos_exists==1)&&(box_text_y_pos_type==1))
                {
                    translate ([0,box_width/2-box_text_y_pos_extrude_height,box_height-box_text_y_pos_line_spacing])
                    rotate ([90,0,180])
                    box_text_y_pos_face();
                }
                
                //add some negitive text to y negitive face
                if ((box_text_y_neg_exists==1)&&(box_text_y_neg_type==1))
                {
                    translate ([0,-box_width/2+box_text_y_neg_extrude_height,box_height-box_text_y_neg_line_spacing])
                    rotate ([90,0,0])
                    box_text_y_neg_face();
                }
                
                 //add some negitive text to base face
                if ((box_text_base_exists==1)&&(box_text_base_type==1))
                {
                    translate ([0,-box_width/2+box_text_base_extrude_height,box_height-box_text_base_line_spacing])
                    rotate ([90,0,0])
                    box_text_base_face();
                }                       
            //remove holes
            #if (box_holes_exist == 1)
            {    
                holes_from_array ( box_first_hole,
                                   box_last_hole);
            }
        }
}

//////////////////////////////////////////////////////////////
// build the basic lid shape and hollow out
//////////////////////////////////////////////////////////////

module  lid_stage_1_build()
{
    difference()
    {
        union()
        {
            //draw the box lid 
            translate([0,0,0])
            rounded_box(lid_length,
                        lid_width,
                        lid_height,
                        lid_corner_radius,
                        lid_top_radius_exists);
            
            //add plinth if required
            
            if (lid_plinth_exists == 1)
            {
                if (lid_hollow_simple_polygon_exists == 1)
                {
                    translate([0,0,lid_height])
                    internal_shape( lid_poly_x_near-lid_plinth_clearance,
                                    lid_poly_x_mid-lid_plinth_clearance,
                                    lid_poly_x_far-lid_plinth_clearance,
                                    lid_poly_x_ext-lid_plinth_clearance,
                                    lid_poly_y_near-lid_plinth_clearance,
                                    lid_poly_y_mid-lid_plinth_clearance,
                                    lid_poly_y_far-lid_plinth_clearance, 
                                    lid_plinth_thickness);

                }
                
                else if (lid_hollow_full_polygon_exists == 1)
                {
                    
                    
                }
                
                else //basic shape
                {
                    translate([0,0,lid_height])
                    rounded_box(lid_length-(lid_hollow_wall_thickness*2)-lid_plinth_clearance,
                                lid_width-(lid_hollow_wall_thickness*2)-lid_plinth_clearance,
                                lid_plinth_thickness,
                                lid_corner_radius,
                                0);   
                }
            
            }
            

        }//end of union
            
            
            //remove bits from the lid
        
            //hollow out the lid
            if (lid_simple_hollow_exists==1)
            {
                if(lid_hollow_simple_polygon_exists==1)
                {
                    translate([0,0,lid_top_thickness])
                    internal_shape( lid_poly_x_near,
                                    lid_poly_x_mid,
                                    lid_poly_x_far,
                                    lid_poly_x_ext,
                                    lid_poly_y_near,
                                    lid_poly_y_mid,
                                    lid_poly_y_far,
                                    (lid_height-lid_top_thickness));
                }
                
                else if (lid_hollow_full_polygon_exists == 1)
                {
                    translate([0,0,lid_top_thickness])
                    linear_extrude(lid_height-lid_top_thickness+0.05)  
                    polygon (box_hollow_full_poly_points);  
                }
                
                else
                {
                    translate([0,0,lid_top_thickness+0.1])
                    rounded_box(lid_length-lid_hollow_wall_thickness*2,
                                lid_width-lid_hollow_wall_thickness*2,
                                lid_height-lid_top_thickness,
                                lid_corner_radius,
                                lid_top_radius_exists);
                }
            }
            
            //remove clearance on lid plinth for box pillers
            if ((lid_plinth_exists == 1) && (box_pillers_1_exist==1))
            {
                translate([box_pillers_1_shift_x,box_pillers_1_shift_y,lid_height])    
                cylinder_array(box_pillers_1_dia +lid_plinth_clearance,
                               lid_plinth_thickness+0.15,
                               box_pillers_1_x_qty,
                               box_pillers_1_y_qty,
                               box_pillers_1_x_pitch,
                               box_pillers_1_y_pitch,
                               box_pillers_1_outer_only);
            }
            
            //remove clearance on lid plinth for box pillers
            if ((lid_plinth_exists == 1) && (box_pillers_2_exist==1))
            {
                translate([box_pillers_2_shift_x,box_pillers_2_shift_y,lid_height])    
                cylinder_array(box_pillers_2_dia +lid_plinth_clearance,
                               lid_plinth_thickness+0.15,
                               box_pillers_2_x_qty,
                               box_pillers_2_y_qty,
                               box_pillers_2_x_pitch,
                               box_pillers_2_y_pitch,
                               box_pillers_2_outer_only);
            }

        }//end difference  
}

////////////////////////////////////////////////////////
//lid stage 2 build
//if required add flange, flange hinge, pillers, filter holders 
//filter holders 1-2
//then if required remove side recesses,lid fixing holes,
//holes 1-5, cutout 1-5, vent grills 1-5, an array of holes,
//fans 1-2, oring grooves
////////////////////////////////////////////////////////

module lid_stage_2_build()

    {
        difference()
        {
            union()
            {
                    //draw the box lid
                    lid_stage_1_build();
                
                    //add a lip to the lid
                    if (lid_flange_exists == 1)
                    {
                    translate([0,0,lid_height-lid_flange_thickness])    
                    flange_plate(lid_length,
                                 lid_width,
                                 lid_flange_thickness,
                                 lid_corner_radius,
                                 lid_flange_opening_X,
                                 lid_flange_opening_Y,
                                 lid_top_opening_pos_X,
                                 lid_top_opening_pos_Y);
                    }
                    
                    //add an array of pillers to the lid
                    if (lid_pillers_1_exist == 1)
                    {
                    translate([lid_pillers_1_shift_x,lid_pillers_1_shift_y,
                               lid_pillers_1_shift_z+(lid_corner_radius*lid_top_radius_exists)])    
                    cylinder_array(lid_pillers_1_dia,
                                   lid_pillers_1_length-(lid_corner_radius*lid_top_radius_exists),
                                   lid_pillers_1_x_qty,
                                   lid_pillers_1_y_qty,
                                   lid_pillers_1_x_pitch,
                                   lid_pillers_1_y_pitch,
                                   lid_pillers_1_outer_only);
                    }
                    
                    //add an array of pillers to the lid
                    if (lid_pillers_2_exist == 1)
                    {
                    translate([lid_pillers_2_shift_x,lid_pillers_2_shift_y,
                               lid_pillers_2_shift_z+(lid_corner_radius*lid_top_radius_exists)])    
                    cylinder_array(lid_pillers_2_dia,
                                   lid_pillers_2_length-(lid_corner_radius*lid_top_radius_exists),
                                   lid_pillers_2_x_qty,
                                   lid_pillers_2_y_qty,
                                   lid_pillers_2_x_pitch,
                                   lid_pillers_2_y_pitch,
                                   lid_pillers_2_outer_only);
                    }
                    
                //add lid backplate pads 1 to bottom
                if (lid_mount_1_pads_exist == 1)
                {
                translate([0,0,0])    
                lid_pads(lid_mount_1_pads_length,
                         lid_mount_1_pads_width,
                         lid_mount_1_pads_height,
                         lid_mount_1_pads_x_qty,
                         lid_mount_1_pads_x_pitch,
                         lid_mount_1_pads_y_qty,
                         lid_mount_1_pads_y_pitch,
                         lid_mount_1_pads_x_move,
                         lid_mount_1_pads_y_move);
                }
                
                //add lid backplate pads 2 to bottom
                if (lid_mount_2_pads_exist == 1)
                {
                translate([0,0,0])    
                lid_pads(lid_mount_2_pads_length,
                         lid_mount_2_pads_width,
                         lid_mount_2_pads_height,
                         lid_mount_2_pads_x_qty,
                         lid_mount_2_pads_x_pitch,
                         lid_mount_2_pads_y_qty,
                         lid_mount_2_pads_y_pitch,
                         lid_mount_2_pads_x_move,
                         lid_mount_2_pads_y_move);
                }
 
                 
                //add sheet filter holder 1 to one of the lid sides
                if (lid_filter_1_exists == 1)
                {
                    translate ([lid_filter_1_x_pos,lid_filter_1_y_pos,lid_filter_1_z_pos])
                    make_filter(lid_filter_1_side,
                                lid_filter_1_length,
                                lid_filter_1_width,
                                lid_filter_1_thickness,
                                lid_filter_1_wall_thickness,
                                lid_filter_1_retain_lip,
                                lid_filter_1_stop_thickness);
                }
                 
                //add sheet filter holder 2 to one of the lid sides
                if (lid_filter_2_exists == 1)
                {
                    translate ([lid_filter_2_x_pos,0,lid_filter_2_z_pos])
                    make_filter(lid_filter_2_side,
                                lid_filter_2_length,
                                lid_filter_2_width,
                                lid_filter_2_thickness,
                                lid_filter_2_wall_thickness,
                                lid_filter_2_retain_lip,
                                lid_filter_2_stop_thickness);
                }  


                
                //add hinge 1 block to the lid top
                if (flange_hinge_exists == 1)
                {
                    //shift the hinge where you want it
                    translate([lid_hinge_pos_x,-(lid_width/2)+lid_hinge_pos_y,lid_hinge_pos_z+lid_height])
                    rotate ([lid_hinge_rot_x,lid_hinge_rot_y,lid_hinge_rot_z])
                    hinge_block(lid_hinge_type,
                                lid_hinge_length,
                                lid_hinge_width,
                                lid_hinge_shaft_height,
                                lid_hinge_screw_size,
                                lid_hinge_cbore_length,
                                hinge_joint_position);
                }
                
                //add angled side hinge to lid
                if (side_hinges_exist ==1)
                {
                    translate([0,-lid_width/2-side_hinge_center_dist,lid_height])
                    rotate ([0,0,0])
                    lid_side_hinge();
                }

                
                
            }//end union
                
            // remove the x positive side recess
            if (box_x_pos_recess_exists==1)
            {
                translate([box_length/2,box_x_pos_recess_move_y,-0.1])
                rotate([0,0,90])
                rounded_slot(box_x_pos_recess_length,
                             lid_height,
                             box_x_pos_recess_radius);
            }
                
            // remove the x negative side recess
            if (box_x_neg_recess_exists==1)
            {
                translate([-box_length/2,box_x_neg_recess_move_y,-0.1])
                rotate([0,0,90])
                rounded_slot(box_x_pos_recess_length,
                             lid_height,
                             box_x_neg_recess_radius);
            }             
            // remove the y positive side recess
            if (box_y_pos_recess_exists==1)
            {
                translate([box_y_pos_recess_move_x,box_width/2,-0.1])
                rotate([0,0,0])
                rounded_slot(box_y_pos_recess_length,
                             lid_height,
                             box_y_pos_recess_radius);
            }
                
            // remove the y negative side recess
            if (box_y_neg_recess_exists==1)
            {
                translate([box_y_neg_recess_move_x,-box_width/2,-0.1])
                rotate([0,0,0])
                rounded_slot(box_y_pos_recess_length,
                             lid_height,
                             box_y_pos_recess_radius);
            } 
            
                 


            //remove lid cutout 1
            if (lid_cutout_1_exists==1)
            {
               translate([lid_cutout_1_x_pos,lid_cutout_1_y_pos,lid_cutout_1_z_pos])
               make_cutout (lid_height,
                            lid_cutout_1_side,
                            lid_cutout_1_x_qty,
                            lid_cutout_1_x_pitch,
                            lid_cutout_1_y_qty,
                            lid_cutout_1_y_pitch,
                            lid_cutout_1_x_dim,
                            lid_cutout_1_y_dim,
                            lid_cutout_1_depth);  
            }
             
            //remove lid cutout 2
            if (lid_cutout_2_exists==1)
            {
               translate([lid_cutout_2_x_pos,lid_cutout_2_y_pos,lid_cutout_2_z_pos])
               make_cutout (lid_height,
                            lid_cutout_2_side,
                            lid_cutout_2_x_qty,
                            lid_cutout_2_x_pitch,
                            lid_cutout_2_y_qty,
                            lid_cutout_2_y_pitch,
                            lid_cutout_2_x_dim,
                            lid_cutout_2_y_dim,
                            lid_cutout_2_depth);  
            }
           
             //remove lid cutout 3
            if (lid_cutout_3_exists==1)
            {
               translate([lid_cutout_3_x_pos,lid_cutout_3_y_pos,lid_cutout_3_z_pos])
               make_cutout (lid_height,
                            lid_cutout_3_side,
                            lid_cutout_3_x_qty,
                            lid_cutout_3_x_pitch,
                            lid_cutout_3_y_qty,
                            lid_cutout_3_y_pitch,
                            lid_cutout_3_x_dim,
                            lid_cutout_3_y_dim,
                            lid_cutout_3_depth); 
            }           

              //remove lid cutout 4
            if (lid_cutout_4_exists==1)
            {
               translate([lid_cutout_4_x_pos,lid_cutout_4_y_pos,lid_cutout_4_z_pos])
               make_cutout (lid_height,
                            lid_cutout_4_side,
                            lid_cutout_4_x_qty,
                            lid_cutout_4_x_pitch,
                            lid_cutout_4_y_qty,
                            lid_cutout_4_y_pitch,
                            lid_cutout_4_x_dim,
                            lid_cutout_4_y_dim,
                            lid_cutout_4_depth); 
            } 
            
            //remove lid cutout 5
            if (lid_cutout_5_exists==1)
            {
               translate([lid_cutout_5_x_pos,lid_cutout_5_y_pos,lid_cutout_5_z_pos])
               make_cutout (lid_height,
                            lid_cutout_5_side,
                            lid_cutout_5_x_qty,
                            lid_cutout_5_x_pitch,
                            lid_cutout_5_y_qty,
                            lid_cutout_5_y_pitch,
                            lid_cutout_5_x_dim,
                            lid_cutout_5_y_dim,
                            lid_cutout_5_depth);  
            }           
                      

            //remove lid vents 1
            if (lid_vents_1_exists==1)
            {
               translate([lid_vents_1_x_pos,lid_vents_1_y_pos,lid_vents_1_z_pos])
               make_grills (lid_height,
                            lid_vents_1_side,
                            lid_vents_1_x_qty,
                            lid_vents_1_x_pitch,
                            lid_vents_1_y_qty,
                            lid_vents_1_y_pitch,
                            lid_vents_1_width,
                            lid_vents_1_height,
                            lid_vents_1_depth,
                            lid_vents_1_radius);  
            }
            
             //remove lid vents 2
            if (lid_vents_2_exists==1)
            {
               translate([lid_vents_2_x_pos,lid_vents_2_y_pos,lid_vents_2_z_pos])
               make_grills (lid_height,
                            lid_vents_2_side,
                            lid_vents_2_x_qty,
                            lid_vents_2_x_pitch,
                            lid_vents_2_y_qty,
                            lid_vents_2_y_pitch,
                            lid_vents_2_width,
                            lid_vents_2_height,
                            lid_vents_2_depth,
                            lid_vents_2_radius);  
            }
            
            //remove lid vents 3
            if (lid_vents_3_exists==1)
            {
               translate([lid_vents_3_x_pos,lid_vents_3_y_pos,lid_vents_3_z_pos])
               make_grills (lid_height,
                            lid_vents_3_side,
                            lid_vents_3_x_qty,
                            lid_vents_3_x_pitch,
                            lid_vents_3_y_qty,
                            lid_vents_3_y_pitch,
                            lid_vents_3_width,
                            lid_vents_3_height,
                            lid_vents_3_depth,
                            lid_vents_3_radius);  
            }

            //remove lid vents 4
            if (lid_vents_4_exists==1)
            {
               translate([lid_vents_4_x_pos,lid_vents_4_y_pos,lid_vents_4_z_pos])
               make_grills (lid_height,
                            lid_vents_4_side,
                            lid_vents_4_x_qty,
                            lid_vents_4_x_pitch,
                            lid_vents_4_y_qty,
                            lid_vents_4_y_pitch,
                            lid_vents_4_width,
                            lid_vents_4_height,
                            lid_vents_4_depth,
                            lid_vents_4_radius);  
            }
         
            //remove lid vents 5
            if (lid_vents_5_exists==1)
            {
               translate([lid_vents_5_x_pos,lid_vents_5_y_pos,lid_vents_5_z_pos])
               make_grills (lid_height,
                            lid_vents_5_side,
                            lid_vents_5_x_qty,
                            lid_vents_5_x_pitch,
                            lid_vents_5_y_qty,
                            lid_vents_5_y_pitch,
                            lid_vents_5_width,
                            lid_vents_5_height,
                            lid_vents_5_depth,
                            lid_vents_5_radius);  
            }
            
            //remove fan 1 from the lid
            if (lid_fan_1_exists == 1)
            {
                translate([lid_fan_1_x_pos,lid_fan_1_y_pos,lid_fan_1_z_pos])    
                fan_cutout (lid_height,
                            lid_fan_1_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                            lid_fan_1_dia,
                            lid_fan_1_fix_dia,
                            lid_fan_1_fixing_square,
                            lid_fan_1_depth);
            }
            
            //remove fan 2 from the lid
            if (lid_fan_2_exists == 1)
            {
                translate([lid_fan_2_x_pos,lid_fan_2_y_pos,lid_fan_2_z_pos])    
                fan_cutout (lid_height,
                            lid_fan_2_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                            lid_fan_2_dia,
                            lid_fan_2_fix_dia,
                            lid_fan_2_fixing_square,
                            lid_fan_2_depth);
            }
              
            if (o_ring_selected==1)
            {
                translate([0,0,lid_height])
                o_ring_insertion();   
            }
            
            //put a mesh through the sliding lid
            if (lid_top_mesh_exists==1)
            {
                translate ([lid_top_mesh_x_pos,lid_top_mesh_y_pos,-lid_top_mesh_depth_offset])
                rotate ([0,0,0])                    
                make_mesh (lid_top_mesh_type, //0=circles,3 up = polygon sides 1&2 invalid
                           lid_top_mesh_dia,
                           lid_top_mesh_rotation,
                           lid_top_mesh_x_qty,
                           lid_top_mesh_x_pitch,
                           lid_top_mesh_y_qty,
                           lid_top_mesh_y_pitch,
                           lid_top_mesh_depth,
                           lid_top_mesh_offset_req);
            }
            
            //remove groove to take ball bearings
            if (lid_ball_groove_exists == 1)
            {
                translate([lid_ball_groove_x_move,lid_ball_groove_y_move,lid_ball_groove_z_move])
                {
                rotate_extrude()
                translate ([lid_ball_groove_dia/2,0,0])
                circle (d=lid_ball_groove_size);
                }
            }
           
            //remove coin slot
            if (lid_coin_slot_exists == 1)
            {
                translate([lid_coin_slot_x_move,lid_coin_slot_y_move,
                            lid_top_thickness+lid_coin_slot_lip/2])
                rotate([0,180,lid_coin_slot_rotation])
                coin_slot(lid_top_thickness);
            }
            
        }//end difference
 
}//end module
    
////////////////////////////////////////////////////////
//lid stage 3 build
////////////////////////////////////////////////////////

module lid_stage_3_build()
{        
        translate([0,0,0])
        rotate ([0,0,0])   
        difference()
            {
            union()
                {
                //import stage 2 build of the lid
                lid_stage_2_build();
                //add stage 3 bits to go onto the lid
                    
                 // add entry boss 1 to lid
                 if (lid_boss_1_exists ==1)
                 {
                    translate ([lid_boss_1_x_pos,lid_boss_1_y_pos,lid_boss_1_z_pos])
                    make_entry_boss(lid_height,
                                    lid_boss_1_side,
                                    lid_boss_1_length,
                                    lid_boss_1_width,
                                    lid_boss_1_height,
                                    lid_boss_1_radius); 
                 }
                 
                 // add entry boss 2 to lid
                 if (lid_boss_2_exists ==1)
                 {
                    translate ([lid_boss_2_x_pos,lid_boss_2_y_pos,lid_boss_2_z_pos])
                    make_entry_boss(lid_height,
                                    lid_boss_2_side,
                                    lid_boss_2_length,
                                    lid_boss_2_width,
                                    lid_boss_2_height,
                                    lid_boss_2_radius); 
                 }
                 
                // import .stl file to lid 1
                if (lid_import_1_stl_exists==1)
                {
                    translate([lid_import_1_pos_x,lid_import_1_pos_y,lid_import_1_pos_z])
                    rotate([lid_import_1_rot_x,lid_import_1_rot_y,lid_import_1_rot_z])
                    scale([lid_import_1_scale_x,lid_import_1_scale_y,lid_import_1_scale_z])
                    import(lid_import_1_stl_file);
                }
                
                //add some positive text
                 if ((lid_text_exists==1)&&(lid_text_type==0))
                {
                    translate ([0,((lid_text_line_spacing*lid_text_no_of_lines)/2)-lid_text_line_spacing/2,-lid_text_extrude_height])
                    rotate ([0,0,0])
                    lid_text();
                }
 
                
                }//end union
                
                //subtract stage 3 bits of the lid

               
                //add some negitive text
                 if ((lid_text_exists==1)&&(lid_text_type==1))
                {
                    translate ([0,((lid_text_line_spacing*lid_text_no_of_lines)/2)-lid_text_line_spacing/2,0])
                    rotate ([0,0,0])

                    lid_text();
                }
 
                #if (lid_holes_exist == 1)
                {    
                    holes_from_array ( lid_first_hole,
                                       lid_last_hole);                
                }
            
    }
}

////////////////////////////////////////////////////////
// draws the lid cut out for extrusion 
////////////////////////////////////////////////////////

module internal_shape(  x_near,
                        x_mid,
                        x_far,
                        x_ext,
                        y_near,
                        y_mid,
                        y_far,
                        shape_height)
{
        translate([0,0,shape_height/2])
        linear_extrude(height = shape_height+0.1, center = true)
        polygon( [  [x_mid,-y_mid],
                    [x_far,-y_near],
                    [x_ext,-y_near],
                    [x_ext,y_near],
                    [x_far,y_near],
                    [x_mid,y_mid],
                    [x_near,y_mid],
                    [x_near,y_far],
                    [-x_near,y_far],
                    [-x_near,y_mid],
                    [-x_mid,y_mid],
                    [-x_far,y_near],
                    [-x_ext,y_near],
                    [-x_ext,-y_near],
                    [-x_far,-y_near],
                    [-x_mid,-y_mid],
                    [-x_near,-y_mid],
                    [-x_near,-y_far],
                    [x_near,-y_far],
                    [x_near,-y_mid]],
        paths=[[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19]] );
}

////////////////////////////////////////////////////////
// draws a flange on box or lid opening
////////////////////////////////////////////////////////

module flange_plate(length,
                    width,
                    height,
                    radius,
                    x_opening,
                    y_opening,
                    opening_pos_X,
                    opening_pos_Y)
{
    difference()
    {
        union()
        {
            //draw the flange plate 
            rounded_box(length,
                        width,
                        height,
                        radius,
                        0);

        }//end of union
            
            //remove centre apperature
            translate([opening_pos_X,opening_pos_Y,height/2])
            cube([x_opening,y_opening,height+0.1],center=true);
    }//end difference  
}


////////////////////////////////////////////////////////
// position box filter 1 when required
////////////////////////////////////////////////////////

module make_filter(filter_side,
                   filter_length,
                   filter_width,
                   filter_thickness,
                   filter_wall_thickness,
                   filter_retain_lip,
                   filter_stop_thickness)
{
                     //x pos side
                    if (filter_side==1)
                    {
                        //move it into place
                        translate([-filter_thickness-filter_wall_thickness+box_poly_x_ext+0.1,0,
                                    filter_width/2+filter_stop_thickness/2])
                        rotate ([90,0,90])
                        filter_holder(filter_length,
                                      filter_width,
                                      filter_thickness,
                                      filter_wall_thickness,
                                      filter_retain_lip,
                                      filter_stop_thickness);
                    }
                    
                    //x neg side
                    else if (filter_side==2)
                    {
                        //move it into place
                        translate([filter_thickness+filter_wall_thickness-box_poly_x_ext-0.1,0,
                                    filter_width/2+filter_stop_thickness/2])
                        rotate ([90,0,270])
                        filter_holder(filter_length,
                                      filter_width,
                                      filter_thickness,
                                      filter_wall_thickness,
                                      filter_retain_lip,
                                      filter_stop_thickness);
                    }
                    
                    //y pos side
                    else if (filter_side==3)
                    {
                        //move it into place
                        translate([0,
                        -filter_thickness-filter_wall_thickness+box_poly_y_far+0.1,
                                   filter_width/2+filter_stop_thickness/2])
                        rotate ([90,0,180])
                        filter_holder(filter_length,
                                      filter_width,
                                      filter_thickness,
                                      filter_wall_thickness,
                                      filter_retain_lip,
                                      filter_stop_thickness);
                    }
                    
                    //y pos side
                    else if (filter_side==4)
                    {
                        //move it into place
                        translate([0,
                        filter_thickness+filter_wall_thickness-box_poly_y_far-0.1,
                                   filter_width/2+filter_stop_thickness/2])
                        rotate ([90,0,0])
                        filter_holder(filter_length,
                                      filter_width,
                                      filter_thickness,
                                      filter_wall_thickness,
                                      filter_retain_lip,
                                      filter_stop_thickness);
                    }
                    
                    //bottom face
                    else if (filter_side==5)
                    {
                        //move it into place
                        translate([0,0,
                                    filter_thickness+filter_wall_thickness+box_base_thickness-0.1])
                        rotate ([0,180,90])
                        filter_holder(filter_length,
                                      filter_width,
                                      filter_thickness,
                                      filter_wall_thickness,
                                      filter_retain_lip,
                                      filter_stop_thickness);
                    }   
}

////////////////////////////////////////////////////////
// make an array of cylinders for addition or subtraction
////////////////////////////////////////////////////////

module cylinder_array(cyl_dia,
                      cyl_length,
                      x_qty,
                      y_qty,
                      x_pitch,
                      y_pitch,
                      outer_only)
{
       
    translate ([0,y_pitch/2,0])
    for(y=[0:y_qty-1])
    {
        translate ([0,y_pitch*y-((y_pitch*(y_qty))/2),0])
        rotate ([0,0,0])
        for(x=[0:x_qty-1])
        {
            if (  (outer_only==1)  &&
                     (  
                     ((x==(0))||(x==(x_qty-1)))  ||  ((y==(0))||(y==(y_qty-1)))  
                     ) 
               )
            {
                translate ([x_pitch*x-((x_pitch*(x_qty-1))/2),0,0])
                rotate ([0,0,0])
                cylinder ( d = cyl_dia, h = cyl_length);

            }
            
            else if (outer_only==0)
            {
                translate ([x_pitch*x-((x_pitch*(x_qty-1))/2),0,0])
                rotate ([0,0,0])
                cylinder ( d = cyl_dia, h = cyl_length);
                    
            }
        }   
    }        
}

////////////////////////////////////////////////////////
// box shape with rounded corners 
////////////////////////////////////////////////////////

module rounded_box( length,
                    width,
                    height,
                    radius,
                    radius_exists)
{
    if (radius_exists ==0)
    {
            radius = (radius + 0.01); //make it tiny so it generates block when zero
        
            hull()
            {
                translate( [length/2-radius,width/2-radius,0] )
                cylinder( r = radius, h = height);
                
                translate( [-length/2+radius,width/2-radius,0] )
                cylinder( r = radius, h = height);
                
                translate( [length/2-radius,-width/2+radius,0] )
                cylinder( r = radius, h = height);
                
                translate( [-length/2+radius,-width/2+radius,0] )
                cylinder( r = radius, h = height);	
            }
    }
    //put a radius on box base / lid top
    else
    {
            hull()
            {
                translate( [length/2-radius,width/2-radius,radius] )
                {
                cylinder( r = radius, h = height-radius);
                sphere (r=radius);
                }
                
                translate( [-length/2+radius,width/2-radius,radius] )
                {
                cylinder( r = radius, h = height-radius);
                sphere (r=radius);
                }
                
                translate( [length/2-radius,-width/2+radius,radius] )
                {
                cylinder( r = radius, h = height-radius);
                sphere (r=radius);
                }
                
                translate( [-length/2+radius,-width/2+radius,radius] )
                {
                cylinder( r = radius, h = height-radius);
                sphere (r=radius);
                }
            }
    }
}
////////////////////////////////////////////////////////
// box shape with 2 rounded corners , used in cable clamps
////////////////////////////////////////////////////////

module half_rounded_box( length,
                         width,
                         height,
                         radius)
{
    hull()
    {
        radius=radius+0.01;
        translate( [length/2-radius,width/2-radius,0] )
        cylinder( r = radius, h = height);
                
        translate( [-length/2+radius,width/2-radius,height/2] )
        cube([radius*2,radius*2,height],center=true);
                
        translate( [length/2-radius,-width/2+radius,0] )
        cylinder( r = radius, h = height);
                
        translate( [-length/2+radius,-width/2+radius,height/2] )
        cube([radius*2,radius*2,height],center=true);
   }
}

////////////////////////////////////////////////////////
// slot shape for cutouts 
////////////////////////////////////////////////////////

module rounded_slot(length,
                    height,
                    radius)
{
            hull()
            {
                translate( [length/2-radius,0,0] )
                cylinder( r = radius, h = height+0.2);
                
                translate( [-length/2+radius,0,0] )
                cylinder( r = radius, h = height+0.2);
            }
}


////////////////////////////////////////////////////////
// makes a cutout where you tell it 
////////////////////////////////////////////////////////

module make_cutout (side_height,
                    cutout_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                    cutout_x_qty,
                    cutout_x_pitch,
                    cutout_y_qty,
                    cutout_y_pitch,
                    cutout_width,
                    cutout_height,
                    cutout_depth)
{
    if (cutout_side==1) //x positive face
    {
         translate ([(box_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists)+0.1,
                    0,side_height/2])
        rotate ([0,270,0])
        for(i=[0:cutout_y_qty-1])
        {
            translate ([cutout_y_pitch*i-((cutout_y_pitch*(cutout_y_qty-1))/2),0,0,])
            translate ([0,(-(cutout_x_pitch*(cutout_x_qty-1))/2),0])
            for(i=[0:cutout_x_qty-1])
            {
                translate ([0,cutout_x_pitch*i,cutout_depth/2])
                cube([cutout_height,cutout_width,cutout_depth],center=true);

            }
        }    
    }
    
else if (cutout_side==2) //x negitive face       
    {
        translate ([-(box_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists)-0.1,
                    0,side_height/2])
        rotate ([0,90,0])
        for(i=[0:cutout_y_qty-1])
        {
            translate ([cutout_y_pitch*i-((cutout_y_pitch*(cutout_y_qty-1))/2),0,0,])
            translate ([0,(-(cutout_x_pitch*(cutout_x_qty-1))/2),0])
            for(i=[0:cutout_x_qty-1])
            {
                translate ([0,cutout_x_pitch*i,cutout_depth/2])
                cube([cutout_height,cutout_width,cutout_depth],center=true);
            }
        }    
    }
    
    else if (cutout_side==3) //y positive face       
    {
        translate ([0,(box_width/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists)+0.1,
                        side_height/2])
        rotate ([90,0,0])
        for(i=[0:cutout_y_qty-1])
        {
            translate ([0,cutout_y_pitch*i-((cutout_y_pitch*(cutout_y_qty-1))/2),0])
            translate ([(-(cutout_x_pitch*(cutout_x_qty-1))/2),0,0])
            for(i=[0:cutout_x_qty-1])
            {
                translate ([cutout_x_pitch*i,0,cutout_depth/2])
                rotate ([0,0,90])
                cube([cutout_height,cutout_width,cutout_depth],center=true);
            }
        }    
    }
    
    else if (cutout_side==4) //y negitive face       
    {
        translate ([0,-(box_width/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists)-0.1,
                        side_height/2])
        rotate ([270,0,0])
        for(i=[0:cutout_y_qty-1])
        {
            translate ([0,cutout_y_pitch*i-((cutout_y_pitch*(cutout_y_qty-1))/2),0])
            translate ([(-(cutout_x_pitch*(cutout_x_qty-1))/2),0,0])
            for(i=[0:cutout_x_qty-1])
            {
                translate ([cutout_x_pitch*i,0,cutout_depth/2])
                rotate ([0,0,90])
                cube([cutout_height,cutout_width,cutout_depth],center=true);
            }
        }    
    }
    
    else if (cutout_side==5) //bottom face       
    {
        for(i=[0:cutout_y_qty-1])
        {
            translate ([0,cutout_y_pitch*i-((cutout_y_pitch*(cutout_y_qty-1))/2),0])
            translate ([(-(cutout_x_pitch*(cutout_x_qty-1))/2),0,0])
            for(i=[0:cutout_x_qty-1])
            {
                translate ([cutout_x_pitch*i,0,cutout_depth/2])
                rotate ([0,0,90])
                cube([cutout_height,cutout_width,cutout_depth],center=true);
            }
        }   
    }     
}
 
////////////////////////////////////////////////////////
// makes a set of grills in any side 
////////////////////////////////////////////////////////

module make_grills (side_height,
                    vents_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                    vents_x_qty,
                    vents_x_pitch,
                    vents_y_qty,
                    vents_y_pitch,
                    vents_width,
                    vents_height,
                    vents_depth,
                    vents_radius)
{
    if (vents_side==1) //x positive face
    {
        
        translate ([(box_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists)+0.1,
                    0,side_height/2])
        rotate ([0,270,0])
        for(i=[0:vents_y_qty-1])
        {
            translate ([vents_y_pitch*i-((vents_y_pitch*(vents_y_qty-1))/2),0,0,])
            translate ([0,(-(vents_x_pitch*(vents_x_qty-1))/2),0])
            for(i=[0:vents_x_qty-1])
            {
                translate ([0,vents_x_pitch*i,0])
                rounded_box( vents_height,
                             vents_width,
                             vents_depth,
                             vents_radius,
                             0);
            }
        }    
    }
    
    else if (vents_side==2) //x negitive face       
    {
        translate ([-(box_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists)-0.1,
                    0,side_height/2])
        rotate ([0,90,0])
        for(i=[0:vents_y_qty-1])
        {
            translate ([vents_y_pitch*i-((vents_y_pitch*(vents_y_qty-1))/2),0,0,])
            translate ([0,(-(vents_x_pitch*(vents_x_qty-1))/2),0])
            for(i=[0:vents_x_qty-1])
            {
                translate ([0,vents_x_pitch*i,0])
                rounded_box( vents_height,
                             vents_width,
                             vents_depth,
                             vents_radius,
                             0);
            }
        }    
    }
    
    else if (vents_side==3) //y positive face       
    {
        translate ([0,(box_width/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists)+0.1,
                        side_height/2])
        rotate ([90,0,0])
        for(i=[0:vents_y_qty-1])
        {
            translate ([0,vents_y_pitch*i-((vents_y_pitch*(vents_y_qty-1))/2),0])
            translate ([(-(vents_x_pitch*(vents_x_qty-1))/2),0,0])
            for(i=[0:vents_x_qty-1])
            {
                translate ([vents_x_pitch*i,0,0])
                rotate ([0,0,90])
                rounded_box( vents_height,
                             vents_width,
                             vents_depth,
                             vents_radius,
                             0);
            }
        }    
    }
    
    else if (vents_side==4) //y negitive face       
    {
        translate ([0,-(box_width/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists)-0.1,
                        side_height/2])
        rotate ([270,0,0])
        for(i=[0:vents_y_qty-1])
        {
            translate ([0,vents_y_pitch*i-((vents_y_pitch*(vents_y_qty-1))/2),0])
            translate ([(-(vents_x_pitch*(vents_x_qty-1))/2),0,0])
            for(i=[0:vents_x_qty-1])
            {
                translate ([vents_x_pitch*i,0,0])
                rotate ([0,0,90])
                rounded_box( vents_height,
                             vents_width,
                             vents_depth,
                             vents_radius,
                             0);
            }
        }    
    }
    
    else if (vents_side==5) //bottom face       
    {
        for(i=[0:vents_y_qty-1])
        {
            translate ([0,vents_y_pitch*i-((vents_y_pitch*(vents_y_qty-1))/2),0])
            translate ([(-(vents_x_pitch*(vents_x_qty-1))/2),0,0])
            for(i=[0:vents_x_qty-1])
            {
                translate ([vents_x_pitch*i,0,0])
                rotate ([0,0,90])
                rounded_box( vents_height,
                            vents_width,
                            vents_depth,
                            vents_radius,
                            0);
            }
        }   
    }     
}
 
////////////////////////////////////////////////////////
// makes a fan cutout in any face 
////////////////////////////////////////////////////////

module fan_cutout (side_height,
                   fan_side,//1=xpos.2=xneg.3=ypos.4=yneg.5=bottom
                   fan_hole_dia,
                   fan_fixing_dia,
                   fan_fix_square,
                   fan_depth)
{
    if (fan_side==1) //x positive face
    {
        translate( [(box_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists)-(fan_depth)+0.1
                    ,0,(side_height/2)] )
        rotate ([0,90,0])
        {
            cylinder( r = fan_hole_dia/2, h = fan_depth+0.1);
        
            translate ([-fan_fix_square/2,-fan_fix_square/2,0])
            for(i=[0:1])
            {
                translate ([0,(fan_fix_square)*i,0])
                for(i=[0:1])
                {
                    translate ([(fan_fix_square)*i,0,0])
                    rotate ([0,0,0])
                    cylinder( h = fan_depth+0.1,d1=fan_fixing_dia,d2=fan_fixing_dia);
                }
            }
        }
    }
    
    else if (fan_side==2) //x negitive face       
    {
        translate( [-(box_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists)-0.1
                    ,0,(side_height/2)] )
        rotate ([0,90,0])
        {
            cylinder( r = fan_hole_dia/2, h = fan_depth+0.1);
        
            translate ([-fan_fix_square/2,-fan_fix_square/2,0])
            for(i=[0:1])
            {
                translate ([0,(fan_fix_square)*i,0])
                for(i=[0:1])
                {
                    translate ([(fan_fix_square)*i,0,0])
                    rotate ([0,0,0])
                    cylinder( h = fan_depth+0.1,d1=fan_fixing_dia,d2=fan_fixing_dia);
                }
            }
        }
    }
    
    else if (fan_side==3) //y positive face       
    {
        translate( [0,(box_width/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists)+0.1,(side_height/2)] )
        rotate ([90,90,0])
        {
            cylinder( r = fan_hole_dia/2, h = fan_depth+0.1);
        
            translate ([-fan_fix_square/2,-fan_fix_square/2,0])
            for(i=[0:1])
            {
                translate ([0,(fan_fix_square)*i,0])
                for(i=[0:1])
                {
                    translate ([(fan_fix_square)*i,0,0])
                    rotate ([0,0,0])
                    cylinder( h = fan_depth+0.1,d1=fan_fixing_dia,d2=fan_fixing_dia);
                }
            }
        }
    }
    
    else if (fan_side==4) //y negitive face       
    {
        translate( [0,-(box_width/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists)-0.1,(side_height/2)] )
        rotate ([270,90,0])
        {
            cylinder( r = fan_hole_dia/2, h = fan_depth+0.1);
        
            translate ([-fan_fix_square/2,-fan_fix_square/2,0])
            for(i=[0:1])
            {
                translate ([0,(fan_fix_square)*i,0])
                for(i=[0:1])
                {
                    translate ([(fan_fix_square)*i,0,0])
                    rotate ([0,0,0])
                    cylinder( h = fan_depth+0.1,d1=fan_fixing_dia,d2=fan_fixing_dia);
                }
            }
        }
    }
    
    else if (fan_side==5) //bottom face       
    {
            translate ([0,0,0])
            cylinder( r = fan_hole_dia/2, h = fan_depth+0.1);
        
            translate ([-fan_fix_square/2,-fan_fix_square/2,0])
            for(i=[0:1])
            {
                translate ([0,(fan_fix_square)*i,0])
                for(i=[0:1])
                {
                    translate ([(fan_fix_square)*i,0,0])
                    rotate ([0,0,0])
                    cylinder( h = fan_depth+0.1,d1=fan_fixing_dia,d2=fan_fixing_dia);
                }
            }
        
    }     
}

////////////////////////////////////////////////////////////////////////////
//put dividers in base of box
////////////////////////////////////////////////////////////////////////////

 module box_dividers(divider_x_qty,
                     divider_x_pitch,
                     divider_x_length,
                     divider_x_thickness,
                     divider_x_height,
                     divider_y_qty,
                     divider_y_pitch,
                     divider_y_length,
                     divider_y_thickness,
                     divider_y_height)
{
    //x axis dividers
    translate ([-(divider_x_pitch*(divider_x_qty-1))/2,,,0])
    for(i=[0:divider_x_qty-1])
    {
            translate ([(divider_x_pitch)*i,0,divider_x_height/2])
            rotate ([0,0,90])
            cube([divider_x_length,divider_x_thickness,divider_x_height],center=true);
    }
    
    //y axis dividers
    translate ([0,-(divider_y_pitch*(divider_y_qty-1))/2,0])
    for(i=[0:divider_y_qty-1])
    {
            translate ([0,(divider_y_pitch)*i,divider_y_height/2])
            cube([divider_y_length,divider_y_thickness,divider_y_height],center=true);
    }

}

////////////////////////////////////////////////////////////////////////////
//put backplate support pads on base
////////////////////////////////////////////////////////////////////////////

 module base_pads(pads_length,
                  pads_width,
                  pads_height,
                  pads_x_qty,
                  pads_x_pitch,
                  pads_y_qty,
                  pads_y_pitch,
                  pads_x_move,
                  pads_y_move)
{
    translate ([pads_x_move,pads_y_move,box_base_thickness+pads_height/2])
    for(i=[0:pads_y_qty-1])
    {
        translate ([0,pads_y_pitch*i-((pads_y_pitch*(pads_y_qty-1))/2),0])
        for(i=[0:pads_x_qty-1])
        {
            translate ([pads_x_pitch*i-((pads_x_pitch*(pads_x_qty-1))/2),0,0])
            rotate ([0,0,0])
            cube([pads_length,pads_width,pads_height],center=true);
        }
     }
}

////////////////////////////////////////////////////////////////////////////
//put backplate support pads on base
////////////////////////////////////////////////////////////////////////////

 module lid_pads(pads_length,
                 pads_width,
                 pads_height,
                 pads_x_qty,
                 pads_x_pitch,
                 pads_y_qty,
                 pads_y_pitch,
                 pads_x_move,
                 pads_y_move)
{
    translate ([pads_x_move,pads_y_move,lid_top_thickness+pads_height/2])
    for(i=[0:pads_y_qty-1])
    {
        translate ([0,pads_y_pitch*i-((pads_y_pitch*(pads_y_qty-1))/2),0])
        for(i=[0:pads_x_qty-1])
        {
            translate ([pads_x_pitch*i-((pads_x_pitch*(pads_x_qty-1))/2),0,0])
            rotate ([0,0,0])
            cube([pads_length,pads_width,pads_height],center=true);
        }
     }
}

////////////////////////////////////////////////////////////////////////////
//generates a groove for insertion of an "O" ring
////////////////////////////////////////////////////////////////////////////

module o_ring_insertion()
{
    //RHS O ring
    // ref a : RHS oring section
    translate([o_ring_x_inner/2,o_ring_y_inner/2+(((o_ring_y_outer-o_ring_y_inner)/2)-o_ring_corner_radius),0]) 
    rotate([90,0,0]) 
    cylinder(r=o_ring_diameter/2,((o_ring_y_outer-o_ring_y_inner)/2)-(2*o_ring_corner_radius));
    
    //ref b :  RHS oring section
    translate([o_ring_x_inner/2-o_ring_corner_radius,o_ring_y_outer/2-o_ring_corner_radius,0])
    rotate([0,0,0])
    rotate_extrude(angle=90, convexity=10)
    translate([o_ring_corner_radius,0,0]) 
    circle(o_ring_diameter/2);    

    //ref c : RHS oring section
    translate([-o_ring_x_inner/2+o_ring_corner_radius,o_ring_y_outer/2,0]) 
    rotate([0,90,0]) 
    cylinder(r=o_ring_diameter/2,o_ring_x_inner-(o_ring_corner_radius*2));
    
    //ref d :  RHS oring section
    translate([-o_ring_x_inner/2+o_ring_corner_radius,o_ring_y_outer/2-o_ring_corner_radius,0])
    rotate([0,0,90])
    rotate_extrude(angle=90, convexity=10)
    translate([o_ring_corner_radius,0,0]) 
    circle(o_ring_diameter/2);
    
    //ref e :  RHS oring section
    translate([-o_ring_x_inner/2,o_ring_y_inner/2+(((o_ring_y_outer-o_ring_y_inner)/2)-o_ring_corner_radius),0]) 
    rotate([90,0,0]) 
    cylinder(r=o_ring_diameter/2,((o_ring_y_outer-o_ring_y_inner)/2)-(2*o_ring_corner_radius));

    //LHS O ring
    //ref m :  LHS oring section
    translate([-o_ring_x_inner/2,-(o_ring_y_inner/2)-o_ring_corner_radius,0])
    rotate([90,0,0]) 
    cylinder(r=o_ring_diameter/2,((o_ring_y_outer-o_ring_y_inner)/2)-(2*o_ring_corner_radius));
    
    //ref n :  LHS oring section
    translate([-o_ring_x_inner/2+o_ring_corner_radius,-o_ring_y_outer/2+o_ring_corner_radius,0])
    rotate([0,0,180])
    rotate_extrude(angle=90, convexity=10)
    translate([o_ring_corner_radius,0,0]) 
    circle(o_ring_diameter/2);

    //ref o : LHS oring section
    translate([-o_ring_x_inner/2+o_ring_corner_radius,-o_ring_y_outer/2,0]) 
    rotate([0,90,0]) 
    cylinder(r=o_ring_diameter/2,o_ring_x_inner-(o_ring_corner_radius*2));
    
    //ref p :  LHS oring section
    translate([o_ring_x_inner/2-o_ring_corner_radius,-o_ring_y_outer/2+o_ring_corner_radius,0])
    rotate([0,0,270])
    rotate_extrude(angle=90, convexity=10)
    translate([o_ring_corner_radius,0,0]) 
    circle(o_ring_diameter/2);
 

    // ref q : LHS oring section
    translate([o_ring_x_inner/2,-(o_ring_y_inner/2+o_ring_corner_radius),0])
    rotate([90,0,0]) 
    cylinder(r=o_ring_diameter/2,((o_ring_y_outer-o_ring_y_inner)/2)-(2*o_ring_corner_radius));
 
 
    //x positive o ring
    if (o_ring_add_x_neg_corner==0)
    {
        // straight section of o ring on x positive side
        //ref z : y axis, no recess, inner tube, moved x-
        translate([-o_ring_x_inner/2,o_ring_y_inner/2+o_ring_corner_radius,0]) 
        rotate([90,0,0]) 
        cylinder(r=o_ring_diameter/2,o_ring_y_inner+(o_ring_corner_radius*2));  
    }
    else
    {
        // bendy section of o ring on x positive side
        //ref f :  top oring section
        translate([-o_ring_x_inner/2-o_ring_corner_radius,o_ring_y_inner/2+o_ring_corner_radius,0])
        rotate([0,0,270])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
   
        //ref g : top oring section
        translate([-o_ring_x_inner/2-o_ring_corner_radius,o_ring_y_inner/2,0]) 
        rotate([0,270,0]) 
        cylinder(r=o_ring_diameter/2,((o_ring_x_outer-o_ring_x_inner)/2)-(o_ring_corner_radius*2));
    
        //ref h :  top oring section
        translate([-o_ring_x_outer/2+o_ring_corner_radius,o_ring_y_inner/2-o_ring_corner_radius,0])
        rotate([0,0,90])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
    
        //ref i : top oring section
        translate([-o_ring_x_outer/2,(o_ring_y_inner-o_ring_corner_radius*2)/2,0]) 
        rotate([90,0,0]) 
        cylinder(r=o_ring_diameter/2,o_ring_y_inner-o_ring_corner_radius*2);
  
        //ref j :  top oring section
        translate([-o_ring_x_outer/2+o_ring_corner_radius,-o_ring_y_inner/2+o_ring_corner_radius,0])
        rotate([0,0,180])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
    
        //ref k : top oring section
        translate([-o_ring_x_inner/2-o_ring_corner_radius,-o_ring_y_inner/2,0]) 
        rotate([0,270,0]) 
        cylinder(r=o_ring_diameter/2,((o_ring_x_outer-o_ring_x_inner)/2)-(o_ring_corner_radius*2));
    
        //ref l :  top oring section
        translate([-o_ring_x_inner/2-o_ring_corner_radius,-o_ring_y_inner/2-o_ring_corner_radius,0])
        rotate([0,0,0])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
        }
    
    //x negative o ring
    if (o_ring_add_x_pos_corner==0)
    {
 
     
     // straight section of o ring on x positive side
        //ref y : y axis, no recess, inner tube, moved x+
        translate([o_ring_x_inner/2,o_ring_y_inner/2+o_ring_corner_radius,0]) 
        rotate([90,0,0]) 
        cylinder(r=o_ring_diameter/2,o_ring_y_inner+(o_ring_corner_radius*2));  
        
    }
    else
    {
        // bendy section of o ring on x positive side
        //ref r :  bottom oring section
        translate([o_ring_x_inner/2+o_ring_corner_radius,-o_ring_y_inner/2-o_ring_corner_radius,0])
        rotate([0,0,90])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
    
        //ref s : bottom oring section
        translate([o_ring_x_inner/2+o_ring_corner_radius,-o_ring_y_inner/2,0]) 
        rotate([0,90,0]) 
        cylinder(r=o_ring_diameter/2,((o_ring_x_outer-o_ring_x_inner)/2)-(o_ring_corner_radius*2));
    
        //ref t :  bottom oring section
        translate([o_ring_x_outer/2-o_ring_corner_radius,-o_ring_y_inner/2+o_ring_corner_radius,0])
        rotate([0,0,270])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
    
        //ref u : bottom oring section
        translate([o_ring_x_outer/2,(o_ring_y_inner-o_ring_corner_radius*2)/2,0]) 
        rotate([90,0,0]) 
        cylinder(r=o_ring_diameter/2,o_ring_y_inner-o_ring_corner_radius*2);
    
        //ref v :  bottom oring section
        translate([o_ring_x_outer/2-o_ring_corner_radius,o_ring_y_inner/2-o_ring_corner_radius,0])
        rotate([0,0,0])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
    
        //ref w : bottom oring section
        translate([o_ring_x_inner/2+o_ring_corner_radius,o_ring_y_inner/2,0]) 
        rotate([0,90,0]) 
        cylinder(r=o_ring_diameter/2,((o_ring_x_outer-o_ring_x_inner)/2)-(o_ring_corner_radius*2));
    
        //ref x :  bottom oring section
        translate([o_ring_x_inner/2+o_ring_corner_radius,o_ring_y_inner/2+o_ring_corner_radius,0])
        rotate([0,0,180])
        rotate_extrude(angle=90, convexity=10)
        translate([o_ring_corner_radius,0,0]) 
        circle(o_ring_diameter/2);
    }
}

////////////////////////////////////////////////////////
//builds a filter holder to mount on a face
//hole or grill to be made seperatly
////////////////////////////////////////////////////////

module filter_holder(filter_length,
                     filter_width,
                     filter_thickness,
                     wall_thickness,
                     retain_lip,
                     stop_thickness)
{
    difference()
    {
        union()
        {
            //draw the filter block
            translate([0,0,(filter_thickness+wall_thickness)/2])
            cube([filter_length+(wall_thickness*2),
                  filter_width+stop_thickness,
                  filter_thickness+wall_thickness],center=true);

        }//end of union
            
            //remove filter volume
            translate([0,(stop_thickness/2)+0.1,(filter_thickness/2)+wall_thickness+0.11])
            cube([filter_length,filter_width,filter_thickness],center=true);
        
            //remove filter inlet
            //translate([0,stop_thickness,(wall_thickness/2)+wall_thickness+filter_thickness+0.1])
            //cube([filter_length-(retain_lip*2),filter_width,wall_thickness],center=true);
            
            //remove filter outlet
            translate([0,0,0])
            cylinder( h = wall_thickness+0.2,d1=filter_length-stop_thickness,d2=filter_length-stop_thickness);
            

    }//end difference  
}

////////////////////////////////////////////////////////////////////////////
// make boss, normally for conduit entry
////////////////////////////////////////////////////////////////////////////

module make_entry_boss(side_height,
                       boss_side,
                       boss_length,
                       boss_width,
                       boss_height,
                       boss_radius)
{
    if (boss_side == 1)
    {
        translate([(box_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists),0,(side_height/2)])
        rotate ([90,0,90])
        //draw the boss body
        rounded_box(boss_length,
                    boss_width,
                    boss_height,
                    boss_radius,
                    0);       
    }
    
    else if (boss_side == 2)
    {
        translate([-(box_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists),0,(side_height/2)])
        rotate ([270,0,90])
        //draw the boss body
        rounded_box(boss_length,
                    boss_width,
                    boss_height,
                    boss_radius,
                    0);       
    }
    
    else if (boss_side == 3)
    {
        translate([0,
                  (box_width/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists),
                  (side_height/2)])
        rotate ([270,0,0])
        //draw the boss body
        rounded_box(boss_length,
                    boss_width,
                    boss_height,
                    boss_radius,
                    0);       
    }

    else if (boss_side == 4)
    {
        translate([0,
                  -(box_width/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists),
                  (side_height/2)])
        rotate ([90,0,0])
        //draw the boss body
        rounded_box(boss_length,
                    boss_width,
                    boss_height,
                    boss_radius,
                    0);       
    }
    
    if (boss_side == 5)
    {
        translate([0,0,box_base_thickness])
        rotate ([0,0,0])
        //draw the boss body
        rounded_box(boss_length,
                    boss_width,
                    boss_height,
                    boss_radius,
                    0);       
    }   
}

////////////////////////////////////////////////////////////////////////////
// make cylinderical clamp, normally for cable entry
////////////////////////////////////////////////////////////////////////////

module make_entry_clamp(clamp_side,
                        clamp_length,
                        clamp_width,
                        clamp_height,
                        grip_height,
                        clamp_radius,
                        clamp_cable_size,
                        clamp_screw_hole,
                        clamp_grip_hole)
{
    if (clamp_side == 1)
    {
        //draw the cable clamp body
        translate([(box_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists)
                    +(clamp_length/2),0,0])
        rotate ([0,0,0])
        half_rounded_box(clamp_length,
                         clamp_width,
                         clamp_height,
                         clamp_radius);
            
        //draw the cable clamp grip
        translate([(box_length/2)+(clamp_length*2),0,0])
        rotate ([0,0,0])
        half_rounded_box(clamp_length,
                         clamp_width,
                         grip_height,
                         clamp_radius);
    }
    
    else if (clamp_side == 2)
    {
        //draw the cable clamp body
        translate([-(box_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists)
                  -(clamp_length/2),0,0])
        rotate ([0,0,180])
        half_rounded_box(clamp_length,
                         clamp_width,
                         clamp_height,
                         clamp_radius);
        
        //draw the cable clamp grip
        translate([-(box_length/2)-(clamp_length*2),0,0])
        rotate ([0,0,180])
        half_rounded_box(clamp_length,
                         clamp_width,
                         grip_height,
                         clamp_radius);
    }
    
    else if (clamp_side == 3)
    {
        //draw the cable clamp body
        translate([0,(box_width/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists)
                   +(clamp_length/2),0])
        rotate ([0,0,90])
        half_rounded_box(clamp_length,
                         clamp_width,
                         clamp_height,
                         clamp_radius);
        
        //draw the cable clamp grip
        translate([0,(box_width/2)+(clamp_length*2),0])
        rotate ([0,0,90])
        half_rounded_box(clamp_length,
                         clamp_width,
                         grip_height,
                         clamp_radius);
    }

    else if (clamp_side == 4)
    {        
        //draw the cable clamp body
        translate([0,-(box_width/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists)
                    -(clamp_length/2),0])
        rotate ([0,0,270])
        half_rounded_box(clamp_length,
                         clamp_width,
                         clamp_height,
                         clamp_radius);
        
        //draw the cable clamp grip
        translate([0,-(box_width/2)-(clamp_length*2),0])
        rotate ([0,0,270])
        half_rounded_box(clamp_length,
                         clamp_width,
                         grip_height,
                         clamp_radius);
    }
}
////////////////////////////////////////////////////////////////////////////
// make cylinderical clamp, normally for cable entry
////////////////////////////////////////////////////////////////////////////

module make_clamp_hole(clamp_side,
                       clamp_length,
                       clamp_width,
                       clamp_height,
                       grip_height,
                       clamp_radius,
                       clamp_cable_size,
                       clamp_screw_hole,
                       clamp_grip_hole)
{
    if (clamp_side == 1)
    {
        //remove clamp hole for cable
        #translate([(box_length/2)-box_hollow_wall_thickness-(box_x_pos_recess_radius*box_x_pos_recess_exists)-0.1,0,clamp_height])
        rotate ([0,90,0])
        cylinder( h = clamp_length+box_hollow_wall_thickness+1,d1=clamp_cable_size,d2=clamp_cable_size);
   
        //remove grip hole for cable
        translate([(box_length/2)+(clamp_length*1.5),0,grip_height])
        rotate ([0,90,0])
        cylinder( h = clamp_length,d1=clamp_cable_size,d2=clamp_cable_size);
        
        //remove clamp fixing 1
        translate([(box_length/2)+(clamp_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists),
                 ((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove clamp fixing 2
        translate([(box_length/2)+(clamp_length/2)-(box_x_pos_recess_radius*box_x_pos_recess_exists),
                 -((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove grip fixing 1
        translate([(box_length/2)+(clamp_length*2),
                 ((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
   
        //remove grip fixing 2
        translate([(box_length/2)+(clamp_length*2),
                 -((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
    }
    
    else if (clamp_side == 2)
    {
        //remove the hole for cable
        translate([-(box_length/2)+box_hollow_wall_thickness+(box_x_pos_recess_radius*box_x_pos_recess_exists)+0.1,0,clamp_height])
        rotate ([0,270,0])
        cylinder( h = clamp_length+box_hollow_wall_thickness+1,d1=clamp_cable_size,d2=clamp_cable_size);
        
        //remove grip hole for cable
        translate([-(box_length/2)-(clamp_length*1.5),0,grip_height])
        rotate ([0,270,0])
        cylinder( h = clamp_length,d1=clamp_cable_size,d2=clamp_cable_size);
   
        //remove clamp fixing 1
        translate([-(box_length/2)-(clamp_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists),
                 ((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove clamp fixing 2
        translate([-(box_length/2)-(clamp_length/2)+(box_x_neg_recess_radius*box_x_neg_recess_exists),
                 -((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove grip fixing 1
        translate([-(box_length/2)-(clamp_length*2),
                 ((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
   
        //remove grip fixing 2
        translate([-(box_length/2)-(clamp_length*2),
                 -((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
    }
    
    else if (clamp_side == 3)
    {
        //remove clamp hole for cable
        translate([0,(box_width/2)-(box_hollow_wall_thickness)
                     -(box_y_pos_recess_radius*box_y_pos_recess_exists)-0.1,clamp_height])
        rotate ([270,0,0])
        cylinder( h = clamp_length+box_hollow_wall_thickness+1,d1=clamp_cable_size,d2=clamp_cable_size);

        //remove grip hole for cable
        translate([0,(box_width/2)+(clamp_length*1.5),grip_height])
        rotate ([270,0,0])
        cylinder( h = clamp_length,d1=clamp_cable_size,d2=clamp_cable_size);

        //remove clamp fixing 1
        translate([((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),
                    (box_width/2)+(clamp_length/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove clamp fixing 2
        translate([-((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),
                    (box_width/2)+(clamp_length/2)-(box_y_pos_recess_radius*box_y_pos_recess_exists),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove grip fixing 1
        translate([((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),
                    (box_width/2)+(clamp_length*2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
   
        //remove grip fixing 2
        translate([-((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),
                    (box_width/2)+(clamp_length*2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
    }

    else if (clamp_side == 4)
    {
        //remove the hole for cable
        translate([0,-(box_width/2)+(box_hollow_wall_thickness)
                     +(box_y_neg_recess_radius*box_y_neg_recess_exists)+0.1,clamp_height])
        rotate ([90,0,0])
        cylinder( h = clamp_length+box_hollow_wall_thickness+1,d1=clamp_cable_size,d2=clamp_cable_size);
        
        //remove grip hole for cable
        translate([0,-(box_width/2)-(clamp_length*1.5),grip_height])
        rotate ([90,0,0])
        cylinder( h = clamp_length,d1=clamp_cable_size,d2=clamp_cable_size);

        //remove clamp fixing 1
        translate([((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),
                    -(box_width/2)-(clamp_length/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove clamp fixing 2
        translate([-((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),
                    -(box_width/2)-(clamp_length/2)+(box_y_neg_recess_radius*box_y_neg_recess_exists),1])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_screw_hole,d2=clamp_screw_hole);
   
        //remove grip fixing 1
        translate([((clamp_width-clamp_cable_size)/4)+(clamp_cable_size/2),
                    -(box_width/2)-(clamp_length*2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
   
        //remove grip fixing 2
        translate([-((clamp_width-clamp_cable_size)/4)-(clamp_cable_size/2),
                    -(box_width/2)-(clamp_length*2),0])
        rotate ([0,0,0])
        cylinder( h = clamp_height+0.1,d1=clamp_grip_hole,d2=clamp_grip_hole);
    }
}

////////////////////////////////////////////////////////////////////////////
// x positive face text
////////////////////////////////////////////////////////////////////////////

module box_text_x_pos_face()
{

    produce_text( box_text_x_pos_string,
                  box_text_x_pos_size,
                  box_text_x_pos_font,
                  box_text_x_pos_direction,
                  box_text_x_pos_halign,
                  box_text_x_pos_valign,
                  box_text_x_pos_spacing,
                  box_text_x_pos_no_of_lines,
                  box_text_x_pos_line_spacing,
                  box_text_x_pos_extrude_height,
                  box_text_x_pos_rotation_x,
                  box_text_x_pos_rotation_y,
                  box_text_x_pos_rotation_z,
                  box_text_x_pos_position_x,
                  box_text_x_pos_position_y,
                  box_text_x_pos_position_z);
}

////////////////////////////////////////////////////////////////////////////
// x negitive face text
////////////////////////////////////////////////////////////////////////////

module box_text_x_neg_face()
{

    produce_text( box_text_x_neg_string,
                  box_text_x_neg_size,
                  box_text_x_neg_font,
                  box_text_x_neg_direction,
                  box_text_x_neg_halign,
                  box_text_x_neg_valign,
                  box_text_x_neg_spacing,
                  box_text_x_neg_no_of_lines,
                  box_text_x_neg_line_spacing,
                  box_text_x_neg_extrude_height,
                  box_text_x_neg_rotation_x,
                  box_text_x_neg_rotation_y,
                  box_text_x_neg_rotation_z,
                  box_text_x_neg_position_x,
                  box_text_x_neg_position_y,
                  box_text_x_neg_position_z);
}

////////////////////////////////////////////////////////////////////////////
// y positive face text
////////////////////////////////////////////////////////////////////////////

module box_text_y_pos_face()
{

    produce_text( box_text_y_pos_string,
                  box_text_y_pos_size,
                  box_text_y_pos_font,
                  box_text_y_pos_direction,
                  box_text_y_pos_halign,
                  box_text_y_pos_valign,
                  box_text_y_pos_spacing,
                  box_text_y_pos_no_of_lines,
                  box_text_y_pos_line_spacing,
                  box_text_y_pos_extrude_height,
                  box_text_y_pos_rotation_x,
                  box_text_y_pos_rotation_y,
                  box_text_y_pos_rotation_z,
                  box_text_y_pos_position_x,
                  box_text_y_pos_position_y,
                  box_text_y_pos_position_z);
}

////////////////////////////////////////////////////////////////////////////
// y negitive face text
////////////////////////////////////////////////////////////////////////////

module box_text_y_neg_face()
{

    produce_text( box_text_y_neg_string,
                  box_text_y_neg_size,
                  box_text_y_neg_font,
                  box_text_y_neg_direction,
                  box_text_y_neg_halign,
                  box_text_y_neg_valign,
                  box_text_y_neg_spacing,
                  box_text_y_neg_no_of_lines,
                  box_text_y_neg_line_spacing,
                  box_text_y_neg_extrude_height,
                  box_text_y_neg_rotation_x,
                  box_text_y_neg_rotation_y,
                  box_text_y_neg_rotation_z,
                  box_text_y_neg_position_x,
                  box_text_y_neg_position_y,
                  box_text_y_neg_position_z);
}

////////////////////////////////////////////////////////////////////////////
// lid text
////////////////////////////////////////////////////////////////////////////

module box_text_base_face()
{

    produce_text( box_text_base_string,
                  box_text_base_size,
                  box_text_base_font,
                  box_text_base_direction,
                  box_text_base_halign,
                  box_text_base_valign,
                  box_text_base_spacing,
                  box_text_base_no_of_lines,
                  box_text_base_line_spacing,
                  box_text_base_extrude_height,
                  box_text_base_rotation_x,
                  box_text_base_rotation_y,
                  box_text_base_rotation_z,
                  box_text_base_position_x,
                  box_text_base_position_y,
                  box_text_base_position_z);
}

////////////////////////////////////////////////////////////////////////////
// lid face text
////////////////////////////////////////////////////////////////////////////

module lid_text()
{

    produce_text( lid_text_to_print_1,
                  lid_text_size,
                  lid_text_font,
                  lid_text_direction,
                  lid_text_halign,
                  lid_text_valign,
                  lid_text_spacing,
                  lid_text_no_of_lines,
                  lid_text_line_spacing,
                  lid_text_extrude_height,
                  lid_text_rotation_x,
                  lid_text_rotation_y,
                  lid_text_rotation_z,
                  lid_text_position_x,
                  lid_text_position_y,
                  lid_text_position_z);
}

////////////////////////////////////////////////////////////////////////////
//make some text
////////////////////////////////////////////////////////////////////////////

 module produce_text( string,
                      size_var,
                      font_var,
                      direction_var,
                      halign_var,
                      valign_var,
                      spacing_var,
                      line_num,
                      line_space,
                      extrude_height,
                      text_rot_x,
                      text_rot_y,
                      text_rot_z,
                      text_pos_x,
                      text_pos_y,
                      text_pos_z)
{
    translate ([text_pos_x,text_pos_y,text_pos_z])
    rotate ([text_rot_x,text_rot_y,text_rot_z])
    for (i=[0:line_num-1])
    {
        translate ([0,-line_space*i,0])
        linear_extrude(extrude_height)
        text (string[i],size=size_var,direction=direction_var,font=font_var,
              halign=halign_var,valign=valign_var);
    }
}

////////////////////////////////////////////////////////////////////////////
//GENERATE A HINGE BLOCK
////////////////////////////////////////////////////////////////////////////

 module hinge_block(hinge_type,//0=screw 1=nut
                    hinge_length,
                    hinge_width,
                    hinge_shaft_height,
                    hinge_screw_size,
                    hinge_cbore_length,
                    hinge_joint_position)
{
    if (hinge_type ==0)//box
    {
        translate ([-(hinge_joint_position/2)-(hinge_length/2),0,0])   
        for (i=[0:1])
        {
            translate ([((i*hinge_joint_position)+(i*hinge_length)),0,0]) 
            rotate ([0,0,i*180])   
            basic_hinge_block(hinge_type,//0=screw 1=nut
                              hinge_length,
                              hinge_width,
                              hinge_shaft_height,
                              hinge_screw_size,
                              hinge_cbore_length,
                              hinge_joint_position);
        }
    }
    
    if (hinge_type ==1)//lid
    {
        translate ([hinge_joint_position/2,0,-hinge_width/2])
        rotate ([90,0,0])   
       
        for (i=[0:1])
        {
            
            translate ([-(hinge_joint_position*i)-(i*-hinge_length)-(hinge_length/2),0,0]) 
            rotate ([0,0,i*180])   
            basic_hinge_block(hinge_type,//0=screw 1=nut
                              hinge_length,
                              hinge_width,
                              hinge_shaft_height,
                              hinge_screw_size,
                              hinge_cbore_length,
                              hinge_joint_position);
        }
    }    
}

////////////////////////////////////////////////////////////////////////////
//GENERATE A BASIC HINGE BLOCK
////////////////////////////////////////////////////////////////////////////

 module basic_hinge_block(hinge_type,//0=screw 1=nut
                          hinge_length,
                          hinge_width,
                          hinge_shaft_height,
                          hinge_screw_size,
                          hinge_cbore_length,
                          hinge_joint_position)
{
  
    difference()
    {
        union()
        {
            //draw the hinge block
            //base block
            translate([0,0,hinge_shaft_height/2])
            cube([hinge_length,hinge_width,hinge_shaft_height],center=true);
            //cylinder top
            translate([-hinge_length/2,0,hinge_shaft_height])
            rotate ([0,90,0])
            cylinder( h = hinge_length,d1=hinge_width,d2=hinge_width);     


        }//end of union
            
            //remove centre shaft
            translate([-(hinge_length/2)-0.1,0,hinge_shaft_height])
            rotate ([0,90,0])
            cylinder( h = hinge_length+0.2,
                      d1=ThreadToClearanceHole(hinge_screw_size),
                      d2=ThreadToClearanceHole(hinge_screw_size));     

            //counter bore or nut housing
            if (hinge_type==1) //nut
            {
            translate([-(hinge_length/2)-0.1,0,hinge_shaft_height])
            rotate ([0,90,0])
            cylinder( h = hinge_cbore_length,
                      d1=HexAcrossFlatsToPointsDia(hinge_screw_size),
                      d2=HexAcrossFlatsToPointsDia(hinge_screw_size),$fn=6);    
            }
            else //counterbore
            {
            translate([-(hinge_length/2)-0.1,0,hinge_shaft_height])
            rotate ([0,90,0])
            cylinder( h = hinge_cbore_length+0.2,
                      d1=ThreadToCapHeadDiameter(hinge_screw_size),
                      d2=ThreadToCapHeadDiameter(hinge_screw_size));    
            }

        }//end difference  
}

////////////////////////////////////////////////////////////////////////////
//generate angled hinge for side of box
////////////////////////////////////////////////////////////////////////////

module box_side_hinge()
{
    difference()
    {
        union()
        {
            //add bits
            
            //hinge shape for box
            hull()
            {
                //hinge cylinder
                translate( [0,0,0] )
                rotate ([0,90,0])
                cylinder( r = side_hinge_radius, h = side_hinge_length,center=true);
                //box side plate
                translate( [0,-side_hinge_center_dist,-box_side_hinge_height/2] )
                cube ([side_hinge_length,1,box_side_hinge_height],center=true);
                
            }

        }//end of union
        
            //remove bits
            //hinge pin
            rotate ([0,90,0])
            cylinder( d = side_hinge_pin_dia, h = side_hinge_length+0.2,center=true);
        
            //fingers
            translate ([-side_hinge_finger_width*(side_hinge_box_finger_qty-2),0,0])
            for (i=[0:side_hinge_box_finger_qty-2])
            translate( [i*side_hinge_finger_width*2,0,0] )
            cube ([side_hinge_finger_width+side_hinge_finger_clearance,side_hinge_center_dist*2,
                    (box_side_hinge_height+side_hinge_radius)*2],center=true);
            
            //end cap 1
            translate( [(side_hinge_length/2)-(side_hinge_end_cap_depth/2)+0.1,0,0] )
            rotate ([0,90,0])
            cylinder( d = side_hinge_end_cap_dia, h = side_hinge_end_cap_depth,center=true);
            //end cap 2
            translate( [-(side_hinge_length/2)+(side_hinge_end_cap_depth/2)-0.1,0,0] )
            rotate ([0,90,0])
            cylinder( d = side_hinge_end_cap_dia, h = side_hinge_end_cap_depth,center=true);
            
        }//end difference  
}
    

////////////////////////////////////////////////////////////////////////////
//generate angled hinge for side of lid
////////////////////////////////////////////////////////////////////////////

module lid_side_hinge()
{
    difference()
    {
        union()
        {
            //add bits
            
            //hinge shape for box
            hull()
            {
                //hinge cylinder
                translate( [0,0,0] )
                rotate ([0,90,0])
                cylinder( r = side_hinge_radius, h = side_hinge_length-side_hinge_finger_width*2,center=true);
                //box side plate
                translate( [0,side_hinge_center_dist,-lid_side_hinge_height/2] )
                cube ([side_hinge_length-side_hinge_finger_width*2,1,lid_side_hinge_height],center=true);
                
            }

        }//end of union
        
            //remove bits
            //hinge pin
            rotate ([0,90,0])
            cylinder( d = side_hinge_pin_dia, h = side_hinge_length+0.2,center=true);
            //fingers
            translate ([-side_hinge_finger_width*(side_hinge_box_finger_qty-1),0,0])
            for (i=[0:side_hinge_box_finger_qty-1])
            translate( [i*side_hinge_finger_width*2,0,0] )
            cube ([side_hinge_finger_width,side_hinge_center_dist*2,
                    (lid_side_hinge_height+side_hinge_radius)*2],center=true);
            
        }//end difference  
}







////////////////////////////////////////////////////////////////////////////
//makes a vertical cradle for a pcb to fit into
////////////////////////////////////////////////////////////////////////////

module pcb_cradle_array(cradle_qty,
                        cradle_pitch,
                        slot_length,
                        slot_width,
                        slot_thickness,
                        slot_cradle_length,
                        slot_cradle_width,
                        slot_cradle_thickness,
                        slot_cradle_base_depth,
                        slot_opening_length)
{
    for(i=[0:cradle_qty-1])
    {
        translate ([0,cradle_pitch*i-((cradle_pitch*(cradle_qty-1))/2),0])
        rotate ([0,0,0])
        difference()
        {
            //draw the cradle block in a verticle position
            translate([0,0,slot_cradle_width/2])
            cube([slot_cradle_length,slot_cradle_thickness,slot_cradle_width],center=true);
        
            union()
            {
                //remove the pcb shape from the cradle
                translate([0,0,slot_cradle_base_depth+slot_width/2])
                cube([slot_length,slot_thickness,slot_width],center=true);
            
                //remove the pcb face opening
                translate([0,0,(slot_width/2)+slot_cradle_base_depth])
                cube([slot_opening_length,slot_cradle_thickness,slot_cradle_width-slot_cradle_base_depth],center=true);

            }//end of union

        }//end difference
  
    }//end loop  
}

////////////////////////////////////////////////////////////////////////////
//makes a sliding lid shape
////////////////////////////////////////////////////////////////////////////

module sliding_lid_shape(length,
                        width,
                        height)
{
        translate([0,0,height/2])
        rotate([90,0,0])
        linear_extrude(height = length, center = true)
        polygon( [  [width/2,-height/2],
                    [width/2-height,height/2],
                    [-width/2+height,height/2],
                    [-width/2,-height/2]],
        paths=[[0,1,2,3]] );
}
 
////////////////////////////////////////////////////////////////////////////
//makes a sliding lid
////////////////////////////////////////////////////////////////////////////

module sliding_lid_build()
{
    for(i=[0:sliding_lid_qty-1])
    {
        translate ([sliding_lid_pitch*i-((sliding_lid_pitch*(sliding_lid_qty-1))/2),
                    -((box_width/2)-box_poly_y_far)/2,box_flange_thickness/2])
        difference()
        {
                translate ([0,0,0])
                {
                 //make the lids
                sliding_lid_shape((box_width/2)+box_poly_y_far,
                                    sliding_lid_width-sliding_lid_clearance,
                                    box_flange_thickness);   
 
                //add some positive text
                if ((sliding_lid_text_exists==1)&&(sliding_lid_text_type==0))
                {
                    translate ([0,0,box_flange_thickness])
                    produce_text( sliding_lid_text_to_print[i],
                                  sliding_lid_text_size,
                                  sliding_lid_text_font,
                                  sliding_lid_text_direction,
                                  sliding_lid_text_halign,
                                  sliding_lid_text_valign,
                                  sliding_lid_text_spacing,
                                  sliding_lid_text_no_of_lines,
                                  sliding_lid_text_line_spacing,
                                  sliding_lid_text_extrude_height,
                                  sliding_lid_text_rotation_x,
                                  sliding_lid_text_rotation_y,
                                  sliding_lid_text_rotation_z,
                                  sliding_lid_text_position_x,
                                  sliding_lid_text_position_y,
                                  sliding_lid_text_position_z);
                }
                }
            
            union()
            {
                //remove indent for opening by finger 

                if (sliding_lid_thumbhole_exists == 1)
                {
                    translate ([sliding_lid_thumbhole_x_move,                            sliding_lid_thumbhole_y_move,              
                                sliding_lid_thumbhole_z_move])
                    rotate ([0,0,sliding_lid_thumbhole_rotation])
                    intersection()
                    {
                        translate ([0,0,sliding_lid_thumbhole_sphere_dia/2])
                        sphere (d=sliding_lid_thumbhole_sphere_dia);
                        translate ([sliding_lid_thumbhole_cube_size/2,
                                     0,-sliding_lid_thumbhole_cube_size/2+                                 sliding_lid_thumbhole_cube_height])
                        cube (sliding_lid_thumbhole_cube_size,true);
                    }
                }

                //remove coin slot
                if (lid_coin_slot_exists == 1)
                {
                    translate([lid_coin_slot_x_move,lid_coin_slot_y_move,0])
                    rotate([0,0,lid_coin_slot_rotation])
                    coin_slot(box_flange_thickness);
                }
                
                //put a mesh through the sliding lid
                if (sliding_lid_mesh_exists==1)
                {
                    translate ([0,sliding_lid_mesh_y_pos,box_mesh_base_depth_offset])
                    rotate ([0,0,0])                    
                    make_mesh (sliding_lid_mesh_type, //0=circles,3 up = polygon sides 1&2 invalid
                            sliding_lid_mesh_dia,
                            sliding_lid_mesh_rotation,
                            sliding_lid_mesh_x_qty,
                            sliding_lid_mesh_x_pitch,
                            sliding_lid_mesh_y_qty,
                            sliding_lid_mesh_y_pitch,
                            sliding_lid_mesh_depth,
                            sliding_lid_mesh_offset_req);
                }
                
                //remove negative text
                translate ([0,0,box_flange_thickness-sliding_lid_text_extrude_height])

                if ((sliding_lid_text_exists==1)&&(sliding_lid_text_type==1))
                {
                    produce_text( sliding_lid_text_to_print[i],
                                  sliding_lid_text_size,
                                  sliding_lid_text_font,
                                  sliding_lid_text_direction,
                                  sliding_lid_text_halign,
                                  sliding_lid_text_valign,
                                  sliding_lid_text_spacing,
                                  sliding_lid_text_no_of_lines,
                                  sliding_lid_text_line_spacing,
                                  sliding_lid_text_extrude_height,
                                  sliding_lid_text_rotation_x,
                                  sliding_lid_text_rotation_y,
                                  sliding_lid_text_rotation_z,
                                  sliding_lid_text_position_x,
                                  sliding_lid_text_position_y,
                                  sliding_lid_text_position_z);
                }
                
                
            }//end of union
        }//end difference
    }//end loop  
}

////////////////////////////////////////////////////////////////////////////
//makes a mesh to subtract from the walls
////////////////////////////////////////////////////////////////////////////

module make_mesh (mesh_type, //0=circles,3 up = polygon sides
                  mesh_dia,
                  mesh_rotation,
                  mesh_horz_qty,
                  mesh_horz_pitch,
                  mesh_vert_qty,
                  mesh_vert_pitch,
                  mesh_depth,
                  mesh_offset_req)
{
    odd=0;
    
    for(i=[0:mesh_vert_qty-1])
    {
        //determine if i= odd or even
        odd=i%2;
        echo ("i=",i,"odd",odd);
        
        if ((odd==1)&&(mesh_offset_req))//row is odd number
        {
            translate ([0,mesh_vert_pitch*i-((mesh_vert_pitch*(mesh_vert_qty-1))/2),0])
            rotate ([0,0,0])
            for(i=[0:mesh_horz_qty-2])
            {
                translate ([mesh_horz_pitch*i-((mesh_horz_pitch*(mesh_horz_qty-1))/2)+mesh_horz_pitch/2,0,0])
                rotate ([0,0,mesh_rotation])
                {
                    if ((mesh_type!=1) && (mesh_type!=2))
                    {
                        cylinder( h = mesh_depth,d=mesh_dia,$fn=mesh_type);
                    }
                    else if (mesh_type==1) // draw a polygon
                    {
                        linear_extrude(mesh_depth)  
                        polygon (mesh_poly_points);  
                    }
                    else if (mesh_type==2) // import a 2D shape
                    {
                        linear_extrude(mesh_depth)
                        import(file = dxf_mesh_file);
                    }

                }   
            }
        }
        else //row is even number
        {
            translate ([0,mesh_vert_pitch*i-((mesh_vert_pitch*(mesh_vert_qty-1))/2),0])
            rotate ([0,0,0])
            for(i=[0:mesh_horz_qty-1])
            {
                translate ([mesh_horz_pitch*i-((mesh_horz_pitch*(mesh_horz_qty-1))/2),0,0])
                rotate ([0,0,mesh_rotation])
                {
                    if ((mesh_type!=1) && (mesh_type!=2))
                    {
                        cylinder( h = mesh_depth,d=mesh_dia,$fn=mesh_type);
                    }
                    else if (mesh_type==1) // draw a polygon
                    {
                        linear_extrude(mesh_depth)  
                        polygon (mesh_poly_points);  
                    }
                    else if (mesh_type==2) // import a 2D shape
                    {
                        linear_extrude(mesh_depth)
                        import(file = dxf_mesh_file);
                    }
                }   
            }
        }
    }
}



//////////////////////////////////////////////////////////////////
// makes holes in rectangular, circular or single positions
//////////////////////////////////////////////////////////////////
module holes_from_array ( first_hole,
                          last_hole)
{
            for (i=[first_hole:last_hole])
            {//start index
                if (hole[i][4] == 1) //rectangular array exists
                {
                    translate ([ hole[i][33],hole[i][34],hole[i][35] ])
                    rotate    ([ hole[i][36],hole[i][37],hole[i][38] ])
                    make_rect_array_holes ( hole[i][4],    //rect array exist
                                            hole[i][5],    //rect array x qty
                                            hole[i][6],    //rect array x pitch
                                            hole[i][7],    //rect array y qty
                                            hole[i][8],    //rect array y pitch
                                            hole[i][9],    //rect array outside only   
                                            hole[i][0],    //hole exists
                                            hole[i][1],    //actual or screw size
                                            hole[i][2],    //hole length
                                            hole[i][3],    //hole / thread diameter
                                            hole[i][15],   //recess exists
                                            hole[i][16],   //recess type
                                            hole[i][17],   //counterbore depth
                                            hole[i][18],   //counterbore height offset
                                            hole[i][19],   //nut retainer exists
                                            hole[i][20],   //nut hole depth
                                            hole[i][21],   //nut hole height offset
                                            hole[i][22],   //nut hole rotation
                                            hole[i][23],   //thread exists
                                            hole[i][24],   //thread pitch
                                            hole[i][25],   //thread depth
                                            hole[i][26],   //thread angle
                                            hole[i][27],   //thread height offset
                                            hole[i][28],   //nut slot exists
                                            hole[i][29],   //slot dim x
                                            hole[i][30],   //slot dim y
                                            hole[i][31],   //slot dim z
                                            hole[i][32]); //slot height offset
                    
                }
                
                else if ((hole [i][10] == 1)|| (hole [i][11] == 1))// circular aray exists
                {
                    translate ([ hole[i][33],hole[i][34],hole[i][35] ])
                    rotate    ([ hole[i][36],hole[i][37],hole[i][38] ])
                    make_circ_array_holes(  hole[i][10],   //axial circ array exist
                                            hole[i][11],   //radial circ array exist
                                            hole[i][12],   //circ qty of holes
                                            hole[i][13],   //circ PCD of holes
                                            hole[i][14],   //span of holes
                                            hole[i][0],    //hole exists
                                            hole[i][1],    //actual or screw size
                                            hole[i][2],    //hole length
                                            hole[i][3],    //hole / thread diameter
                                            hole[i][15],   //recess exists
                                            hole[i][16],   //recess type
                                            hole[i][17],   //counterbore depth
                                            hole[i][18],   //counterbore height offset
                                            hole[i][19],   //nut retainer exists
                                            hole[i][20],   //nut hole depth
                                            hole[i][21],   //nut hole height offset
                                            hole[i][22],   //nut hole rotation
                                            hole[i][23],   //thread exists
                                            hole[i][24],   //thread pitch
                                            hole[i][25],   //thread depth
                                            hole[i][26],   //thread angle
                                            hole[i][27],   //thread height offset
                                            hole[i][28],   //nut slot exists
                                            hole[i][29],   //slot dim x
                                            hole[i][30],   //slot dim y
                                            hole[i][31],   //slot dim z
                                            hole[i][32]); //slot height offset
                    }
                
                else // hole is a single entity
                {
                    translate ([ hole[i][33],hole[i][34],hole[i][35] ])
                    rotate    ([ hole[i][36],hole[i][37],hole[i][38] ])
                    make_single_hole( hole[i][0],    //hole exists
                                      hole[i][1],    //actual or screw size
                                      hole[i][2],    //hole length
                                      hole[i][3],    //hole / thread diameter
                                      hole[i][15],   //recess exists
                                      hole[i][16],   //recess type
                                      hole[i][17],   //counterbore depth
                                      hole[i][18],   //counterbore height offset
                                      hole[i][19],   //nut retainer exists
                                      hole[i][20],   //nut hole depth
                                      hole[i][21],   //nut hole height offset
                                      hole[i][22],   //nut hole rotation
                                      hole[i][23],   //thread exists
                                      hole[i][24],   //thread pitch
                                      hole[i][25],   //thread depth
                                      hole[i][26],   //thread angle
                                      hole[i][27],   //thread height offset
                                      hole[i][28],   //nut slots exists
                                      hole[i][29],   //slot dim x
                                      hole[i][30],   //slot dim y
                                      hole[i][31],   //slot dim z
                                      hole[i][32]);  //slot height offset
                                      
                }
                   
                
            }//end index
}

   //////////////////////////////////////////////////////////////////
   // make a rectangular array of holes
   //////////////////////////////////////////////////////////////////
   
    module make_rect_array_holes(rect_exist,    //rect array exist
                                 x_qty,         //rect array x qty
                                 x_pitch,       //rect array x pitch
                                 y_qty,         //rect array y qty
                                 y_pitch,       //rect array y pitch
                                 outer_only,    //rect array outside only
                                 hole_exist,    //hole exists
                                 hole_type,     //actual or screw size
                                 dia,           //thread diameter
                                 height,        //hole depth
                                 recess_exist,  //recess exists
                                 recess_type,   //recess type
                                 cbore_depth,   //counterbore depth
                                 cbore_offset,  //counterbore height offset
                                 nut_exist,     //nut retainer exists
                                 nut_depth,     //nut hole depth
                                 nut_offset,    //nut hole height offset
                                 nut_rotation,  //nut hole rotation
                                 thread_exist,  //thread exists
                                 thread_pitch,  //thread pitch
                                 thread_depth,  //thread depth
                                 thread_angle,  //thread angle
                                 thread_offset, //thread height offset
                                 slot_exist,   //nut slot exists
                                 slot_x_dim,    //slot x size
                                 slot_y_dim,    //slot y size
                                 slot_z_dim,    //slot z size
                                 slot_offset)   //slot offset                                
                                 
                                 
   {
       
    translate ([0,y_pitch/2,0])
    for(y=[0:y_qty-1])
    {
        translate ([0,y_pitch*y-((y_pitch*(y_qty))/2),0])
        rotate ([0,0,0])
        for(x=[0:x_qty-1])
        {
            if (  (outer_only==1)  &&
                     (  
                     ((x==(0))||(x==(x_qty-1)))  ||  ((y==(0))||(y==(y_qty-1)))  
                     ) 
               )
            {
                translate ([x_pitch*x-((x_pitch*(x_qty-1))/2),0,0])
                rotate ([0,0,0])
                make_single_hole(hole_exist,    //hole exists
                                 hole_type,     //actual or screw size
                                 dia,           //thread diameter
                                 height,        //hole depth
                                 recess_exist,   //recess exists
                                 recess_type,    //recess type
                                 cbore_depth,    //counterbore depth
                                 cbore_offset,   //counterbore height offset
                                 nut_exist,      //nut retainer exists
                                 nut_depth,      //nut hole depth
                                 nut_offset,     //nut hole height offset
                                 nut_rotation,   //nut hole rotation
                                 thread_exist,   //thread exists
                                 thread_pitch,   //thread pitch
                                 thread_depth,   //thread depth
                                 thread_angle,   //thread angle
                                 thread_offset, //thread height offset
                                 slot_exist,   //nut slot exists
                                 slot_x_dim,    //slot x size
                                 slot_y_dim,    //slot y size
                                 slot_z_dim,    //slot z size
                                 slot_offset);   //slot offset                                

            }
            
            else if (outer_only==0)
            {
                translate ([x_pitch*x-((x_pitch*(x_qty-1))/2),0,0])
                rotate ([0,0,0])
                make_single_hole(hole_exist,    //hole exists
                                 hole_type,     //actual or screw size
                                 dia,           //thread diameter
                                 height,        //hole depth
                                 recess_exist,   //recess exists
                                 recess_type,    //recess type
                                 cbore_depth,    //counterbore depth
                                 cbore_offset,   //counterbore height offset
                                 nut_exist,      //nut retainer exists
                                 nut_depth,      //nut hole depth
                                 nut_offset,     //nut hole height offset
                                 nut_rotation,  //nut hole rotation
                                 thread_exist,   //thread exists
                                 thread_pitch,   //thread pitch
                                 thread_depth,   //thread depth
                                 thread_angle,   //thread angle
                                 thread_offset, //thread height offset
                                 slot_exist,   //nut slot exists
                                 slot_x_dim,    //slot x size
                                 slot_y_dim,    //slot y size
                                 slot_z_dim,    //slot z size
                                 slot_offset);   //slot offset                                
                    
            }
        }   
    }        
}

    
   
   
   //////////////////////////////////////////////////////////////////
   // make a circular array of holes
   //////////////////////////////////////////////////////////////////
   
    module make_circ_array_holes(holes_axial,   //axial array exist
                                 holes_radial,  //radial array exist
                                 circ_qty,      //circ array qty
                                 pcd,           //circ PCD
                                 span,          //span of circles
                                 hole_exist,    //hole exists
                                 hole_type,     //actual or screw size
                                 dia,           //thread diameter
                                 height,        //hole depth
                                 recess_exist,  //recess exists
                                 recess_type,   //recess type
                                 cbore_depth,   //counterbore depth
                                 cbore_offset,  //counterbore height offset
                                 nut_exist,     //nut retainer exists
                                 nut_depth,     //nut hole depth
                                 nut_offset,    //nut hole height offset
                                 nut_rotation,  //nut hole rotation
                                 thread_exist,  //thread exists
                                 thread_pitch,  //thread pitch
                                 thread_depth,  //thread depth
                                 thread_angle,  //thread angle
                                 thread_offset, //thread height offset
                                 slot_exist,   //nut slot exists
                                 slot_x_dim,    //slot x size
                                 slot_y_dim,    //slot y size
                                 slot_z_dim,    //slot z size
                                 slot_offset)   //slot offset                                
   {

       
    #if (span >=360) // since last hole is same as first hole
    {
        translate ([0,0,0])
        for(i=[0:circ_qty-1])
        {
            //echo (i*(span/circ_qty));
            rotate ([0,90*holes_radial,i*(span/circ_qty)])
            translate ([pcd/2*holes_axial,0,pcd/2*holes_radial])
            make_single_hole(hole_exist,    //hole exists
                             hole_type,     //actual or screw size
                             dia,           //thread diameter
                             height,        //hole depth
                             recess_exist,   //recess exists
                             recess_type,    //recess type
                             cbore_depth,    //counterbore depth
                             cbore_offset,   //counterbore height offset
                             nut_exist,      //nut retainer exists
                             nut_depth,      //nut hole depth
                             nut_offset,     //nut hole height offset
                             nut_rotation,  //nut hole rotation
                             thread_exist,   //thread exists
                             thread_pitch,   //thread pitch
                             thread_depth,   //thread depth
                             thread_angle,   //thread angle
                             thread_offset, //thread height offset
                             slot_exist,   //nut slot exists
                             slot_x_dim,    //slot x size
                             slot_y_dim,    //slot y size
                             slot_z_dim,    //slot z size
                             slot_offset);   //slot offset                                
  
        }
    }
    else    // since last hole not same as first hole
    {
        for(i=[0:circ_qty-1])
        {
            echo (i*(span/(circ_qty-1)));
            rotate ([ 0,90*holes_radial,i*(span/(circ_qty-1))])   
            translate ([pcd/2*holes_axial,0,pcd/2*holes_radial])
            make_single_hole(hole_exist,    //hole exists
                             hole_type,     //actual or screw size
                             dia,           //thread diameter
                             height,        //hole depth
                             recess_exist,   //recess exists
                             recess_type,    //recess type
                             cbore_depth,    //counterbore depth
                             cbore_offset,   //counterbore height offset
                             nut_exist,      //nut retainer exists
                             nut_depth,      //nut hole depth
                             nut_offset,     //nut hole height offset
                             nut_rotation,  //nut hole rotation
                             thread_exist,   //thread exists
                             thread_pitch,   //thread pitch
                             thread_depth,   //thread depth
                             thread_angle,   //thread angle
                             thread_offset, //thread height offset
                             slot_exist,   //nut slot exists
                             slot_x_dim,    //slot x size
                             slot_y_dim,    //slot y size
                             slot_z_dim,    //slot z size
                             slot_offset);   //slot offset                                
  
        }     
   } 
 }  
   
   //////////////////////////////////////////////////////////////////
   // make a single hole as defined
   //////////////////////////////////////////////////////////////////
   
    module make_single_hole( hole_exist,    //hole exists
                             hole_type,     //actual or screw size
                             dia,           //thread diameter
                             height,        //hole depth
                             recess_exist,  //recess exists
                             recess_type,   //recess type
                             cbore_depth,   //counterbore depth
                             cbore_offset,  //counterbore height offset
                             nut_exist,     //nut retainer exists
                             nut_depth,     //nut hole depth
                             nut_offset,    //nut hole height offset
                             nut_rotation,  //nut hole rotation
                             thread_exist,  //thread exists
                             thread_pitch,  //thread pitch
                             thread_depth,  //thread depth
                             thread_angle,  //thread angle
                             thread_offset, //thread height offset
                             slot_exist,   //nut slot exists
                             slot_x_dim,    //slot x size
                             slot_y_dim,    //slot y size
                             slot_z_dim,    //slot z size
                             slot_offset)   //slot offset                                
                             
   {
    if (hole_exist == 1)
    {
        if (hole_type == 1) //1=clearance
        {    
            cylinder( d = ThreadToClearanceHole(dia), h = height + 0.01);
        }
        else // 0=actual size
        {
            cylinder( d = dia, h = height + 0.01);
        }
    }
       
    if ( recess_exist==1)
    {
        if (recess_type == 1) //counter bore
        {
            translate ([0,0,height - cbore_depth + cbore_offset ])
            cylinder( d = ThreadToCapHeadDiameter(dia),h = cbore_depth + 0.01);
        }
        else //countersink
        {
            cylinder((dia/2)*tan(45),
                      d1=dia*2,
                      d2=dia);   
        }
    }
    
    if ( nut_exist==1)
    {
        translate ([0,0,nut_offset-0.1 ])
        rotate ([0,0,nut_rotation])
        cylinder( d = HexAcrossFlatsToPointsDia(dia),h = nut_depth + 0.01,$fn=6);        
        
    }
    
    if ( thread_exist==1)
    {
        translate ([0,0,thread_offset ])
        // ScrewHole(outer_diam, height, position=[0,0,0], rotation=[0,0,0], pitch=0, tooth_angle=30, tolerance=0.4, tooth_height=0)
        ScrewHole(dia, thread_depth, [0, 0, 0], [0,0,0], thread_pitch);   
    }
    
    if ( slot_exist==1)
    {
        translate ([0,0,slot_z_dim/2+slot_offset-0.1 ])
        rotate ([0,0,0])
        cube([slot_x_dim,slot_y_dim,slot_z_dim],center=true);        
        
    }
       
}

////////////////////////////////////////////////////////////////////////////
//produce a coin slot shape
////////////////////////////////////////////////////////////////////////////
module coin_slot(depth)
{
  hull()
  {
   cube([lid_coin_slot_length,lid_coin_slot_width,0.1],true);
   translate ([0,0,depth])   
   cube([lid_coin_slot_length+lid_coin_slot_surround*2,lid_coin_slot_width+lid_coin_slot_surround*2,lid_coin_slot_lip],true);

  }  
}

////////////////////////////////////////////////////////////////////////////
//function to return across flats dimention for metric bolts
// Provides standard metric hex head widths across the flats from thread size.

function ThreadToHexHeadAcrossFlats(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 4],
      [2.5, 5],
      [3, 5.5],
      [3.5, 6],
      [4, 7],
      [5, 8],
      [6, 10],
      [7, 11],
      [8, 13],
      [10, 16],
      [12, 18],
      [14, 21],
      [16, 24],
      [18, 27],
      [20, 30],
      [22, 34],
      [24, 36],
      [27, 41],
      [30, 46],
      [33, 50],
      [36, 55],
      [39, 60],
      [42, 65],
      [48, 75],
      [52, 80],
      [56, 85],
      [60, 90],
      [64, 95]
        ]) : return;
 

////////////////////////////////////////////////////////////////////////////
//function to return across points dimention for metric bolts
// Provides standard metric hex head widths across points from across flats size.


function HexAcrossFlatsToPointsDia(diameter) =
  ThreadToHexHeadAcrossFlats(diameter) / cos(30);

 
////////////////////////////////////////////////////////////////////////////
//function to return across flats dimention for metric Allen head sockets
// Provides standard metric hex Allen head widths across the flats from thread size.
 
function ThreadToAllenHeadAcrossFlats(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 1.5],
      [2.5, 2],
      [3, 2.5],
      [3.5, 3],
      [4, 3],
      [5, 4],
      [6, 5],
      [7, 5],
      [8, 6],
      [10, 8],
      [12, 10],
      [14, 12],
      [16, 14],
      [18, 15],
      [20, 17],
      [22, 18],
      [24, 19],
      [27, 20],
      [30, 22],
      [33, 24],
      [36, 27],
      [39, 30],
      [42, 32],
      [48, 36],
      [52, 36],
      [56, 41],
      [60, 42],
      [64, 46]
        ]) : return;
  
 

////////////////////////////////////////////////////////////////////////////
//function to return across points dimention for metric Allen head sockets
// Provides standard metric hex Allen head diameters across points from across flats size.


function AllenAcrosFlatsToPointsDia(diameter) =
  ThreadToAllenHeadAcrossFlats(diameter) / cos(30);
  
  
////////////////////////////////////////////////////////////////////////////
//function to return standard metric nut thickness from thread size

function NutThickness(height) =
  (height <= 64) ?
    lookup(height, [
      [2, 1.6],
      [2.5, 2],
      [3, 2.4],
      [3.5, 2.8],
      [4, 3.2],
      [5, 4.7],
      [6, 5.2],
      [7, 6.0],
      [8, 6.8],
      [10, 8.4],
      [12, 10.8],
      [14, 12.8],
      [16, 14.8],
      [18, 15.8],
      [20, 18.0],
      [22, 21.1],
      [24, 21.5],
      [27, 23.8],
      [30, 25.6],
      [33, 28.7],
      [36, 31.0],
      [42, 34],
      [48, 38],
      [56, 45],
      [64, 51]
        ]) : return;
  
 ////////////////////////////////////////////////////////////////////////////
//function to return clearance hole size for a metric thread

 
function ThreadToClearanceHole(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.05],
      [2.5, 2.6],
      [3, 3.1],
      [3.5, 3.6],
      [4, 4.1],
      [5, 5.1],
      [6, 6.1],
      [7, 7.2],
      [8, 8.2],
      [10, 10.2],
      [12, 12.2],
      [14, 14.25],
      [16, 16.25],
      [18, 18.25],
      [20, 20.25],
      [22, 22.25],
      [24, 24.25],
      [27, 27.25],
      [30, 30.5],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 64.75]
        ]) : return;
        
        
////////////////////////////////////////////////////////////////////////////
//function to return cap screw head diameter clearance hole size

 
function ThreadToCapHeadDiameter(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 3.5],
      [2.5, 2.6],
      [3, 5.5],
      [3.5, 3.6],
      [4, 7.0],
      [5, 8.5],
      [6, 10.0],
      [7, 11.5],
      [8, 13.0],
      [10, 16.0],
      [12, 18.0],
      [14, 21.0],
      [16, 24.0],
      [18, 27.0],
      [20, 30.0],
      [22, 33.0],
      [24, 36.0],
      [27, 40.0],
      [30, 45.0],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 60.75]
        ]) : return;
  
 ////////////////////////////////////////////////////////////////////////////
//function to return cap screw head diameter clearance hole size

 
function ThreadToCapHeadDepth(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.0],
      [2.5, 2.5],
      [3, 3.0],
      [3.5, 3.5],
      [4, 4.0],
      [5, 5.0],
      [6, 6.0],
      [7, 7.0],
      [8, 8.0],
      [10, 10.0],
      [12, 12.0],
      [14, 14.0],
      [16, 16.0],
      [18, 18.0],
      [20, 20.0],
      [22, 22.0],
      [24, 24.0],
      [27, 27.0],
      [30, 30.0],
      [33, 33.0],
      [36, 36.0],
      [39, 39.0],
      [42, 42.0],
      [45, 45.0],
      [48, 48.0],
      [52, 52.0],
      [56, 56.0],
      [60, 60.0],
      [64, 60.0]
        ]) : return;
  
 
////////////////////////////////////////////////////////////////////////////
//END MODULE SECTION 
////////////////////////////////////////////////////////////////////////////




